@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -20,11 +20,13 @@
 	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	02111-1307, USA.  */
 
-
 #include "ne.h"
 #include "version.h"
 #include <limits.h>
 
+/* ne's temporary file name template for the THROUGH command. */
+
+#define NE_TMP "netmp.XXXXXX"
 
 /* Turns an unspecified integer argument (-1) to 1. This
 is what most commands require. */
@@ -68,9 +70,16 @@ function was aborted or not-a-number error if an invalid number was read. */
 
 
 int do_action(buffer *b, action a, int c, unsigned char *p) {
-	int i, error = 0;
+	
+	line_desc *next_ld;
+	HIGHLIGHT_STATE next_line_state;
+	int i, error = OK, col;
 	char *q;
 
+	assert_buffer(b);
+	assert_buffer_content(b);
+	assert(b->encoding != ENC_UTF8 || b->cur_pos >= b->cur_line_desc->line_len || utf8len(b->cur_line_desc->line[b->cur_pos]) > 0);
+
 	stop = FALSE;
 
 	if (b->recording) record_action(b->cur_macro, a, c, p, b->opt.verbose_macros);
@@ -111,16 +120,19 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return stop ? STOPPED : error;
 
 	case LINEDOWN_A:
+	
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = line_down(b)) && !stop; i++);
 		return stop ? STOPPED : error;
 
 	case PREVPAGE_A:
+	
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = prev_page(b)) && !stop; i++);
 		return stop ? STOPPED : error;
 
 	case NEXTPAGE_A:
+		
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = next_page(b)) && !stop; i++);
 		return stop ? STOPPED : error;
@@ -239,7 +251,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			break;
 		case GOTOBOOKMARK_A:
 			if (! (b->bookmark_mask & (1 << c))) return BOOKMARK_NOT_SET;
-			{
+			else {
 				const int prev_line = b->cur_line;
 				const int prev_pos  = b->cur_pos;
 				delay_update();
@@ -251,13 +263,14 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			}
 		}
 		return OK;
-
+	
 	case GOTOLINE_A:
 		if (c < 0 && (c = request_number("Line", b->cur_line + 1))<0) return NUMERIC_ERROR(c);
 		if (c == 0 || c > b->num_lines) c = b->num_lines;
 		goto_line(b, --c);
 		return OK;
-
+	
+	
 	case GOTOCOLUMN_A:
 		if (c < 0 && (c = request_number("Column", b->cur_x + b->win_x + 1))<0) return NUMERIC_ERROR(c);
 		goto_column(b, c ? --c : 0);
@@ -289,8 +302,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	}
 		
 	case INSERTCHAR_A: {
+
 		static int last_inserted_char = ' ';
-		int old_char;
+		int deleted_char, old_char, error = ERROR;
 
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
 		
@@ -307,163 +321,238 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		
 		old_char = b->cur_pos < b->cur_line_desc->line_len ? get_char(&b->cur_line_desc->line[b->cur_pos], b->encoding) : 0;
 
+		/* Freeze the line attributes before any real update. */
+		if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
+			
 		start_undo_chain(b);
 
-		if (!(b->opt.insert) && b->cur_pos < b->cur_line_desc->line_len)
-			delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-
-		if (b->cur_pos > b->cur_line_desc->line_len)
+		if (deleted_char = !b->opt.insert && b->cur_pos < b->cur_line_desc->line_len) delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
+		if (b->cur_pos > b->cur_line_desc->line_len) {
+			/* We insert spaces to reach the insertion position. */
 			insert_spaces(b, b->cur_line_desc, b->cur_line, b->cur_line_desc->line_len, b->cur_pos - b->cur_line_desc->line_len);
-
+			if (b->syn) update_line(b, b->cur_y, TRUE);
+		}
+		
 		insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, c);
-
-		end_undo_chain(b);
-
-		if (b->opt.insert) update_inserted_char(b, c, b->cur_line_desc, b->cur_pos, b->cur_y, b->cur_x);
-		else update_overwritten_char(b, old_char, c, b->cur_line_desc, b->cur_pos, b->cur_y, b->cur_x);
+		
+		end_undo_chain(b); 
+		need_attr_update = TRUE;
+		
+		/* At this point the line has been modified: note that if we are in overwrite mode and write a character
+			at or beyond the length of the current line, we are actually doing an insertion. */
+		
+		if (!deleted_char) update_inserted_char(b, c, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);
+		else update_overwritten_char(b, old_char, c, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);
 
 		char_right(b);
 
 		/* Note the use of ne_columns-1. This avoids a double horizontal scrolling each time a
 			word wrap happens with b->opt.right_margin = 0. */
 
-		if (b->opt.word_wrap && b->win_x + b->cur_x >= (b->opt.right_margin ? b->opt.right_margin : ne_columns - 1)) {
-			if ((i = word_wrap(b)) != ERROR) {
+		if (b->opt.word_wrap && b->win_x + b->cur_x >= (b->opt.right_margin ? b->opt.right_margin : ne_columns - 1)) error = word_wrap(b);
 
-				int j = 0;
 
-				/* If b->win_x is nonzero, the move_to_sol() call will refresh
-					the entire video, so we shouldn't scroll. */
+		if (error == ERROR) {
+			assert_buffer_content(b);
+			/* No word wrap. */
+			if (b->syn) update_line(b, b->cur_y, FALSE);
+			assert_buffer_content(b);
+		}		
+		else {		
+			/* Fixes in case of word wrapping. */
+			int wont_scroll = b->win_x == 0;
+			int a = 0;
+			if (b->syn) update_line(b, b->cur_y, TRUE);
+			else update_partial_line(b, b->cur_y, calc_width(b->cur_line_desc, b->cur_line_desc->line_len, b->opt.tab_size, b->encoding) - b->win_x, FALSE, FALSE);	
 
-				if (b->win_x) {
-					move_to_sol(b);
-					line_down(b);
-				}
-				else {
-					update_line(b, b->cur_y);
-					move_to_sol(b);
-					line_down(b);
-					if (b->cur_y < ne_lines - 1) scroll_window(b, b->cur_y, 1);
-				}
-
-				if (b->opt.auto_indent && (j = auto_indent_line(b))) update_line(b, b->cur_y);
-				goto_pos(b, i + j);
+			need_attr_update = FALSE;
+			/* Poke the correct state into the next line. */
+			if (b->syn) ((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
 
+			if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next);
+			move_to_sol(b);
+			line_down(b);
+			goto_pos(b, error + a);
+                                                               
+			if (wont_scroll) {
+				if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE);
+				else scroll_window(b, b->cur_y, 1);
 			}
+
+			need_attr_update = TRUE;
+			assert_buffer_content(b);
 		}
+
+		assert_buffer_content(b);
 		return OK;
 	}
 
 
 	case BACKSPACE_A:
 	case DELETECHAR_A:
-
+		
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
 
 		NORMALIZE(c);
-
+		
 		for(i = 0; i < c && !stop; i++) {
 			if (a == BACKSPACE_A) {
 				if (b->win_x + b->cur_x == 0 && b->cur_line == 0) return ERROR;
-				if (b->cur_pos > b->cur_line_desc->line_len && b->cur_line_desc->line_len > 0) {
-					char_left(b);
-					return OK;
-				}
-				char_left(b);
+				else char_left(b);
+			}	
+			/* TODO: Here an additional condition line_len > 0 was present. Is it useful? It's just for deleting outside of lines. */
+			if (b->cur_pos > b->cur_line_desc->line_len) {
+				/* We are not over text. */
+				continue;
 			}
 
+			if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
+
 			if (b->cur_pos < b->cur_line_desc->line_len) {
+				/* Deletion inside a line. */
 				const int old_char = b->encoding == ENC_UTF8 ? utf8char(&b->cur_line_desc->line[b->cur_pos]) : b->cur_line_desc->line[b->cur_pos];
-
 				delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-
-				update_deleted_char(b, old_char, b->cur_line_desc, b->cur_pos, b->cur_y, b->cur_x);
+				
+				update_deleted_char(b, old_char, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);	
+				if (b->syn) update_line(b, b->cur_y, TRUE);
 			}
-			else if (b->cur_pos == b->cur_line_desc->line_len) {
-
+			else {
+				/* Here we handle the case in which two lines are joined. Note that if the first line is empty,
+					it is just deleted by delete_one_char(), so we must store its initial state and restore
+					it after the deletion. */
+				if (b->syn && b->cur_pos == 0) next_line_state = b->cur_line_desc->highlight_state;
 				delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-				update_partial_line(b, b->cur_y, b->cur_x, TRUE);
-
+				if (b->syn && b->cur_pos == 0) b->cur_line_desc->highlight_state = next_line_state;
+				
+				if (b->syn) {
+					b->next_state = parse(b->syn, b->cur_line_desc, b->cur_line_desc->highlight_state, b->encoding == ENC_UTF8); 
+					update_line(b, b->cur_y, TRUE);
+				}	
+				else update_partial_line(b, b->cur_y, b->cur_x, TRUE, FALSE);
+					
 				if (b->cur_y < ne_lines - 2) scroll_window(b, b->cur_y + 1, -1);
-			}
+			}			
 		}
+		need_attr_update = TRUE;
 		return stop ? STOPPED : 0;
 
 	case INSERTLINE_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
 
 		for(i = 0; i < c && !stop; i++) {
-			if (insert_one_line(b, b->cur_line_desc, b->cur_line, b->cur_pos > b->cur_line_desc->line_len ? b->cur_line_desc->line_len : b->cur_pos) == OK) {
-
-				/* If b->win_x is nonzero, the move_to_sol() call will refresh
-					the entire video, so we shouldn't scroll. */
+			if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
 
+			if (insert_one_line(b, b->cur_line_desc, b->cur_line, b->cur_pos > b->cur_line_desc->line_len ? b->cur_line_desc->line_len : b->cur_pos) == OK) {		
+				/* TODO: check that update agress with word wrapping in INSERTCHAR. */
 				if (b->win_x) {
+					/* If b->win_x is nonzero, the move_to_sol() call will
+					refresh the entire video, so we shouldn't do anything. However, we 
+					must poke into the next line initial state the correct state. */
+					if (b->syn) {
+						freeze_attributes(b, b->cur_line_desc);
+						((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
+					}
 					move_to_sol(b);
 					line_down(b);
 				}
 				else {
-					update_partial_line(b, b->cur_y, b->cur_x, FALSE);
+					int a = -1;
+					if (b->syn) update_line(b, b->cur_y, TRUE);
+					else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
+					/* We need to avoid updates until we fix the next line. */
+					need_attr_update = FALSE;
+					/* We poke into the next line initial state the correct state. */
+					if (b->syn) ((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
+
+					assert(b->cur_line_desc->ld_node.next->next != NULL);
+					if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next);
+
 					move_to_sol(b);
+					line_down(b);
+					if (a != -1) goto_pos(b, a);					
 
-					if (b->cur_y < ne_lines - 2) {
-						/* This call will cause no scroll: thus, afterwards we must update the line (if it
-							is the last in the buffer) or scroll by ourselves. */
-						line_down(b);
-						if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y);
-						else scroll_window(b, b->cur_y, 1);
-					}
-					else line_down(b); /* In this case, we cause a scroll, and everything is updated automatically. */
+					if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE);
+					else scroll_window(b, b->cur_y, 1);
+
+					need_attr_update = TRUE;
 				}
-
-				if (b->opt.auto_indent && (i = auto_indent_line(b))) {
-					update_line(b, b->cur_y);
-					goto_pos(b, i);
-				}
-
 			}
 		}
+		
 		return stop ? STOPPED : 0;
 
 	case DELETELINE_A:
-
+			
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
+		
+		col = b->win_x + b->cur_x;
+		start_undo_chain(b);
 		for(i = 0; i < c && !stop; i++) {
-			const int curr_col = b->win_x + b->cur_x;
-			delete_one_line(b, b->cur_line_desc, b->cur_line);
-			move_to_sol(b);
-			goto_column( b, curr_col );
+			if (error = delete_one_line(b, b->cur_line_desc, b->cur_line)) break;
 			scroll_window(b, b->cur_y, -1);
 		}
-		return stop ? STOPPED : 0;
+		end_undo_chain(b);
+		if (b->syn) {
+			update_line(b, b->cur_y, FALSE);
+			need_attr_update = TRUE;
+		}
+		resync_pos(b);
+		goto_column(b, col);
+		
+		return stop ? STOPPED : error;
 
 	case UNDELLINE_A:
-
+		
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
 
 		NORMALIZE(c);
+		
+		next_ld = (line_desc *)b->cur_line_desc->ld_node.next;
 
+		start_undo_chain(b);
 		for(i = 0; i < c && !stop; i++) {
-			if (undelete_line(b) == OK) {
-				update_partial_line(b, b->cur_y, b->cur_x, FALSE);
-				if (b->cur_y < ne_lines - 2) scroll_window(b, b->cur_y + 1, 1);
+			/* This is a bit tricky. First of all, if we are undeleting for
+				the first time and the local attribute buffer is not valid
+				we fill it. */
+			if (i == 0 && b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
+			if (error = undelete_line(b)) break;
+			if (i == 0) {
+				if (b->syn) {
+					/* Now the only valid part of the local attribute buffer is before b->cur_pos. 
+						We perform a differential update so that if we undelete in the middle of
+						a line we avoid to rewrite the part up to b->cur_pos. */
+					b->attr_len = b->cur_pos;
+					update_line(b, b->cur_y, TRUE);
+					next_line_state = b->next_state;
+				}
+				else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
 			}
-			else return ERROR;
+			if (b->syn) {
+				assert(b->cur_line_desc->ld_node.next->next != NULL);
+				/* For each undeletion, we must poke into the next line its correct initial state. */
+				((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = next_line_state;
+			}
+			/* We actually scroll down the remaining lines, if necessary. */
+			if (b->cur_y < ne_lines - 2) scroll_window(b, b->cur_y + 1, 1);
 		}
-		return stop ? STOPPED : 0;
+		if (b->syn) {
+			/* Finally, we force the update of the initial states of all following lines up to next_ld. */
+			need_attr_update = TRUE;
+			update_syntax_states(b, b->cur_y, b->cur_line_desc, next_ld);
+		}
+		end_undo_chain(b);
+		return stop ? STOPPED : error;
 
 	case DELETEEOL_A:
 
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
+		if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
 		delete_to_eol(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-		update_partial_line(b, b->cur_y, b->cur_x, FALSE);
+		if (b->syn) update_line(b, b->cur_y, TRUE);
+		else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
+		need_attr_update = TRUE;
 
 		return OK;
 
@@ -506,6 +595,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			buffer *dup = get_buffer_named(p);
 
 			if (!dup || dup == b || request_response(b, "There is another document with the same name; are you sure?", FALSE)) {
+				b->syn = NULL; /* So that autoprefs will load the right syntax. */
 				if (b->opt.auto_prefs && extension(p)) load_auto_prefs(b, extension(p));
 				error = load_file_in_buffer(b, p);
 				if (error != FILE_IS_MIGRATED && error != FILE_IS_DIRECTORY) change_filename(b, p);
@@ -609,7 +699,12 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 						if (b->last_was_regexp) error = replace_regexp(b, p);
 						else error = replace(b, strlen(b->find_string), p);
-						update_line(b, b->cur_y);
+						update_line(b, b->cur_y, FALSE);
+						if (b->syn) {
+							need_attr_update = TRUE;
+							update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+						}
+						
 						if (print_error(error)) {
 							if (a == REPLACEALL_A || c == 'A') end_undo_chain(b);
 							return ERROR;
@@ -664,7 +759,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 					if (b->last_was_replace) {
 						if (b->last_was_regexp) error = replace_regexp(b, b->replace_string);
 						else error = replace(b, strlen(b->find_string), b->replace_string);
-						update_line(b, b->cur_y);
+						update_line(b, b->cur_y, FALSE);
 						if (print_error(error)) {
 							return_code = ERROR;
 							break;
@@ -684,6 +779,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case MATCHBRACKET_A:
 		return print_error(match_bracket(b)) ? ERROR : 0;
 
+	case ALERT_A:
+		alert();
+		return OK;
+
 	case BEEP_A:
 		ring_bell();
 		return OK;
@@ -737,6 +836,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		SET_USER_FLAG(b, c, is_CRLF);
 		return OK;
 
+	case VISUALBELL_A:
+		SET_USER_FLAG(b, c, opt.visual_bell);
+		return OK;
+
 	case STATUSBAR_A:
 		SET_USER_FLAG(b, c, opt.status_bar);
 		reset_status_bar();
@@ -792,9 +895,14 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			else return BUFFER_IS_NOT_UTF8;
 		}
 		else b->encoding = encoding == ENC_ASCII ? ENC_ASCII : ENC_8_BIT;
+		if (old_encoding != b->encoding) {
+			reset_syntax_states(b);
+			reset_undo_buffer(&b->undo);
+		}
+		b->attr_len = -1;
+		need_attr_update = FALSE;
 		move_to_sol(b);
 		reset_window();
-		if (old_encoding != b->encoding) reset_undo_buffer(&b->undo);
 		return OK;
 	}
 
@@ -850,7 +958,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case SAVEMACRO_A:
 		if (p || (p = request_file(b, "Macro Name", NULL))) {
 			print_info(SAVING);
-			optimize_macro( b->cur_macro, b->opt.verbose_macros );
+			optimize_macro(b->cur_macro, b->opt.verbose_macros);
 			if ((error = print_error(save_stream(b->cur_macro, p, b->is_CRLF, FALSE))) == OK) print_info(SAVED);
 			free(p);
 			return error ? ERROR : 0;
@@ -891,7 +999,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			unset_interactive_mode();
 			exit(0);
 		}
-		keep_cursor_on_screen( cur_buffer );
+		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 
 		/* We always return ERROR after a buffer has been deleted. Otherwise,
@@ -902,22 +1010,28 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case NEXTDOC_A: /* Was NEXT_BUFFER: */
 		if (b->b_node.next->next) cur_buffer = (buffer *)b->b_node.next;
 		else cur_buffer = (buffer *)buffers.head;
-		keep_cursor_on_screen( cur_buffer );
+		keep_cursor_on_screen(cur_buffer);
 		reset_window();
+		need_attr_update = FALSE;
+		b->attr_len = -1;
 		return OK;
 
 	case PREVDOC_A:
 		if (b->b_node.prev->prev) cur_buffer = (buffer *)b->b_node.prev;
 		else cur_buffer = (buffer *)buffers.tail_pred;
-		keep_cursor_on_screen( cur_buffer );
+		keep_cursor_on_screen(cur_buffer);
 		reset_window();
+		need_attr_update = FALSE;
+		b->attr_len = -1;
 		return OK;
 
 	case SELECTDOC_A:
 		if ((i = request_document()) < 0 || !(b = get_nth_buffer(i))) return ERROR;
 		cur_buffer = b;
-		keep_cursor_on_screen( cur_buffer );
+		keep_cursor_on_screen(cur_buffer);
 		reset_window();
+		need_attr_update = FALSE;
+		b->attr_len = -1;
 		return OK;
 
 	case MARK_A:
@@ -931,37 +1045,28 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case CUT_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
 
 	case COPY_A:
-
-		delay_update();
-
 		if (!(error = print_error((b->mark_is_vertical ? copy_vert_to_clip : copy_to_clip)(b, c < 0 ? b->opt.cur_clip : c, a == CUT_A)))) {
 			b->marking = 0;
-			if (a == CUT_A) update_window(b);
+			update_window_lines(b, b->cur_y, ne_lines - 2, FALSE);
 		}
 		return error ? ERROR : 0;
 
 	case ERASE_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
-		delay_update();
-
 		if (!(error = print_error((b->mark_is_vertical ? erase_vert_block : erase_block)(b)))) {
 			b->marking = 0;
-			update_window(b);
+			update_window_lines(b, b->cur_y, ne_lines - 2, FALSE);
 		}
 		return OK;
 
 	case PASTE_A:
 	case PASTEVERT_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-		delay_update();
-		if (!(error = print_error((a == PASTE_A ? paste_to_buffer : paste_vert_to_buffer)(b, c < 0 ? b->opt.cur_clip : c)))) update_window(b);
+		if (!(error = print_error((a == PASTE_A ? paste_to_buffer : paste_vert_to_buffer)(b, c < 0 ? b->opt.cur_clip : c)))) update_window_lines(b, b->cur_y, ne_lines - 2, FALSE);
+		assert_buffer_content(b);
 		return error ? ERROR : 0;
 
 	case GOTOMARK_A:
@@ -1021,48 +1126,55 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (!b->marking) b->mark_is_vertical = 0;
 
 		if (p || (p = request_string("Filter", NULL, FALSE, TRUE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
+			int fin = -1, fout = -1;
 
-			char tmpnam1[L_tmpnam], tmpnam2[L_tmpnam], *command;
+			char tmpnam1[strlen(P_tmpdir)+strlen(NE_TMP)+2], tmpnam2[strlen(P_tmpdir)+strlen(NE_TMP)+2], *command;
 
-			tmpnam(tmpnam1);
-			tmpnam(tmpnam2);
+			strcat(strcat(strcpy(tmpnam1, P_tmpdir), "/"), NE_TMP);
+			strcat(strcat(strcpy(tmpnam2, P_tmpdir), "/"), NE_TMP);
+			if ((fin = mkstemp(tmpnam1)) != -1) close(fin);
+			if ((fout = mkstemp(tmpnam2)) != -1) close(fout);
+			if (fin != -1 && fout != -1) {
 
-			realloc_clip_desc(get_nth_clip(INT_MAX), INT_MAX, 0);
+				realloc_clip_desc(get_nth_clip(INT_MAX), INT_MAX, 0);
 
-			if (!b->marking || !(error = (b->mark_is_vertical ? copy_vert_to_clip : copy_to_clip)(b, INT_MAX, FALSE))) {
+				if (!b->marking || !(error = (b->mark_is_vertical ? copy_vert_to_clip : copy_to_clip)(b, INT_MAX, FALSE))) {
 
-				if (!(error = save_clip(INT_MAX, tmpnam1, b->is_CRLF, b->opt.binary))) {
-					if (command = malloc(strlen(p) + strlen(tmpnam1) + strlen(tmpnam2) + 16)) {
+					if (!(error = save_clip(INT_MAX, tmpnam1, b->is_CRLF, b->opt.binary))) {
+						if (command = malloc(strlen(p) + strlen(tmpnam1) + strlen(tmpnam2) + 16)) {
 
-						strcat(strcat(strcat(strcat(strcpy(command, p), " <"), tmpnam1), " >"), tmpnam2);
+							strcat(strcat(strcat(strcat(strcat(strcpy(command, "( "), p), " ) <"), tmpnam1), " >"), tmpnam2);
 
-						unset_interactive_mode();
-						if (system(command)) error = EXTERNAL_COMMAND_ERROR;
-						set_interactive_mode();
+							unset_interactive_mode();
+							if (system(command)) error = EXTERNAL_COMMAND_ERROR;
+							set_interactive_mode();
 							
-						if (!error) {
+							if (!error) {
 
-							if (!(error = load_clip(INT_MAX, tmpnam2, b->opt.preserve_cr, b->opt.binary))) {
+								if (!(error = load_clip(INT_MAX, tmpnam2, b->opt.preserve_cr, b->opt.binary))) {
 
-								start_undo_chain(b);
+									start_undo_chain(b);
 
-								if (b->marking) (b->mark_is_vertical ? erase_vert_block : erase_block)(b);
-								error = (b->mark_is_vertical ? paste_vert_to_buffer : paste_to_buffer)(b, INT_MAX);
+									if (b->marking) (b->mark_is_vertical ? erase_vert_block : erase_block)(b);
+									error = (b->mark_is_vertical ? paste_vert_to_buffer : paste_to_buffer)(b, INT_MAX);
 
-								end_undo_chain(b);
+									end_undo_chain(b);
 
-								b->marking = 0;
+									b->marking = 0;
 
-								realloc_clip_desc(get_nth_clip(INT_MAX), INT_MAX, 0);
+									realloc_clip_desc(get_nth_clip(INT_MAX), INT_MAX, 0);
+								}
 							}
+							free(command);
 						}
-						free(command);
+						else error = OUT_OF_MEMORY;
 					}
-					else error = OUT_OF_MEMORY;
 				}
-				remove(tmpnam1);
-				remove(tmpnam2);
 			}
+			else error = CANT_OPEN_TEMPORARY_FILE;
+			
+			remove(tmpnam1);
+			remove(tmpnam2);
 
 			ttysize();
 			keep_cursor_on_screen(cur_buffer);
@@ -1077,7 +1189,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 		NORMALIZE(c);
 
+		start_undo_chain(b);
 		for(i = 0; i < c && !(error = to_upper(b)) && !stop; i++);
+		end_undo_chain(b);
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
 
@@ -1087,7 +1201,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 		NORMALIZE(c);
 
+		start_undo_chain(b);
 		for(i = 0; i < c && !(error = to_lower(b)) && !stop; i++);
+		end_undo_chain(b);
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
 
@@ -1097,7 +1213,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 		NORMALIZE(c);
 
+		start_undo_chain(b);
 		for(i = 0; i < c && !(error = capitalize(b)) && !stop; i++);
+		end_undo_chain(b);
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
 
@@ -1108,11 +1226,15 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 		NORMALIZE(c);
 
+		start_undo_chain(b);
 		for(i = 0; i < c && !(error = center(b)) && !stop; i++) {
-			update_line(b, b->cur_y);
+			need_attr_update = TRUE;
+			b->attr_len = -1;
+			update_line(b, b->cur_y, FALSE);
 			move_to_sol(b);
 			if (line_down(b) != OK) break;
 		}
+		end_undo_chain(b);
 
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
@@ -1152,6 +1274,20 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case SAVEDEFPREFS_A:
 		return print_error(save_auto_prefs(b, DEF_PREFS_NAME)) ? ERROR : OK;
 
+	case SYNTAX_A:
+		if (!do_syntax) return SYNTAX_NOT_ENABLED;
+		if (p || (p = request_string("Syntax",  b->syn ? b->syn->name : NULL, TRUE, FALSE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
+			if (!strcmp(p, "*")) b->syn = NULL;
+			else error = print_error(load_syntax_by_name(b, p));
+			if (error == OK) {
+				reset_syntax_states(b);
+				reset_window();
+			}
+			free(p);
+			return error ? ERROR : OK;
+		}
+		return ERROR;
+
 	case ESCAPE_A:
 		handle_menus();
 		return OK;
