@@ -1,6 +1,6 @@
 /* Hardwired ANSI terminal control sequences.
 
-	Copyright (C) 2001-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 2001-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -20,6 +20,7 @@
 	02111-1307, USA.  */
 
 
+
 #include "ne.h"
 #ifdef TERMCAP
 #include "info2cap.h"
@@ -36,9 +37,16 @@ void setup_ansi_term(void) {
 
 #ifdef TERMCAP
 	ne_cursor_address = "\x1b[%i%d;%dH";
+	ne_set_background = "\x1b[4%dm";
+	ne_set_foreground = "\x1b[3%dm";
 #else
 	ne_cursor_address = "\x1b[%i%p1%d;%p2%dH";
+	ne_set_background = "\x1b[4%p1%dm";
+	ne_set_foreground = "\x1b[3%p1%dm";
 #endif
+	ne_enter_bold_mode = "\x1b[1m";
+	ne_enter_underline_mode = "\x1b[4m";
+	ne_enter_blink_mode = "\x1b[5m";
 	ne_lines = 25;
 	ne_columns = 80;
 	ne_carriage_return = "\xd";
@@ -54,18 +62,19 @@ void setup_ansi_term(void) {
 	ne_bell = "\x7";
 	ne_scroll_forward = "\xa";
 	ne_enter_standout_mode = "\x1b[7m";
-	ne_exit_standout_mode = "\x1b[m";
+	ne_exit_standout_mode = ne_exit_attribute_mode = "\x1b[m";
 	ne_magic_cookie_glitch = -1;
 	ne_move_standout_mode = 0;
 	ne_insert_line = "\x1b[L";
 	ne_delete_line = "\x1b[M";
 	ne_delete_character = "\x1b[P";
 	ne_move_insert_mode = 1;
-	ne_exit_attribute_mode = "\x1b[0;10m";
 	ne_exit_alt_charset_mode = "\x1b[10m";
 	ne_tilde_glitch = 0;
 	ne_memory_below = 0;
 	ne_has_meta_key = 0;
 	ne_clr_eol = "\x1b[K";
 	ne_transparent_underline = 0;
+	ne_no_color_video = 3;
+	ansi_color_ok = TRUE;
 }
