@@ -1,7 +1,7 @@
 /* Buffer handling functions, including allocation, deallocation, and I/O.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -21,8 +21,7 @@
 	02111-1307, USA.  */
 
 
-
-#include "ne.h"
+#include "ne.h" 
 
 /* The standard pool allocation dimension. */
 
@@ -139,15 +138,14 @@ line_desc_pool *alloc_line_desc_pool(int pool_size) {
 	if (pool_size < STD_LINE_DESC_POOL_SIZE) pool_size = STD_LINE_DESC_POOL_SIZE;
 
 	if (ldp = calloc(1, sizeof(line_desc_pool))) {
-		if (ldp->pool = calloc(pool_size, sizeof(line_desc))) {
+		if (ldp->pool = calloc(pool_size, do_syntax ? sizeof(line_desc) : sizeof(no_syntax_line_desc))) {
 			int i;
 
 			ldp->size = pool_size;
 			new_list(&ldp->free_list);
-			for(i = 0; i < pool_size; i++)
-			  {
-			    add_tail(&ldp->free_list, &ldp->pool[i].ld_node);
-			  }
+			for(i = 0; i < pool_size; i++) 
+				if (do_syntax) add_tail(&ldp->free_list, &ldp->pool[i].ld_node);
+				else add_tail(&ldp->free_list, &((no_syntax_line_desc *)ldp->pool)[i].ld_node);
 			return ldp;
 		}
 		free(ldp);
@@ -183,7 +181,7 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 		new_list(&b->line_desc_pool_list);
 		new_list(&b->line_desc_list);
 		new_list(&b->char_pool_list);
-
+		
 		b->cur_macro = alloc_char_stream(0);
 		b->opt.tab_size = 8;
 
@@ -195,6 +193,8 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 
 		b->opt.utf8auto = io_utf8;
 
+		b->attr_len = -1;
+
 		if (cur_b) {
 
 			b->opt.cur_clip       = cur_b->opt.cur_clip;
@@ -217,6 +217,7 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 			b->opt.case_search    = cur_b->opt.case_search;
 			b->opt.binary         = cur_b->opt.binary;
 			b->opt.utf8auto       = cur_b->opt.utf8auto;
+			b->opt.visual_bell    = cur_b->opt.visual_bell;
 
 		}
 		/* This leaves out onlyopt.read_only and opt.search_back, which are
@@ -276,6 +277,11 @@ void clear_buffer(buffer * const b) {
 
 	ld = alloc_line_desc(b);
 	add_head(&b->line_desc_list, &ld->ld_node);
+	if (do_syntax) {
+		ld->highlight_state.state = 0;
+		ld->highlight_state.stack = NULL;
+		ld->highlight_state.saved_s[0] = 0;
+	}
 
 	b->num_lines = 1;
 	reset_position_to_sof(b);
@@ -301,7 +307,7 @@ void free_buffer(buffer * const b) {
 	free(b->find_string);
 	free(b->replace_string);
 	free(b->command_line);
-
+	if (b->attr_buf) free(b->attr_buf);
 	free(b);
 }
 
@@ -406,10 +412,12 @@ int save_all_modified_buffers(void) {
 
 
 /* Allocates a line descriptor from the pools available in the given buffer. A
-	new pool is allocated and linked if necessary. */
-
+	new pool is allocated and linked if necessary. New line descriptors are
+	created with an invalid syntax state, so they will always force an update. */
 
 line_desc *alloc_line_desc(buffer * const b) {
+	
+	
 	line_desc_pool *ldp;
 	line_desc *ld;
 
@@ -436,6 +444,7 @@ line_desc *alloc_line_desc(buffer * const b) {
 
 			ld->line = NULL;
 			ld->line_len = 0;
+			if (do_syntax) ld->highlight_state.state = -1;
 			release_signals();
 			return ld;
 		}
@@ -450,6 +459,7 @@ line_desc *alloc_line_desc(buffer * const b) {
 		add_head(&b->line_desc_pool_list, &ldp->ldp_node);
 		rem(&(ld = (line_desc *)ldp->free_list.head)->ld_node);
 		ldp->allocated_items = 1;
+		if (do_syntax) ld->highlight_state.state = -1;
 		release_signals();
 		return ld;
 	}
@@ -477,7 +487,7 @@ void free_line_desc(buffer * const b, line_desc * const ld) {
 	while(ldp->ldp_node.next) {
 		assert_line_desc_pool(ldp);
 
-		if (ld >= ldp->pool && ld < ldp->pool + ldp->size) break;
+		if (ld >= ldp->pool && (do_syntax && ld < ldp->pool + ldp->size || !do_syntax && ld < (line_desc*)((no_syntax_line_desc *)ldp->pool + ldp->size))) break;
 		ldp = (line_desc_pool *)ldp->ldp_node.next;
 	}
 
@@ -685,7 +695,7 @@ void free_chars(buffer *const b, unsigned char *const p, const int len) {
 	a stream containing one NULL. */
 
 
-int insert_one_line(buffer * const b, line_desc * const ld, const int line, const int pos) {
+int insert_one_line(buffer * const b, line_desc * const ld, const int line, const int pos) {	
 	return insert_stream(b, ld, line, pos, "", 1);
 }
 
@@ -694,7 +704,6 @@ int insert_one_line(buffer * const b, line_desc * const ld, const int line, cons
 	UndelLine command. */
 
 int delete_one_line(buffer * const b, line_desc * const ld, const int line) {
-
 	int error;
 
 	assert_line_desc(ld, b->encoding);
@@ -714,14 +723,19 @@ int delete_one_line(buffer * const b, line_desc * const ld, const int line) {
 }
 
 
+
+
 /* Undeletes the last deleted line, using the last_deleted stream. */
 
 int undelete_line(buffer * const b) {
-
+	line_desc *ld = b->cur_line_desc;
 	if (!b->last_deleted) return ERROR;
 	start_undo_chain(b);
-	insert_one_line(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-	insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, b->last_deleted->stream, b->last_deleted->len);
+	if (b->cur_pos > ld->line_len) 
+		insert_spaces(b, ld, b->cur_line, ld->line_len, b->win_x + b->cur_x - calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding));
+
+	insert_one_line(b, ld, b->cur_line, b->cur_pos);
+	insert_stream(b, ld, b->cur_line, b->cur_pos, b->last_deleted->stream, b->last_deleted->len);
 	end_undo_chain(b);
 	return OK;
 }
@@ -752,8 +766,8 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 
 	if (!b || !ld || !stream || stream_len < 1 || pos > ld->line_len) return ERROR;
 
-	assert_buffer(b);
 	assert_line_desc(ld, b->encoding);
+	assert_buffer(b);
 
 	block_signals();
 
@@ -818,10 +832,10 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 			b->is_modified = 1;
 			
 			/* We just inserted len chars at (line,pos); adjust bookmarks and mark accordingly. */
-			if (b->marking && b->block_start_line == line && b->block_start_col > pos ) b->block_start_col += len;
+			if (b->marking && b->block_start_line == line && b->block_start_col > pos) b->block_start_col += len;
 
 			for (i=0, mask=b->bookmark_mask;  mask;  i++, mask >>= 1) 
-				if ((mask & 1) && b->bookmark[i].line == line && b->bookmark[i].pos > pos ) b->bookmark[i].pos += len;
+				if ((mask & 1) && b->bookmark[i].line == line && b->bookmark[i].pos > pos) b->bookmark[i].pos += len;
 		}
 
 		/* If the string we have inserted has a NULL at the end, we create a new
@@ -848,19 +862,19 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 				/* We just inserted a line break at (line,pos);
 				   adjust the buffer bookmarks and mark accordingly. */
 				if (b->marking) {
-					if (b->block_start_line == line && b->block_start_col > pos ) {
+					if (b->block_start_line == line && b->block_start_col > pos) {
 						b->block_start_col -= pos;
 						b->block_start_line++;
 					}
-					else if ( b->block_start_line > line ) b->block_start_line++;
+					else if (b->block_start_line > line) b->block_start_line++;
 				}
 				for (i=0, mask=b->bookmark_mask;  mask;  i++, mask >>= 1) {
 					if (mask & 1) {
-						if (b->bookmark[i].line == line && b->bookmark[i].pos > pos ) {
+						if (b->bookmark[i].line == line && b->bookmark[i].pos > pos) {
 							b->bookmark[i].pos -= pos;
 							b->bookmark[i].line++;
 						}
-						else if (b->bookmark[i].line > line ) b->bookmark[i].line++;
+						else if (b->bookmark[i].line > line) b->bookmark[i].line++;
 					}
 				}
 				pos = 0;
@@ -871,6 +885,9 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 				return OUT_OF_MEMORY;
 			}
 		}
+		
+		
+		
 
 		s += len + 1;
 	}
@@ -888,6 +905,7 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 
 int insert_one_char(buffer * const b, line_desc * const ld, const int line, const int pos, const int c) {
 
+	
 	static unsigned char t[8];
 
 	assert(b->encoding == ENC_8_BIT || b->encoding == ENC_UTF8 || c <= 127);
@@ -905,6 +923,9 @@ int insert_one_char(buffer * const b, line_desc * const ld, const int line, cons
 /* Inserts a number of spaces. */
 
 int insert_spaces(buffer * const b, line_desc * const ld, const int line, const int pos, int n) {
+	
+	
+	
 	static unsigned char spaces[MAX_STACK_SPACES];
 	int result = OK, i;
 
@@ -950,8 +971,10 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 
 	while(len) {
 
-		/* First case: we are just on the end of a line. We join the current line with
-		the following one (if it's there of course). */
+		/* First case: we are just on the end of a line. We join the current
+		line with the following one (if it's there of course). If, however,
+		the current line is empty, we rather remove it. The only difference
+		is in the resulting syntax state. */
 
 		if (pos == ld->line_len) {
 			unsigned char *p;
@@ -966,7 +989,7 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 					b->block_start_line--;
 					b->block_start_col += ld->line_len;
 				}
-				else if (b->block_start_line > line ) b->block_start_line--;
+				else if (b->block_start_line > line) b->block_start_line--;
 			}
 			for (n=0, mask=b->bookmark_mask;  mask;  n++, mask >>= 1) {
 				if (mask & 1) {
@@ -974,21 +997,20 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 						b->bookmark[n].line--;
 						b->bookmark[n].pos += ld->line_len;
 					}
-					else if (b->bookmark[n].line > line ) b->bookmark[n].line--;
+					else if (b->bookmark[n].line > line) b->bookmark[n].line--;
 				}
 			}
 			
 			/* If one of the lines is empty, or their contents are adjacent,
-			we simply set a pointer. */
+				we either do nothing or simply set a pointer. */
 
 			if (!ld->line || !next_ld->line || ld->line + ld->line_len == next_ld->line) {
 				if (!ld->line) ld->line = next_ld->line;
 			}
-
-			/* We try to allocate characters around one line or the other
-			one; if we fail, we allocate enough space for both lines elsewhere. */
-
 			else if ((n = alloc_chars_around(b, ld, next_ld->line_len, FALSE))<0 && (m = alloc_chars_around(b, next_ld, ld->line_len, TRUE))<0) {
+				/* We try to allocate characters around one line or the other
+					one; if we fail, we allocate enough space for both lines elsewhere. */
+
 				if (p = alloc_chars(b, ld->line_len + next_ld->line_len)) {
 
 					memcpy(p, ld->line, ld->line_len);
@@ -1049,17 +1071,17 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 			
 			/* We're about to erase n chars at (line,pos); adjust mark and bookmarks accordingly. */
 			if (b->marking)
-				if (b->block_start_line == line )
-					if ( b->block_start_col >= pos )
-						if (b->block_start_col < pos + n )
+				if (b->block_start_line == line)
+					if (b->block_start_col >= pos)
+						if (b->block_start_col < pos + n)
 							b->block_start_col = pos;
 						else
 							b->block_start_col -= n;
 			for (m=0, mask=b->bookmark_mask;  mask;  m++, mask>>=1) {
 				if (mask & 1) {
-					if (b->bookmark[m].line == line )
-						if ( b->bookmark[m].pos >= pos )
-							if (b->bookmark[m].pos < pos + n ) b->bookmark[m].pos = pos;
+					if (b->bookmark[m].line == line)
+						if (b->bookmark[m].pos >= pos)
+							if (b->bookmark[m].pos < pos + n) b->bookmark[m].pos = pos;
 							else b->bookmark[m].pos -= n;
 				}
 			}
@@ -1130,10 +1152,10 @@ int load_file_in_buffer(buffer * const b, const char *name) {
 	
 	name = tilde_expand(name);
 
-	if ( is_migrated(name) ) return FILE_IS_MIGRATED;
-	if ( is_directory(name) ) return FILE_IS_DIRECTORY;
+	if (is_migrated(name)) return FILE_IS_MIGRATED;
+	if (is_directory(name)) return FILE_IS_DIRECTORY;
     
-	if ((fh = open(name, READ_FLAGS)) >= 0 ) {
+	if ((fh = open(name, READ_FLAGS)) >= 0) {
 		result = load_fh_in_buffer(b, fh);
 		close(fh);
 		if (!result) b->opt.read_only = (access(name, W_OK) != 0);
@@ -1152,6 +1174,7 @@ int load_file_in_buffer(buffer * const b, const char *name) {
 	single read in a big pool. */
 
 int load_fh_in_buffer(buffer *b, int fh) {
+	
 
 	int i, len, num_lines;
 	encoding_type encoding;
@@ -1196,7 +1219,7 @@ int load_fh_in_buffer(buffer *b, int fh) {
 		if preserve_cr is set. */
 
 		for(num_lines = i=0; i < len; i++,p++)
-			if (!b->opt.binary && (*p == terminators[0] || *p == terminators[1] ) || !*p) {
+			if (!b->opt.binary && (*p == terminators[0] || *p == terminators[1]) || !*p) {
 				if (i < len - 1 && !b->opt.preserve_cr && p[0] == '\r' && p[1] == '\n') {
 					b->is_CRLF = TRUE;
 					p++, i++;
@@ -1228,31 +1251,37 @@ int load_fh_in_buffer(buffer *b, int fh) {
 			on the first pass).*/
 
 			for(i = 0; i < num_lines; i++) {
-				rem(&ldp->pool[i].ld_node);
-				add_tail(&b->line_desc_list, &ldp->pool[i].ld_node);
-
+				line_desc *ld = do_syntax ? &ldp->pool[i] : (line_desc *)&((no_syntax_line_desc *)ldp->pool)[i];
+				rem(&ld->ld_node);
+				add_tail(&b->line_desc_list, &ld->ld_node);
+				
+				/* ultima riga */
 				if (i == num_lines - 1) {
 					if (p - cp->pool < len) {
 						assert(*p && *p != terminators[0] && *p != terminators[1]);
-						ldp->pool[i].line = p;
-						ldp->pool[i].line_len = len - (p - cp->pool);
+						ld->line = p;
+						ld->line_len = len - (p - cp->pool);
 					}
 				}
+				
 				else {
 					q = p;
 					while((b->opt.binary || *q != terminators[0] && *q != terminators[1]) && *q) q++;
 					
-					ldp->pool[i].line_len = q - p;
-					ldp->pool[i].line = q - p ? p : NULL;
+					ld->line_len = q - p;
+					ld->line = q - p ? p : NULL;
 					
 					if (q - cp->pool < len - 1 && !b->opt.preserve_cr && q[0] == '\r' && q[1] == '\n') *q++ = 0;
 					*q++ = 0;
 					p = q;
+					
 				}
 			}
 
 
 			ldp->allocated_items = num_lines;
+			
+			
 
 			/* We set correctly the offsets of the first and last character used. If no
 			character is used (i.e., we have a file of line feeds), the char pool is
@@ -1274,7 +1303,7 @@ int load_fh_in_buffer(buffer *b, int fh) {
 
 			reset_position_to_sof(b);
 			if (b->opt.do_undo) b->undo.last_save_step = 0;
-
+			reset_syntax_states(b);
 			release_signals();
 			return OK;
 		}
@@ -1285,6 +1314,36 @@ int load_fh_in_buffer(buffer *b, int fh) {
 	return OUT_OF_MEMORY;
 }
 
+/* Recomputes initial states for all lines in a buffer. */
+
+void reset_syntax_states(buffer *b) {
+	if (b->syn) {
+		line_desc *ld;
+		HIGHLIGHT_STATE next_line_state = { 0, 0, "" };
+		ld = (line_desc *)b->line_desc_list.head;
+		while(ld->ld_node.next) {
+			ld->highlight_state = next_line_state;
+			next_line_state = parse(b->syn, ld, next_line_state, b->encoding == ENC_UTF8);
+			ld = (line_desc *)ld->ld_node.next;
+		}
+	}	
+}
+
+
+/* Ensures that the attribute buffer of this buffer is large enough. */
+
+void ensure_attr_buf(buffer * const b, const int capacity) {
+	/* attr_buf already exists? */
+	if (!b->attr_buf) {
+		b->attr_size = capacity;
+		b->attr_buf = malloc(b->attr_size * sizeof *b->attr_buf); 
+	}
+	else if (capacity > b->attr_size) {
+		b->attr_size = capacity;
+		b->attr_buf = realloc(b->attr_buf, b->attr_size * sizeof *b->attr_buf);
+	}
+}
+
 
 /* Here we save a buffer to a given file. If no file is specified, the
 buffer filename field is used. The is_modified flag is set to 0. */
@@ -1307,8 +1366,8 @@ int save_buffer_to_file(buffer *b, const char *name) {
 	
 	name = tilde_expand(name);
 
-	if ( is_migrated(name) ) return FILE_IS_MIGRATED;
-	if ( is_directory(name) ) return FILE_IS_MIGRATED;
+	if (is_migrated(name)) return FILE_IS_MIGRATED;
+	if (is_directory(name)) return FILE_IS_MIGRATED;
     
 	block_signals();
 
