@@ -1,7 +1,7 @@
 /* Clip handling functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -188,6 +188,7 @@ int copy_to_clip(buffer *b, int n, int cut) {
 					goto_line(b, b->block_start_line);
 					goto_column(b, b->block_start_col);
 					delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, clip_len);
+					update_syntax_and_lines(b, b->cur_line_desc, NULL);
 				}
 
 				return OK;
@@ -237,6 +238,7 @@ int copy_to_clip(buffer *b, int n, int cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, clip_len);
+				update_syntax_and_lines(b, b->cur_line_desc, NULL);
 				return OK;
 			}
 
@@ -251,7 +253,7 @@ int copy_to_clip(buffer *b, int n, int cut) {
 
 
 
-/* Simply erases a block, without putting it in a clip. */
+/* Simply erases a block, without putting it in a clip. Calls update_syntax_and_lines(). */
 
 int erase_block(buffer *b) {
 
@@ -301,6 +303,7 @@ int erase_block(buffer *b) {
 	}
 
 	delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, erase_len - 1);
+	update_syntax_and_lines(b, b->cur_line_desc, NULL);
 	return OK;
 }
 
@@ -318,16 +321,18 @@ int paste_to_buffer(buffer *b, int n) {
 
 	if (!cd->cs->len) return OK;
 	if (cd->cs->encoding == ENC_ASCII || b->encoding == ENC_ASCII || cd->cs->encoding == b->encoding) {
-		line_desc * const ld = b->cur_line_desc;
+		line_desc * const ld = b->cur_line_desc, * const end_ld = (line_desc *)b->cur_line_desc->ld_node.next;
 		if (b->encoding == ENC_ASCII) b->encoding = cd->cs->encoding;
 
 		start_undo_chain(b);
 		if (b->cur_pos > ld->line_len) 
 			insert_spaces(b, ld, b->cur_line, ld->line_len, b->win_x + b->cur_x - calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding));
 
-		insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, cd->cs->stream, cd->cs->len);
+		insert_stream(b, ld, b->cur_line, b->cur_pos, cd->cs->stream, cd->cs->len);
 		end_undo_chain(b);
 
+		assert(ld == b->cur_line_desc);
+		update_syntax_and_lines(b, ld, end_ld);
 		return OK;
 	}
 
@@ -399,6 +404,7 @@ int copy_vert_to_clip(buffer *b, int n, int cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) {
+					update_syntax_and_lines(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
 					goto_line(b, min(b->block_start_line, b->cur_line));
 					goto_column(b, min(b->block_start_col, b->win_x + b->cur_x));
 					end_undo_chain(b);
@@ -439,6 +445,7 @@ int copy_vert_to_clip(buffer *b, int n, int cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) {
+					update_syntax_and_lines(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
 					goto_line(b, min(b->block_start_line, b->cur_line));
 					goto_column(b, min(b->block_start_col, b->win_x + b->cur_x));
 					end_undo_chain(b);
@@ -457,6 +464,7 @@ int copy_vert_to_clip(buffer *b, int n, int cut) {
 	return OK;
 }
 
+/* Simply erases a vertical block, without putting it in a clip. Calls update_syntax_and_lines(). */
 
 int erase_vert_block(buffer *b) {
 
@@ -487,6 +495,7 @@ int erase_vert_block(buffer *b) {
 			delete_stream(b, ld, i, start_pos, len);
 			ld = (line_desc *)ld->ld_node.prev;
 		}
+		update_syntax_and_lines(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
 	}
 	else {
 		for(i = y; i <= b->block_start_line; i++) {
@@ -495,13 +504,13 @@ int erase_vert_block(buffer *b) {
 			delete_stream(b, ld, i, start_pos, len);
 			ld = (line_desc *)ld->ld_node.next;
 		}
+		update_syntax_and_lines(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
 	}
 
-	goto_line(b, min(b->block_start_line, b->cur_line));
-	goto_column(b, min(b->block_start_col, b->win_x + b->cur_x));
-
 	end_undo_chain(b);
 
+	goto_line(b, min(b->block_start_line, b->cur_line));
+	goto_column(b, min(b->block_start_col, b->win_x + b->cur_x));
 
 	return OK;
 }
@@ -517,7 +526,7 @@ int paste_vert_to_buffer(buffer *b, int n) {
 	int len, stream_len, i, x, line;
 	unsigned char *p;
 	clip_desc *cd;
-	line_desc *ld = b->cur_line_desc;
+	line_desc * ld = b->cur_line_desc;
 	unsigned char *stream;
 
 	if (!(cd = get_nth_clip(n))) return CLIP_DOESNT_EXIST;
@@ -560,7 +569,7 @@ int paste_vert_to_buffer(buffer *b, int n) {
 	}
 		
 	end_undo_chain(b);
-		
+	update_syntax_and_lines(b, b->cur_line_desc, ld);		
 	return OK;
 }
 
