@@ -37,8 +37,6 @@
 
 #define	BIG	9999
 
-extern char *BC, *UP;
-
 int	cost;					/* sums up costs */
 
 
@@ -294,8 +292,6 @@ void cmgoto (int row, int col) {
 
 void Wcm_clear () {
 	memset(&Wcm, 0, sizeof Wcm);
-	UP = 0;
-	BC = 0;
 }
 
 /* Initialises stuff. Returns 0 if can do CM, -1 if cannot, -2 if size not
