@@ -1,7 +1,7 @@
 /* Command table manipulation functions and vectors.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -68,6 +68,7 @@ static const command commands[ACTION_COUNT] = {
 
 	{ NAHL(ABOUT         ), NO_ARGS                                                               },
 	{ NAHL(ADJUSTVIEW    ),           ARG_IS_STRING                                               },
+	{ NAHL(ALERT         ), NO_ARGS                                                               },
 	{ NAHL(AUTOINDENT    ),                           IS_OPTION                                   },
 	{ NAHL(AUTOPREFS     ),                           IS_OPTION                                   },
 	{ NAHL(BACKSPACE     ),0                                                                      },
@@ -171,6 +172,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(SETBOOKMARK   ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(STATUSBAR     ),                           IS_OPTION                                   },
 	{ NAHL(SUSPEND       ), NO_ARGS                                                               },
+	{ NAHL(SYNTAX        ),           ARG_IS_STRING | IS_OPTION                                   },
 	{ NAHL(SYSTEM        ),           ARG_IS_STRING                                               },
 	{ NAHL(TABSIZE       ),                           IS_OPTION                                   },
 	{ NAHL(THROUGH       ),           ARG_IS_STRING                                               },
@@ -187,6 +189,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(UTF8AUTO      ),                           IS_OPTION                                   },
 	{ NAHL(UTF8IO        ),                           IS_OPTION                                   },
 	{ NAHL(VERBOSEMACROS ),                           IS_OPTION                                   },
+	{ NAHL(VISUALBELL    ),                           IS_OPTION                                   },
 	{ NAHL(WORDWRAP      ),                           IS_OPTION                                   },
 
 };
@@ -227,11 +230,11 @@ static macro_desc *macro_hash_table[MACRO_HASH_TABLE_SIZE];
 	but uses MACRO_HASH_TABLE_SIZE for its modulo. */
 
 
-static int hash(const unsigned char * const s, int len) {
+static int hash_cmd(const unsigned char * const s, int len) {
 	int h = -1;
 
 	while(len-- != 0) h = (h * 31 + ascii_up_case[s[len]]) % HASH_TABLE_SIZE;
-	return ( h + HASH_TABLE_SIZE ) % HASH_TABLE_SIZE;
+	return (h + HASH_TABLE_SIZE) % HASH_TABLE_SIZE;
 }
 
 
@@ -239,7 +242,7 @@ static int hash_macro(const unsigned char * const s, int len) {
 	int h = -1;
 
 	while(len-- != 0) h = (h * 31 + ascii_up_case[s[len]]) % MACRO_HASH_TABLE_SIZE;
-	return ( h + MACRO_HASH_TABLE_SIZE ) % MACRO_HASH_TABLE_SIZE;
+	return (h + MACRO_HASH_TABLE_SIZE) % MACRO_HASH_TABLE_SIZE;
 }
 
 
@@ -277,7 +280,7 @@ action parse_command_line(const unsigned char * command_line, int * const num_ar
 
 	while(*p && !isasciispace(*p)) p++;
 
-	h = hash(command_line, p - command_line);
+	h = hash_cmd(command_line, p - command_line);
 
 	if ((a = hash_table[h]) && !cmdcmp(commands[--a].name, command_line)
 		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, command_line)) {
@@ -406,12 +409,12 @@ void record_action(char_stream *cs, action a, int c, unsigned char *p, int verbo
 	is a valid "InsertChar ##" command. If it is, then insertchar_val() returns
 	the character code, otherwise it returns 0. */
 
-static int insertchar_val( const unsigned char *p ) {
+static int insertchar_val(const unsigned char *p) {
 	int h;
 	const unsigned char *cmd;
 	action a;
     
-	if ( !p || !*p ) return 0;
+	if ( !p || !*p) return 0;
     
 	while(isasciispace(*p)) p++;
 
@@ -421,7 +424,7 @@ static int insertchar_val( const unsigned char *p ) {
 	
 	while(*p && !isasciispace(*p)) p++;
 	
-	h = hash(cmd, p - cmd);
+	h = hash_cmd(cmd, p - cmd);
 	
 	if (((a = hash_table[h]) && !cmdcmp(commands[--a].name, cmd)
 		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, cmd))	&& a == INSERTCHAR_A) {
@@ -439,34 +442,34 @@ static int insertchar_val( const unsigned char *p ) {
 	character inserted by InsertChar is not an ASCII character, then we should
 	leave it as an InsertChar command to maximize portability of the macros. */
 
-void optimize_macro( char_stream *cs, int verbose ) {
+void optimize_macro(char_stream *cs, int verbose) {
 	char *cmd;
 	int	pos;
 	int	chr;
 	int	building = 0;
 	 
-	if ( !cs || !cs->len ) return;
+	if (!cs || !cs->len) return;
 
-	for (pos = 0; pos < cs->len; pos += strlen(&cs->stream[pos]) + 1 ) {
+	for (pos = 0; pos < cs->len; pos += strlen(&cs->stream[pos]) + 1) {
 		cmd = &cs->stream[pos];
-		chr = insertchar_val( cmd );
+		chr = insertchar_val(cmd);
 		if (chr < 0x80 && isprint(chr)) {
 			char two[2] = " ";
 			two[0] = chr;
-			delete_from_stream( cs, pos, strlen(cmd) + 1 );
-			if ( building ) {
+			delete_from_stream(cs, pos, strlen(cmd) + 1);
+			if (building) {
 				building++;
-				insert_in_stream( cs, two, building, 1 );
-				}
+				insert_in_stream(cs, two, building, 1);
+			}
 			else {
 				const char *insert;
 				int	len;
 				insert = verbose ? INSERTSTRING_NAME : INSERTSTRING_ABBREV;
 				len	 = strlen(insert);
-				insert_in_stream( cs, "\"",	pos, 2 );	/* Closing quote */
-				insert_in_stream( cs, two,	 pos, 1 );		/* The character itself */
-				insert_in_stream( cs, " \"",  pos, 2 );	/* space and openning quote */
-				insert_in_stream( cs, insert, pos, len );	/* The command itself */
+				insert_in_stream(cs, "\"",	pos, 2);	/* Closing quote */
+				insert_in_stream(cs, two,	 pos, 1);		/* The character itself */
+				insert_in_stream(cs, " \"",  pos, 2);	/* space and openning quote */
+				insert_in_stream(cs, insert, pos, len);	/* The command itself */
 				building = pos + len + 2;						/* This is where the char is now */
 			}
 		}
@@ -502,15 +505,20 @@ int play_macro(buffer *b, char_stream *cs) {
 	stop = FALSE;
 
 	while(!stop && p - stream < len) {	
-#ifdef TEST
+#ifdef NE_TEST
 		fprintf(stderr, "%s\n", p); /* During tests, we output to stderr the current command. */
 #endif
 
 		if (error = execute_command_line(b, p))
-#ifndef TEST
+#ifndef NE_TEST
 			break /* During tests, we never interrupt a macro. */
 #endif
 				;
+
+#ifdef NE_TEST
+		refresh_window(cur_buffer);
+		draw_status_bar();
+#endif
 		p += strlen(p) + 1;
 	}
 
@@ -635,12 +643,12 @@ void help(char *p) {
 	int i;
 	D(fprintf(stderr,"Help Called with parm %ld.\n",p);)
 	do {
-		if (p || (i = request_strings(command_names, ACTION_COUNT, MAX_COMMAND_WIDTH)) >= 0) {
+		if (p || (i = request_strings(command_names, ACTION_COUNT, MAX_COMMAND_WIDTH, FALSE)) >= 0) {
          D(fprintf(stderr,"Help check #2: p=%lx, i=%d\n",p,i);)
 			if (p) {
 				for(i = 0; i < strlen(p); i++) if (isasciispace((unsigned char)p[i])) break;
 
-				i = hash(p, i);
+				i = hash_cmd(p, i);
 
 				if ((a = hash_table[i]) && !cmdcmp(commands[--a].name, p)
 				|| (a = short_hash_table[i]) && !cmdcmp(commands[--a].short_name, p)) i = a;
@@ -661,7 +669,7 @@ void help(char *p) {
 
 			assert(i >= 0 && i < ACTION_COUNT);
 
-			i = request_strings(commands[i].help, commands[i].help_len, ne_columns);
+			i = request_strings(commands[i].help, commands[i].help_len, ne_columns, FALSE);
 		}
 	} while(i >= 0);
 }
