@@ -1,7 +1,7 @@
 /* Various editing functions such as word wrap, to upper, etc.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -9,12 +9,12 @@
 	under the terms of the GNU General Public License as published by the
 	Free Software Foundation; either version 2, or (at your option) any
 	later version.
-	
+
 	This program is distributed in the hope that it will be useful, but
 	WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	General Public License for more details.
-	
+
 	You should have received a copy of the GNU General Public License along
 	with this program; see the file COPYING.  If not, write to the Free
 	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -33,7 +33,7 @@
 
 static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 
-	int i, c, new_c, first_char = TRUE, x, pos = b->cur_pos, len, new_len, changed = FALSE;
+	int c, new_c, x, pos = b->cur_pos, len, new_len, changed = FALSE;
 	unsigned char *word;
 
 	assert_buffer(b);
@@ -64,12 +64,12 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 		char_right(b);
 		return OK;
 	}
-	
+
 	if (changed) {
 		/* We actually perform changes only if some character was case folded. */
 
 		if (!(word = malloc(new_len * sizeof *word))) return OUT_OF_MEMORY;
-		
+
 		pos = b->cur_pos;
 		new_len = 0;
 		/* Second pass: we actually build the transformed word. */
@@ -79,21 +79,27 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 				word[new_len] = new_len ? to_rest(c) : to_first(c);
 				new_len++;
 			}
-			
+
 			pos = next_pos(b->cur_line_desc->line, pos, b->encoding);
 		}
-		
+
 		start_undo_chain(b);
-		
+
 		delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, len);
 		insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, word, new_len);
-		
+
 		free(word);
-		update_partial_line(b, b->cur_y, x, TRUE);
 
 		end_undo_chain(b);
 	}
 
+	b->attr_len = -1;
+	update_line(b, b->cur_y, FALSE);
+	if (b->syn) {
+		need_attr_update = TRUE;
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+	}
+
 	search_word(b, 1);
 
 	return OK;
@@ -190,7 +196,7 @@ int match_bracket(buffer *b) {
 /* Breaks a line at the first possible position before the current cursor
 	position (i.e., at a tab or at a space). The space is deleted, and a new
 	line is inserted. The cursor is repositioned coherently.  The number of
-	characters existing on the new line is returned, or 0 if no word wrap was
+	characters existing on the new line is returned, or ERROR if no word wrap was
 	possible. */
 
 int word_wrap(buffer * const b) {
@@ -208,10 +214,10 @@ int word_wrap(buffer * const b) {
 
 	/* Skip non_space after leading white space */
 	while (first_pos < len && !ne_isspace(get_char(&line[first_pos], b->encoding), b->encoding)) first_pos = next_pos(line, first_pos, b->encoding);
-	
+
 	/* Now we know that the line shouldn't be broken before &line[first_pos]. */
-	
-   /* Start from the other end now and find a candidate space to break the line on.*/
+
+	/* Start from the other end now and find a candidate space to break the line on.*/
 	while((pos = prev_pos(line, pos, b->encoding)) && !ne_isspace(get_char(&line[pos], b->encoding), b->encoding));
 
 	if (! pos || pos < first_pos) return ERROR;
@@ -235,7 +241,7 @@ leading US-ASCII white space. The strategy is as follows:
      taken from the current line. Save a copy of it for later as space[].
 
   2. Start an undo chain.
-  
+
   3. Trim trailing space off the current line.
 
   4. while the current line is too long (i.e., needs to be split:
@@ -257,7 +263,7 @@ leading US-ASCII white space. The strategy is as follows:
   6. end the undo chain.
 
   7. Free space[].
-  
+
   8. and refresh the screen
 
   9. move to the next non-blank after the current line. (We have to do
@@ -279,21 +285,17 @@ static int	pa_space_pos;			/* How long pa_space is without expanding tabs */
 
 static int save_space(line_desc * const ld, const int tab_size, const encoding_type encoding) {
 	int pos;
-	if ( pa_space ) free(pa_space);
+	if (pa_space) free(pa_space);
 	pa_space	  = NULL;
 	pa_space_len = 0;
 	pa_space_pos = 0;
 
-	if ( !ld->line )
-		{
-			/* No data on this line. */
-			return 0;
-		}
+	if (!ld->line) return 0; /* No data on this line. */
 
 	pos = 0;
 	while(pos < ld->line_len && isasciispace(ld->line[pos])) pos = next_pos(ld->line, pos, encoding);
 
-	if (pos == ld->line_len ) return 0; /* Blank lines don't count. */
+	if (pos == ld->line_len) return 0; /* Blank lines don't count. */
 
 	pa_space_pos = pos;
 	pa_space_len = calc_width(ld, pos, tab_size, encoding);
@@ -303,27 +305,26 @@ static int save_space(line_desc * const ld, const int tab_size, const encoding_t
 		return 1;
 	}
 
-	if ( (pa_space = malloc(pos) ) )
-		{
-			memcpy(pa_space, ld->line, pos);
-			return 1;
-		}
+	if ((pa_space = malloc(pos))) {
+		memcpy(pa_space, ld->line, pos);
+		return 1;
+	}
 	return 0;
 }
 
 
-/* trim_trailing_space() removes spaces from the end of the line refered to by
+/* trim_trailing_space() removes spaces from the end of the line referred to by
 	the line_desc ld. The int line is necessary if you want to be able to undo
 	later. */
 
 static void trim_trailing_space(buffer * const b, line_desc *ld, const int line, const encoding_type encoding) {
 	int pos;
-	if ( !ld->line ) return;
-	insert_one_char(b, ld, line, ld->line_len, ' '); /* Make sure there's a space on the end and not a UTF8 
+	if (!ld->line) return;
+	insert_one_char(b, ld, line, ld->line_len, ' '); /* Make sure there's a space on the end and not a UTF8
                                                        so the prev_pos() below won't go berzerk. */
 	pos = ld->line_len;
 	while(pos > 0 && isasciispace(ld->line[pos - 1])) pos = prev_pos(ld->line, pos, encoding);
-	  
+
 	if (pos >= 0 && pos < ld->line_len) delete_stream(b, ld, line, pos, ld->line_len - pos);
 }
 
@@ -334,17 +335,16 @@ static void trim_trailing_space(buffer * const b, line_desc *ld, const int line,
 	*first_non_blank is set to the position of the first non-blank character on
 	the line. Otherwise, *first_non_blank is -1 and is_part_of_paragraph()
 	returns 0. */
-	 
+
 static int is_part_of_paragraph(const line_desc * const ld, const int tab_size, int * const first_non_blank, const encoding_type encoding) {
 	int pos = 0;
-	*first_non_blank = -1;  
-	if ( !ld->line ) return 0;
-	while ( pos < ld->line_len && isasciispace(ld->line[pos]) ) pos = next_pos(ld->line, pos, encoding);
-	if ( pos < ld->line_len && calc_width(ld, pos, tab_size, encoding) == pa_space_len )
-		{
-			*first_non_blank = pos;
-			return 1;
-		}
+	*first_non_blank = -1;
+	if (!ld->line) return 0;
+	while (pos < ld->line_len && isasciispace(ld->line[pos])) pos = next_pos(ld->line, pos, encoding);
+	if (pos < ld->line_len && calc_width(ld, pos, tab_size, encoding) == pa_space_len) {
+		*first_non_blank = pos;
+		return 1;
+	}
 	return 0;
 }
 
@@ -366,162 +366,144 @@ int paragraph(buffer * const b) {
 		done,
 		line			= b->cur_line,
 		right_margin = b->opt.right_margin ? b->opt.right_margin : ne_columns;
-	line_desc *ld = b->cur_line_desc;
+	line_desc *ld = b->cur_line_desc, *start_line_desc = ld;
 
-	if (!ld->line)
-		{
-			line_down(b);
-			return OK;
-		}
+	if (!ld->line)	return line_down(b);
 
 	/** Step 1 **/
-	if ( !(
-			 ( ld->ld_node.next->next &&
+	if (!(
+			 (ld->ld_node.next->next &&
 				save_space((line_desc *)ld->ld_node.next, b->opt.tab_size, b->encoding)
 				)
-			 || save_space( ld, b->opt.tab_size, b->encoding)
-			 )
-		  )
-		{
-			line_down(b);
-			return OK;
-		}
-	 
-	 
+			 || save_space(ld, b->opt.tab_size, b->encoding)
+			)
+		 ) return line_down(b);
+
+
 	/** Step 2 **/
 	start_undo_chain(b);
 
+	/* This useless insertion and deletion of a single character ensures
+	that the text isn't shifted way over to the left after an undo. */
+	insert_one_char(b, ld, line, 0, ' ');
+	delete_stream(b, ld, line, 0, 1);
+
 	done = FALSE;
-	do{
+	do {
 		/** Step 3 **/
-		trim_trailing_space( b, ld, line, b->encoding );
+		trim_trailing_space(b, ld, line, b->encoding);
 
 		/** Step 4 **/
-		while (!done && calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding) > right_margin)
-			{
-				int spaces;
-				int split;
-				int did_split;
-				  
-				/** 4.1  Find the split point **/
+		while (!done && calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding) > right_margin) {
+			int spaces;
+			int split_pos;
+			int did_split;
 
-				pos = ld->line_len;
-				did_split = split = spaces = 0;
-				while ( calc_width(ld, pos, b->opt.tab_size, b->encoding) > right_margin )
-					{ int tspaces = 0;
-						while(pos && !isasciispace(ld->line[prev_pos(ld->line, pos, b->encoding)]))
-							{
-								pos = prev_pos(ld->line, pos, b->encoding);
-							   if ( pos < 0 ) pos = 0;
-							}
-						while(pos &&  isasciispace(ld->line[prev_pos(ld->line, pos, b->encoding)]))
-							{
-								pos = prev_pos(ld->line, pos, b->encoding);
-								if ( pos < 0 ) pos = 0;
-								tspaces++;
-							}
-						if ( calc_width(ld, pos, b->opt.tab_size, b->encoding)>pa_space_len)
-							{
-								split  = pos;
-								spaces = tspaces;
-							}
+			/** 4.1  Find the split point **/
+
+			pos = 0;   /* Skip past leading spaces. */
+			while(pos < ld->line_len && isasciispace(ld->line[pos]))
+			  pos = next_pos(ld->line, pos, b->encoding);
+
+			did_split = split_pos = spaces = 0;
+
+			while (pos < ld->line_len &&
+					(calc_width(ld, pos, b->opt.tab_size, b->encoding) < right_margin ||
+					  ! split_pos)) {
+				if (isasciispace(ld->line[pos])) {
+					split_pos = pos;
+					spaces = 0;
+					while (pos < ld->line_len && isasciispace(ld->line[pos])) {
+						pos = next_pos(ld->line, pos, b->encoding);
+						spaces++;
 					}
-
-				/** 4.2  Remove any space at the split point **/
-
-				if ( split && spaces )
-					{
-						delete_stream(b, ld, line, split, spaces);
-						/** 4.3  Split the line at the split point.  (We are done with this line) **/
-						if ( ld->line_len > pos )
-							{
-								insert_one_line(b, ld, line, split);
-								did_split = 1;
-							}
-					}
-						
-				/** 4.4  Make the (new?) next line the current line **/
-				if ( ld->ld_node.next->next )
-					{
-						ld = (line_desc *)ld->ld_node.next;
-						line++;
-
-						/** 4.5  Insert the ps_space[] stream we saved in step 1. Note that  **/
-						/** we only want to do this if this line is the result of a split,	**/
-						/** which is true if did_split != 0.											**/
-
-						if ( did_split && pa_space && pa_space_len )
-							{ 
-								insert_stream(b, ld, line, 0, pa_space, pa_space_pos);
-							}
-	 
-						/** 4.6  Trim trailing space off the end of the line. **/
-						trim_trailing_space( b, ld, line, b->encoding );
-					}
-				else
-					done = TRUE;  
+				}
+				else pos = next_pos(ld->line, pos, b->encoding);
 			}
-				
+
+			if (split_pos) {
+				/** 4.2  Remove any space at the split point. **/
+				if (spaces) delete_stream(b, ld, line, split_pos, spaces);
+				/** 4.3  Split the line at the split point.  (We are done with this line) **/
+				insert_one_line(b, ld, line, split_pos);
+				did_split = 1;
+			}
+
+			/** 4.4  Make the (new?) next line the current line **/
+			if (ld->ld_node.next->next) {
+				ld = (line_desc *)ld->ld_node.next;
+				line++;
+
+				/** 4.5  Insert the ps_space[] stream we saved in step 1. Note that  **/
+				/** we only want to do this if this line is the result of a split,	**/
+				/** which is true if did_split != 0.											**/
+
+				if (did_split && pa_space && pa_space_len) {
+					insert_stream(b, ld, line, 0, pa_space, pa_space_pos);
+				}
+
+				/** 4.6  Trim trailing space off the end of the line. **/
+				trim_trailing_space(b, ld, line, b->encoding);
+			}
+			else done = TRUE;
+		}
+
 		/** 5. If the _following_ line is part of this paragraph (i.e., its first **/
 		/**	 non-blank character is in the correct position):						 **/
 
-		if ( ld->ld_node.next->next &&
-			  is_part_of_paragraph( (line_desc *)ld->ld_node.next, b->opt.tab_size, &pos, b->encoding ))
-			{
-				/** 5.1  Add a space to the end of the current line.							 **/
+		if (ld->ld_node.next->next && is_part_of_paragraph((line_desc *)ld->ld_node.next, b->opt.tab_size, &pos, b->encoding)) {
+			/** 5.1  Add a space to the end of the current line.							 **/
+			insert_one_char(b, ld, line, ld->line_len, ' ');
 
-				insert_one_char(b, ld, line, ld->line_len, ' ');
-
-				/** 5.2  Move following line's data starting with the first					**/
-				/**		non-blank to the end of the current line.								**/
-				/**  We do this by first deleting the leading spaces, then we				 **/
-				/**  delete the newline at the end of the current line.						 **/
-				if ( pos > 0 )
-					delete_stream( b, (line_desc *)ld->ld_node.next, line + 1, 0, pos );
-				delete_stream( b, ld, line, ld->line_len, 1 );
-			}
-		else
-			done = TRUE;
-	} while ( !done );
+			/** 5.2  Move following line's data starting with the first					**/
+			/** non-blank to the end of the current line.								**/
+			/**  We do this by first deleting the leading spaces, then we				 **/
+			/**  delete the newline at the end of the current line.						 **/
+			if (pos > 0) delete_stream(b, (line_desc *)ld->ld_node.next, line + 1, 0, pos);
+			delete_stream(b, ld, line, ld->line_len, 1);
+		}
+		else done = TRUE;
+	} while (!done);
 
 	/** Step 6 **/
 	end_undo_chain(b);
 
 	/** Step 7 **/
-	if ( pa_space )
-		{
-			free( pa_space );
-			pa_space = NULL;
-		}
+	if (pa_space) {
+		free(pa_space);
+		pa_space = NULL;
+	}
 
 	/** Step 8 **/
+	if (b->syn) {
+		b->attr_len = -1;
+		need_attr_update = TRUE;
+		update_syntax_states(b, -1, start_line_desc, (line_desc *)ld->ld_node.next);
+	}
 	update_window_lines(b, b->cur_y, ne_lines - 2, FALSE);
 
 	/** Step 9 **/
 	goto_line(b, line);
-	line_down(b);
+	if (line_down(b) == ERROR) return ERROR;
 
 	/* Try to find the first non-blank starting with this line. */
 	ld	= b->cur_line_desc;
-	line = b->cur_line;  
+	line = b->cur_line;
 
 	do {
-		if (ld->line)
-			{
-				for (pos = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding))
-					{
-						if ( !isasciispace(ld->line[pos]) )
-							{
-								goto_line(b, line );
-								goto_pos(b, pos );
-								return ld->ld_node.next ? OK : ERROR; 
-							}
-					}
+		if (ld->line) {
+			for (pos = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding)) {
+				if (!isasciispace(ld->line[pos])) {
+					goto_line(b, line);
+					goto_pos(b, pos);
+					return ld->ld_node.next ? OK : ERROR;
+				}
 			}
+		}
 		ld = (line_desc *)ld->ld_node.next;
 		line++;
-	} while ( ld->ld_node.next);
-			
+	} while (ld->ld_node.next);
+
 	return b->cur_line_desc->ld_node.next ? OK : ERROR;
 }
 
@@ -534,9 +516,9 @@ int center(buffer * const b) {
 	line_desc * const ld = b->cur_line_desc;
 	const int right_margin = b->opt.right_margin ? b->opt.right_margin : ne_columns;
 
-	int 
+	int
 		len,
-		start_pos = 0, 
+		start_pos = 0,
 		end_pos = ld->line_len;
 
 	while(start_pos < ld->line_len && isasciispace(ld->line[start_pos])) start_pos = next_pos(ld->line, start_pos, b->encoding);
@@ -551,7 +533,7 @@ int center(buffer * const b) {
 	delete_stream(b, ld, b->cur_line, end_pos, ld->line_len - end_pos);
 	delete_stream(b, ld, b->cur_line, 0, start_pos);
 	insert_spaces(b, ld, b->cur_line, 0, (right_margin - len) / 2);
-
+	
 	end_undo_chain(b);
 
 	return OK;
@@ -562,18 +544,17 @@ int center(buffer * const b) {
 /* Indents a line of the amount of whitespace present on the previous line. The
 	number of inserted bytes is returned. */
 
-int auto_indent_line(buffer * const b) {
+int auto_indent_line(buffer * const b, const int line, line_desc * const ld) {
 
-	int i;
-	line_desc * const ld = (line_desc *)b->cur_line_desc->ld_node.prev;
+	line_desc * const prev_ld = (line_desc *)ld->ld_node.prev;
 	int pos = 0;
 
-	assert(ld->ld_node.prev != NULL);
-	assert_line_desc(ld, b->encoding);
+	assert(prev_ld->ld_node.prev != NULL);
+	assert_line_desc(prev_ld, b->encoding);
 
-	if (ld->line_len == 0) return 0;
+	if (prev_ld->line_len == 0) return 0;
 
-	while(pos < ld->line_len && ne_isspace(get_char(&ld->line[pos], b->encoding), b->encoding)) pos = next_pos(ld->line, pos, b->encoding);
-	insert_stream(b, b->cur_line_desc, b->cur_line, 0, ld->line, pos);
+	while(pos < prev_ld->line_len && ne_isspace(get_char(&prev_ld->line[pos], b->encoding), b->encoding)) pos = next_pos(prev_ld->line, pos, b->encoding);
+	insert_stream(b, ld, line, 0, prev_ld->line, pos);
 	return pos;
 }
