@@ -1,7 +1,7 @@
 /* Error index enum, and extern declaration of the error vector.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -69,6 +69,7 @@ enum error {
 	REQUIRES_ARGUMENT,
 	WRONG_CHAR_AFTER_BACKSLASH,
 	CANT_OPEN_FILE,
+	CANT_OPEN_TEMPORARY_FILE,
 	ERROR_WHILE_WRITING,
 	HAS_NO_EXTENSION,
 	CANT_FIND_PREFS_DIR,
@@ -96,6 +97,8 @@ enum error {
 	UTF8_REGEXP_CHARACTER_CLASS_NOT_SUPPORTED,
 	UTF8_REGEXP_COMP_CHARACTER_CLASS_NOT_SUPPORTED,
 	GROUP_NOT_AVAILABLE,
+	SYNTAX_NOT_ENABLED,
+	NO_SYNTAX_FOR_EXT,
 
 	ERROR_COUNT
 };
