@@ -49,6 +49,13 @@ const char * const ADJUSTVIEW_HELP[20] = {
   "limits of the current TAB size).",
 };
 
+const char * const ALERT_HELP[5] = {
+  "Syntax: Alert",
+  "Abbreviation: AL",
+  "",
+  "beeps or flashes, depending on the value of the visual bell flag.",
+};
+
 const char * const AUTOINDENT_HELP[22] = {
   "Syntax: AutoIndent [0|1]",
   "Abbreviation: AI",
@@ -113,7 +120,7 @@ const char * const BINARY_HELP[22] = {
   "document is performed in a different way. On loading, only nulls are",
   "considered newlines; on saving, nulls are saved instead of newlines.",
   "This allows you to edit a binary file, fix some text in it, and save it",
-  "without modifying anything else. Normally, linefeeds, carriage returns",
+  "without modifying anything else. Normally, line feeds, carriage returns",
   "and nulls are considered newlines, so that what you load will have all",
   "nulls and carriage returns substituted by newlines when saved.",
   "",
@@ -300,7 +307,7 @@ const char * const ESCAPE_HELP[7] = {
   "",
   "toggles the menus on and off, or escapes from the input line. This",
   "command is mainly useful for reprogramming the menu activator, and it is",
-  "never registered while recording a macro. `Record'.",
+  "never registered while recording a macro. See `Record'.",
 };
 
 const char * const ESCAPETIME_HELP[16] = {
@@ -622,7 +629,7 @@ const char * const LOADPREFS_HELP[16] = {
   "purposes, such as filtering out specific settings.",
 };
 
-const char * const MACRO_HELP[31] = {
+const char * const MACRO_HELP[32] = {
   "Syntax: Macro [FILENAME]",
   "Abbreviation: MA",
   "",
@@ -636,23 +643,24 @@ const char * const MACRO_HELP[31] = {
   "   If you escape from the file requester, you can input the file name",
   "on the command line.",
   "",
-  "   Note that macros whose name does not conflict with a command can be",
+  "   Note that macros whose names do not conflict with a command can be",
   "called without using `Macro'. Whenever `ne' is required to perform a",
   "command it cannot find in its internal tables, it will look for a macro",
   "by that name in the current directory. If this search also fails, `ne'",
-  "looks in `~/.ne' and finally in the global ne directory (`/usr/lib/ne'",
-  "by default, or in a place specified by your `NE_GLOBAL_DIR' environment",
-  "variable) for a macro file by that name.",
+  "looks in `~/.ne' and finally in `ne''s  global directory (defined when",
+  "`ne' was built, or in a place specified by your `NE_GLOBAL_DIR'",
+  "environment variable) for a macro file by that name.",
   "",
-  "*Warning:* to improve (greatly) efficiency, the first time a macro is",
-  "executed it is cached into a hash table and is kept _forever_ in memory",
-  "unless the `UnloadMacros' command is issued; see `UnloadMacros'.",
-  "The next time a macro with the same file name is invoked, the cached",
-  "list is searched for it before accessing the file using a case",
-  "insensitive string comparison. That is, if you call `~/foobar/macro', a",
-  "subsequent call for `/usr/MACRO' or even just `MaCrO' will use the",
-  "cached version of `~/foobar/macro'.  Note that the cache table is",
-  "global to `ne' and not specific to any single document.",
+  "*Warning:* the first time a macro is executed it is cached into a hash",
+  "table and is kept _forever_ in memory unless the `UnloadMacros' command",
+  "is issued; see `UnloadMacros'. The next time a macro with the",
+  "same file name is invoked, the cached list is searched for it before",
+  "accessing the file using a case insensitive string comparison. That is,",
+  "if you call `~/foobar/macro', a subsequent call for `/usr/MACRO' or",
+  "even just `MaCrO' will use the cached version of `~/foobar/macro'.",
+  "Note that the cache table is global to `ne' and not specific to any",
+  "single document. This greatly improves efficiency when macros are used",
+  "repeatedly.",
 };
 
 const char * const MARK_HELP[14] = {
@@ -986,7 +994,7 @@ const char * const PLAY_HELP[15] = {
   "   A (possibly iterated) macro execution terminates as soon as its",
   "stream of instructions is exhausted, or one of its commands returns an",
   "error. This means that, for instance, you can perform some complex",
-  "operation on all the lines contaning a certain pattern by recording a",
+  "operation on all the lines containing a certain pattern by recording a",
   "macro that searches for the pattern and performs the operation, and",
   "then playing it a preposterously huge number of times.",
   "",
@@ -997,10 +1005,10 @@ const char * const POPPREFS_HELP[24] = {
   "Syntax: PopPrefs [N]",
   "Abbreviation: POPP",
   "",
-  "pops N sets of preferences from the prefs stack (where they were placed",
-  "previously by `PushPrefs') and applies those preferences to the current",
-  "buffer. See `PushPrefs'. If not specified, N defaults to one.",
-  "Note that the preferences stack is global, not buffer specific.",
+  "pops N sets of preferences from the preferences stack (where they were",
+  "placed previously by `PushPrefs') and applies those preferences to the",
+  "current buffer. See `PushPrefs'. If not specified, N defaults to",
+  "one. Note that the preferences stack is global, not buffer specific.",
   "Therefore you could `PushPrefs' one buffer's preferences, switch",
   "buffers, then `PopPrefs' those settings altering the preferences for",
   "the second buffer. The maximum preferences stack depth is 32.",
@@ -1010,7 +1018,7 @@ const char * const POPPREFS_HELP[24] = {
   "preferences necessary, do its work, then `PopPrefs' to restore the",
   "users previous preferences settings.",
   "",
-  "PopPrefs restores the following values from the prefs stack:",
+  "PopPrefs restores the following values from the preferences stack:",
   "",
   "        AutoIndent   DoUndo    NoFileReq    StatusBar",
   "        AutoPrefs    FastGUI   PreserveCR   TabSize",
@@ -1080,7 +1088,7 @@ const char * const PUSHPREFS_HELP[24] = {
   "preferences necessary, do its work, then `PopPrefs' to restore the",
   "users previous preferences settings.",
   "",
-  "`PushPrefs' saves the following values on the prefs stack:",
+  "`PushPrefs' saves the following values on the preferences stack:",
   "",
   "        AutoIndent   DoUndo    NoFileReq    StatusBar",
   "        AutoPrefs    FastGUI   PreserveCR   TabSize",
@@ -1445,6 +1453,49 @@ const char * const SUSPEND_HELP[6] = {
   "command `fg' is used to resume `ne'.",
 };
 
+const char * const SYNTAX_HELP[41] = {
+  "Syntax: Syntax [NAME|*]",
+  "Abbreviation: SY",
+  "",
+  "loads the syntax with the given name, and colors the current buffer",
+  "accordingly.",
+  "",
+  "   If the optional NAME argument is not specified, you are prompted for",
+  "one. The special NAME * turns off syntax highlighting for the current",
+  "document. Otherwise, NAME must match a syntax definition either in your",
+  "`~/.ne/syntax' directory or in a directory named `syntax' inside `ne''s",
+  "global directory. Additionally, `ne' has a table mapping common",
+  "suffixes to syntax names. If there is no syntax with a given name, `ne'",
+  "will try to remap the name using the following table  (the string",
+  "before the colon is the name of the syntax file):",
+  "",
+  "        ada: adb, ads",
+  "        asm: s",
+  "        c: c++, cc, cpp, h, h++ hpp, l, lex, y, yacc",
+  "        cobol: cbl, cob",
+  "        csh: tcsh",
+  "        diff: patch",
+  "        fortran: f, for",
+  "        html: htm",
+  "        java: js",
+  "        lisp: el, lsp",
+  "        mason: mas",
+  "        ocaml: ml, mli",
+  "        pascal: p, pas",
+  "        perl: pl, pm",
+  "        ps: eps",
+  "        python: py",
+  "        rexx: rex",
+  "        ruby: rb",
+  "        sh: bash, bash_login, bash_logout, bash_profile, bashrc, ksh,",
+  "            profile, rc",
+  "        skill: il",
+  "        tex: sty",
+  "        troff: 1",
+  "        verilog: v, vh, vhd",
+  "        xml: xsd",
+};
+
 const char * const SYSTEM_HELP[10] = {
   "Syntax: System [COMMAND]",
   "Abbreviation: SYS",
@@ -1634,7 +1685,7 @@ const char * const UTF8_HELP[20] = {
   "failure. When you try to reset it, the buffer is set to ASCII or",
   "ISO8859, depending on its content.  A `U' will appear on the status bar",
   "if the flag is true. Alternatively, an `A' or an `8' will be displayed",
-  "to denote whether the buffer is composed esclusively by US-ASCII",
+  "to denote whether the buffer is composed exclusively by US-ASCII",
   "characters, or also by other 8-bit characters (which are likely to be",
   "ISO-8859 encoded). Note that each time this command modifies the buffer",
   "encoding, it also resets the undo buffer.",
@@ -1690,6 +1741,18 @@ const char * const VERBOSEMACROS_HELP[17] = {
   "short to be an issue with respect to execution timing.",
 };
 
+const char * const VISUALBELL_HELP[10] = {
+  "Syntax: VisualBell [0|1]",
+  "Abbreviation: VB",
+  "",
+  "sets the visual bell flag. When this flag is true, the terminal will",
+  "flash (if possible) instead of beeping.",
+  "",
+  "   If you invoke `VisualBell' with no arguments, it will toggle the",
+  "flag. If you specify 0 or 1, the flag will be set to false or true,",
+  "respectively.",
+};
+
 const char * const WORDWRAP_HELP[12] = {
   "Syntax: WordWrap [0|1]",
   "Abbreviation: WW",
