@@ -23,6 +23,7 @@
 
 extern const char * const ABOUT_HELP[ 5 ];
 extern const char * const ADJUSTVIEW_HELP[ 20 ];
+extern const char * const ALERT_HELP[ 5 ];
 extern const char * const AUTOINDENT_HELP[ 22 ];
 extern const char * const AUTOPREFS_HELP[ 14 ];
 extern const char * const BACKSPACE_HELP[ 6 ];
@@ -66,7 +67,7 @@ extern const char * const LINEDOWN_HELP[ 6 ];
 extern const char * const LINEUP_HELP[ 6 ];
 extern const char * const LOADAUTOPREFS_HELP[ 7 ];
 extern const char * const LOADPREFS_HELP[ 16 ];
-extern const char * const MACRO_HELP[ 31 ];
+extern const char * const MACRO_HELP[ 32 ];
 extern const char * const MARK_HELP[ 14 ];
 extern const char * const MARKVERT_HELP[ 10 ];
 extern const char * const MATCHBRACKET_HELP[ 8 ];
@@ -126,6 +127,7 @@ extern const char * const SELECTDOC_HELP[ 14 ];
 extern const char * const SETBOOKMARK_HELP[ 10 ];
 extern const char * const STATUSBAR_HELP[ 19 ];
 extern const char * const SUSPEND_HELP[ 6 ];
+extern const char * const SYNTAX_HELP[ 41 ];
 extern const char * const SYSTEM_HELP[ 10 ];
 extern const char * const TABSIZE_HELP[ 9 ];
 extern const char * const THROUGH_HELP[ 15 ];
@@ -142,6 +144,7 @@ extern const char * const UTF8_HELP[ 20 ];
 extern const char * const UTF8AUTO_HELP[ 14 ];
 extern const char * const UTF8IO_HELP[ 13 ];
 extern const char * const VERBOSEMACROS_HELP[ 17 ];
+extern const char * const VISUALBELL_HELP[ 10 ];
 extern const char * const WORDWRAP_HELP[ 12 ];
 
 /* This file was automatically generated by info2src.pl. */
