@@ -45,6 +45,7 @@ my $names_c_fname = "names.c";
 my @infofiles = glob "ne.info{,-*}";
 
 my ( %commands,
+     %exts,
      $body_enums,
      $body_names,
      $hash_table_size,
@@ -206,10 +207,64 @@ sub gen_hash_tables
     die("Failed to generate non-conflicting hash tables.");
   }
 
-# Now we're done with hash table generation. Let's produce the output files.
+# Now we're done with hash table generation.
 ############################################################################
 
 
+############################################################################
+# Syntax filename Extension mapping
+#
+# The Syntax command docs contain a map of filename extensions to
+# .jsf file stems for commonly occuring real life files. We'll use that
+# to create the ext.c file, which contains that map in C.
+{
+  my $state = 'searching';
+  my $base  = '';
+
+  foreach ( @{$commands{SYNTAX}->{text}} )
+    {
+      my $line = $_;
+      if ( $state eq 'searching' )
+          {
+            if ( $line =~ m/^\s+([a-z0-9+_]+):\s+/i )
+                {
+                  $base = $1;
+                  $state = 'building';
+                }
+          }
+      if ( $state eq 'building' )
+          {
+            my @token = split /\s+/, $line;
+            foreach my $token ( @token )
+              {
+                if ( $token =~ m/([a-z0-9+_]+)(,|$)/i ) # conveniently skips "base:"
+                    {
+                      $exts{$1} = $base;
+                    }
+              }
+            $state = 'searching' unless $line =~ m/,\s*$/;
+          }
+    }
+  if ( open EXTIN,"ext.c.in" )
+      {
+        my @lines = <EXTIN>;
+        close EXTIN;
+        my $data = '';
+        foreach my $k ( sort keys %exts )
+          {
+            $data .= qq[\t{ "$k", "$exts{$k}" },\n];
+          }
+        $data =~ s/,\n$/\n/; # It's C, so take that last ',' off.
+        my $lines = join '', @lines;
+        $lines =~ s/<%%EXTMAP%%>/$data/;
+        if ( open EXT,">ext.c" )
+            {
+              print EXT $lines;
+              close EXT;
+            }
+      }
+}
+
 open_out_files();
 
 foreach my $cmd ( sort keys %commands )
@@ -299,12 +354,12 @@ sub body_names_c
 
     if ( ! defined $commands{$uccmd} )
         {
-          print "$0: \$commands{$uccmd} undefined.\n";
+          print "$0 [", __LINE__, "]: \$commands{$uccmd} undefined.\n";
           exit 2;
         }
     if ( ! defined $commands{$uccmd}->{"abbr"} )
         {
-          print "$0: \$commands{$uccmd}->{\"abbr\"} undefined.\n";
+          print "$0 [", __LINE__, "]: \$commands{$uccmd}->{\"abbr\"} undefined.\n";
           exit 2;
         }
 
