@@ -1,7 +1,7 @@
 /* Input line handling.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -51,7 +51,7 @@ static unsigned int print_prompt(const unsigned char * const prompt) {
 
 	standout_off();
 
-	output_char(' ', FALSE);
+	output_string(" ", FALSE);
 
 	reset_status_bar();
 
@@ -90,9 +90,10 @@ char request_char(const buffer * const b, const char * const prompt, const char
 	int c;
 	input_class ic;
 
+	set_attr(0);
 	print_prompt(prompt);
 
-	if (default_value) output_char(default_value, FALSE);
+	if (default_value) output_char(default_value, -1, FALSE);
 	move_cursor(b->cur_y, b->cur_x);
 
 	while(TRUE) {
@@ -155,82 +156,71 @@ char *request_string(const char * const prompt, const char * const default_strin
 
 static buffer *history_buff = NULL;
 
-static void init_history(void)
-	{
-		if ( !history_buff )
-				{
-					history_buff = alloc_buffer(NULL);
-					if (history_buff)
-						{
-							const char * const history_filename = tilde_expand( "~/.ne/.history" );
-							clear_buffer(history_buff);
-							history_buff->opt.do_undo = 0;
-							history_buff->opt.auto_prefs = 0;
-							load_file_in_buffer(history_buff, history_filename);
-							/* The history buffer is agnostic. The actual encoding of each line is detected dynamically. */
-							history_buff->encoding = ENC_8_BIT;
-							change_filename(history_buff, str_dup(history_filename));
-							assert_buffer(history_buff);
+static void init_history(void) {
+	if (!history_buff){
+		history_buff = alloc_buffer(NULL);
+		if (history_buff) {
+			const char * const history_filename = tilde_expand("~/.ne/.history");
+			clear_buffer(history_buff);
+			history_buff->opt.do_undo = 0;
+			history_buff->opt.auto_prefs = 0;
+			load_file_in_buffer(history_buff, history_filename);
+			/* The history buffer is agnostic. The actual encoding of each line is detected dynamically. */
+			history_buff->encoding = ENC_8_BIT;
+			change_filename(history_buff, str_dup(history_filename));
+			assert_buffer(history_buff);
 
-							/* This should be never necessary with new histories, as all lines will
-								be terminated by a life feed, but it is kept for backward
-								compatibility. */
-							
-							move_to_bof(history_buff);
-							if ( history_buff->cur_line_desc->line && history_buff->cur_line_desc->line_len ) {
-								insert_stream(	history_buff,
-													history_buff->cur_line_desc,
-													history_buff->cur_line,
-													history_buff->cur_line_desc->line_len,
-													"", 1);
-							}
-						}
-				}
-		if ( history_buff )
-				{
-					move_to_bof(history_buff);
-					move_to_sol(history_buff);
-				}
-	}
-
-void close_history( void )
-	{
-		if ( history_buff )
-			{
-				if (history_buff->is_modified)
-					{
-						while( history_buff->num_lines > 500 )
-							{
-								move_to_sof(history_buff);
-								delete_one_line(history_buff,
-								           history_buff->cur_line_desc,
-								           history_buff->cur_line );
-								assert_buffer(history_buff);
-							}
-						save_buffer_to_file(history_buff,NULL);
-					}
-				free_buffer(history_buff);
-				history_buff = NULL;
+			/* This should be never necessary with new histories, as all lines will
+				be terminated by a life feed, but it is kept for backward
+				compatibility. */
+			
+			move_to_bof(history_buff);
+			if (history_buff->cur_line_desc->line && history_buff->cur_line_desc->line_len) {
+				insert_stream(	history_buff,
+									history_buff->cur_line_desc,
+									history_buff->cur_line,
+									history_buff->cur_line_desc->line_len,
+									"", 1);
 			}
-	}	  
-
-static void add_to_history(const unsigned char * const str)
-	{
-		if ( !history_buff || !str || !*str ) return;
-
-		move_to_bof(history_buff);
-
-		/* This insert_stream() takes care of adding a line, including a life
-			feed at the end. */
-
-		insert_stream( history_buff,
-							history_buff->cur_line_desc,
-							history_buff->cur_line,
-							history_buff->cur_line_desc->line_len,
-							str,
-							strlen(str) + 1);
-		assert_buffer( history_buff );
+		}
 	}
+	if (history_buff) {
+		move_to_bof(history_buff);
+		move_to_sol(history_buff);
+	}
+}
+
+void close_history(void) {
+	if (history_buff) {
+		if (history_buff->is_modified) {
+			while(history_buff->num_lines > 500) {
+				move_to_sof(history_buff);
+				delete_one_line(history_buff, history_buff->cur_line_desc, history_buff->cur_line);
+				assert_buffer(history_buff);
+			}
+			save_buffer_to_file(history_buff,NULL);
+		}
+		free_buffer(history_buff);
+		history_buff = NULL;
+	}
+}
+
+static void add_to_history(const unsigned char * const str) {
+	if (!history_buff || !str || !*str) return;
+
+	move_to_bof(history_buff);
+
+	/* This insert_stream() takes care of adding a line, including a life
+		feed at the end. */
+
+	insert_stream(history_buff,
+						history_buff->cur_line_desc,
+						history_buff->cur_line,
+						history_buff->cur_line_desc->line_len,
+						str,
+						strlen(str) + 1);
+	assert_buffer(history_buff);
+}
 
 /* request() is the main function that serves request_number() and
 	request_string(). Given a prompt, a default string and a boolean flag which
@@ -278,7 +268,7 @@ static void input_refresh(void) {
 	move_cursor(ne_lines - 1, start_x);
 	for(i = start_x, j = offset; j < len; i += get_char_width(&input_buffer[j], encoding), j = next_pos(input_buffer, j, encoding)) {
 		if (i + get_char_width(&input_buffer[j], encoding) >= ne_columns) break;
-		output_char(get_char(&input_buffer[j], encoding), encoding);
+		output_char(get_char(&input_buffer[j], encoding), -1, encoding);
 	}
 	clear_to_eol();
 	fflush(stdout);
@@ -303,7 +293,7 @@ static void input_move_left(const int do_refresh) {
 					delete_chars(get_char_width(&input_buffer[j], encoding));
 				}
 				move_cursor(ne_lines - 1, start_x);
-				insert_char(get_char(&input_buffer[pos], encoding), encoding);
+				insert_char(get_char(&input_buffer[pos], encoding), 0, encoding);
 				move_cursor(ne_lines - 1, start_x);
 			}
 			else if (do_refresh) input_refresh();
@@ -340,12 +330,12 @@ static void input_move_right(const int do_refresh) {
 				move_cursor(ne_lines - 1, start_x);
 				delete_chars(width);
 				move_cursor(ne_lines - 1, x - x_delta);
-				output_char(get_char(&input_buffer[prev_pos], encoding), encoding);
+				output_char(get_char(&input_buffer[prev_pos], encoding), -1, encoding);
 
 				i = x;
 				j = pos;
 				while(j < len && i + (width = get_char_width(&input_buffer[j], encoding)) < ne_columns) {
-					output_char(get_char(&input_buffer[j], encoding), encoding);
+					output_char(get_char(&input_buffer[j], encoding), -1, encoding);
 					i += width;
 					j = next_pos(input_buffer, j, encoding);
 				}
@@ -434,7 +424,7 @@ static void input_paste(void) {
 
 	if (cd) {
 		if (cd->cs->encoding != ENC_ASCII && encoding != ENC_ASCII && cd->cs->encoding != encoding) {
-			ring_bell();
+			alert();
 			return;
 		}
 
@@ -456,6 +446,8 @@ char *request(const char *prompt, const char * const default_string, const int a
 	int c, c_len, c_width, first_char_typed = TRUE, last_char_completion = FALSE, selection = FALSE, quoted;
 	unsigned char *completion, *prefix, *p;
 
+	set_attr(0);
+
 	input_buffer[pos = len = offset = 0] = 0;
 	encoding = ENC_ASCII;
 	x = start_x = print_prompt(prompt);
@@ -486,7 +478,7 @@ char *request(const char *prompt, const char * const default_string, const int a
 		switch(ic) {
 
 		case INVALID:
-			ring_bell();
+			alert();
 			break;
 					
 		case ALPHA:
@@ -513,8 +505,8 @@ char *request(const char *prompt, const char * const default_string, const int a
 				move_cursor(ne_lines - 1, x);
 
 				if (x < ne_columns - c_width) {
-					if (pos == len - c_len) output_char(c, encoding);
-					else if (char_ins_del_ok) insert_char(c, encoding);
+					if (pos == len - c_len) output_char(c, -1, encoding);
+					else if (char_ins_del_ok) insert_char(c, -1, encoding);
 					else input_refresh();
 				}
 
@@ -556,7 +548,7 @@ char *request(const char *prompt, const char * const default_string, const int a
 				else {
 					completion = p = complete(prefix);
 					last_char_completion = TRUE;
-					if (!completion) ring_bell();
+					if (!completion) alert();
 				}
 
 				if (completion && (prefix - input_buffer) + strlen(completion) + 1 < MAX_INPUT_LINE_LEN) {
@@ -572,7 +564,7 @@ char *request(const char *prompt, const char * const default_string, const int a
 						if (quoted) input_move_left(FALSE);
 						input_refresh();
 					}
-					else ring_bell();
+					else alert();
 				}
 				else if (quoted) strcat(prefix, "\"");
 
@@ -594,16 +586,16 @@ char *request(const char *prompt, const char * const default_string, const int a
 				case PAGEDOWN_A:
 				case NEXTPAGE_A:
 				case PREVPAGE_A:
-					if ( history_buff ) {
+					if (history_buff) {
 						switch(a) {
-						case LINEUP_A: line_up( history_buff ); break;
-						case LINEDOWN_A: line_down( history_buff ); break;
-						case MOVESOF_A: move_to_sof( history_buff ); break;
-						case MOVEEOF_A: move_to_bof( history_buff ); break;
+						case LINEUP_A: line_up(history_buff); break;
+						case LINEDOWN_A: line_down(history_buff); break;
+						case MOVESOF_A: move_to_sof(history_buff); break;
+						case MOVEEOF_A: move_to_bof(history_buff); break;
 						case PAGEUP_A:
-						case PREVPAGE_A: prev_page( history_buff ); break;
+						case PREVPAGE_A: prev_page(history_buff); break;
 						case PAGEDOWN_A:
-						case NEXTPAGE_A: next_page( history_buff ); break;
+						case NEXTPAGE_A: next_page(history_buff); break;
 						}
 
 						/* In some cases, the default displayed on the command line will be the same as the 
@@ -613,9 +605,9 @@ char *request(const char *prompt, const char * const default_string, const int a
 							 && a == LINEUP_A 
 							 && history_buff->cur_line_desc->line 
 							 && !strncmp(history_buff->cur_line_desc->line, input_buffer, history_buff->cur_line_desc->line_len))
-							line_up( history_buff);
+							line_up(history_buff);
 						
-						if ( history_buff->cur_line_desc->line ) {
+						if (history_buff->cur_line_desc->line) {
 							strncpy(input_buffer,
 									  history_buff->cur_line_desc->line,
 									  min(history_buff->cur_line_desc->line_len,MAX_INPUT_LINE_LEN));
@@ -666,7 +658,7 @@ char *request(const char *prompt, const char * const default_string, const int a
 							if (j < len) {
 								move_cursor(ne_lines - 1, i);
 								while(j < len && i + get_char_width(&input_buffer[j], encoding) < ne_columns) {
-									output_char(get_char(&input_buffer[j], encoding), encoding);
+									output_char(get_char(&input_buffer[j], encoding), -1, encoding);
 									i += get_char_width(&input_buffer[j], encoding);
 									j = next_pos(input_buffer, j, encoding);
 								}
