@@ -1,7 +1,7 @@
 /* Terminfo database scanning and keyboard escape sequence matching functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -88,17 +88,17 @@ int binsearch(const char * const s) {
 
 
 #ifdef DEBUGPRINTF
-void dump_keys( void )
+void dump_keys(void)
   {
     int i;
     for (i = 0; i < num_keys; i++)
       {
         unsigned char *p = key[i].string;
         fprintf(stderr,"%3d: \"",i);
-        while ( *p )
+        while (*p)
           {
-            if ( isprint(*p) ) fprintf(stderr,"%c",*p);
-            else               fprintf(stderr,"\\x%02x", *p );
+            if (isprint(*p)) fprintf(stderr,"%c",*p);
+            else fprintf(stderr,"\\x%02x", *p );
             p++;
           }
         fprintf (stderr,"\"\t-> %d\n", key[i].code );
@@ -408,7 +408,6 @@ void set_escape_time(const int new_escape_time) {
 #ifndef _AMIGA
 
 static void set_termios_timeout(const int timeout) {
-
 	struct termios termios;
 
 	tcgetattr(0, &termios);
@@ -442,7 +441,7 @@ int get_key_code(void) {
 	static int cur_len = 0;
 	static unsigned char kbd_buffer[KBD_BUF_SIZE];
 
-	int c, last_match = 0, cur_key = 0, partial_match = FALSE, partial_is_utf8 = FALSE;
+	int c, e, last_match = 0, cur_key = 0, partial_match = FALSE, partial_is_utf8 = FALSE;
 
 	while(TRUE) {
 
@@ -532,14 +531,17 @@ int get_key_code(void) {
 		if (partial_match) set_termios_timeout(escape_time);
 
 		errno = 0;
-
 		c = getchar();
 
-		if (c == EOF && (!partial_match || errno) && errno != EINTR) kill(getpid(), SIGTERM);
+		e = errno;
+		/* This is necessary to circumvent the slightly different behaviour of getc() in Linux and BSD. */
+		clearerr(stdin);
+		if (c == EOF && (!partial_match || e) && e != EINTR) kill(getpid(), SIGTERM);
 
-		if (c == EOF && errno == EINTR) return -NE_KEY_IGNORE - 1;
+		if (c == EOF && e == EINTR) return -NE_KEY_IGNORE - 1;
 
 		if (partial_match) set_termios_timeout(0);
+
 #endif
 
 		partial_match = FALSE;
