@@ -1,7 +1,7 @@
 /* Menu handling function. Includes also key and menu configuration parsing.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -321,7 +321,7 @@ int dump_config(void)
     
     for (key = 0; key < NUM_KEYS; key++)
       {
-        if ( key_binding[key] && key_binding[key][0] )
+        if (key_binding[key] && key_binding[key][0])
           {
             fprintf(f,"%s %4x %s\n",KEY_KEYWORD, key, key_binding[key] );
           }
@@ -333,15 +333,16 @@ int dump_config(void)
   
 static void draw_cur_item(const int n) {
 	move_cursor(menus[n].cur_item + 1, menus[n].xpos);
-	if (!cur_buffer->opt.fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
+	if (!cur_buffer->opt.fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, NULL, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
 }
 
 
 static void undraw_cur_item(const int n) {
 	if (!cur_buffer->opt.fast_gui && standout_ok)  {
+		set_attr(0);
 		standout_on();
 		move_cursor(1 + menus[n].cur_item, menus[n].xpos);
-		output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
+		output_chars(menus[n].items[menus[n].cur_item].text, NULL, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
 		standout_off();
 	}
 }
@@ -357,6 +358,7 @@ static void draw_menu(const int n) {
 	if (menus[n].cur_item + 1 + (standout_ok == 0) >= ne_lines - 1) menus[n].cur_item = 0;
 
 	move_cursor(0, menus[n].xpos);
+	set_attr(0);
 	output_string(menus[n].text, TRUE);
 
 	for(i = 0; i < menus[n].item_num; i++) {
@@ -364,20 +366,20 @@ static void draw_menu(const int n) {
 
 		move_cursor(i + 1, menus[n].xpos - 1);
 
-		if (!standout_ok) output_chars("|", 1, FALSE);
+		if (!standout_ok) output_string("|", FALSE);
 
 		standout_on();
-		output_char(' ', FALSE);
+		output_string(" ", FALSE);
 		output_string(menus[n].items[i].text, TRUE);
-		output_char(' ', FALSE);
+		output_string(" ", FALSE);
 		standout_off();
 
-		if (!standout_ok) output_chars("|", 1, FALSE);
+		if (!standout_ok) output_string("|", FALSE);
 	}
 
 	if (!standout_ok) {
 		move_cursor(i + 1, menus[n].xpos - 1);
-		for(i = 0; i < menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA); i++) output_chars("-", 1, FALSE);
+		for(i = 0; i < menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA); i++) output_string("-", FALSE);
 	}
 
 	draw_cur_item(n);
@@ -391,7 +393,7 @@ static void undraw_menu(const int n) {
 	int i;
 	line_desc *ld = cur_buffer->top_line_desc;
 
-
+	set_attr(0);
 	standout_on();
 	move_cursor(0, menus[n].xpos);
 	output_string(menus[n].text, TRUE);
@@ -401,7 +403,8 @@ static void undraw_menu(const int n) {
 		if (i >= ne_lines - 1) break;
 		if (ld->ld_node.next->next) {
 			ld = (line_desc *)ld->ld_node.next;
-			output_line_desc(i, menus[n].xpos - 1, ld, cur_buffer->win_x + menus[n].xpos - 1, menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA), cur_buffer->opt.tab_size, FALSE, cur_buffer->encoding == ENC_UTF8);
+			if (cur_buffer->syn) parse(cur_buffer->syn, ld, ld->highlight_state, cur_buffer->encoding == ENC_UTF8);
+			output_line_desc(i, menus[n].xpos - 1, ld, cur_buffer->win_x + menus[n].xpos - 1, menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA), cur_buffer->opt.tab_size, FALSE, cur_buffer->encoding == ENC_UTF8, cur_buffer->syn ? attr_buf : NULL, NULL, 0);
 		}
 		else {
 			move_cursor(i, menus[n].xpos - 1);
@@ -434,7 +437,7 @@ static void draw_item(const int item) {
 
 static void draw_next_menu(void) {
 	undraw_menu(current_menu);
-	current_menu = ++current_menu % menu_num;
+	current_menu = (current_menu + 1) % menu_num;
 	if (menus[current_menu].xpos >= ne_columns) current_menu = 0;
 	draw_menu(current_menu);
 }
@@ -466,14 +469,13 @@ int search_menu_item(int n, int c) {
 
 	for(i = 0, j = menus[n].cur_item; i < menus[n].item_num - 1; i++) {
 		if (++j % menus[n].item_num + 1 + (standout_ok == 0) >= ne_lines - 1) continue;
-		if (menus[n].items[++j % menus[n].item_num].text[0] == c) return j % menus[n].item_num;
+		if (menus[n].items[j % menus[n].item_num].text[0] == c) return j % menus[n].item_num;
 	}
 
 	return -1;
 }
 
 
-
 static void item_search(const int c) {
 	int new_item;
 
@@ -498,11 +500,12 @@ static void draw_first_menu(void) {
 
 	move_cursor(0,0);
 
+	set_attr(0);
 	standout_on();
 	if (!cur_buffer->opt.fast_gui && standout_ok) cursor_off();
 
 	while(i < ne_columns) {
-		output_char(' ', FALSE);
+		output_string(" ", FALSE);
 		i++;
 		if (n < menu_num) {
 			output_string(menus[n].text, TRUE);
@@ -519,7 +522,7 @@ static void draw_first_menu(void) {
 
 static void undraw_last_menu(void) {
 	undraw_menu(current_menu);
-	update_line(cur_buffer, 0);
+	update_line(cur_buffer, 0, FALSE);
 	cursor_on();
 }
 
@@ -636,6 +639,8 @@ void draw_status_bar(void) {
 		return;
 	}
 
+	set_attr(0);
+
 	if (!bar_gone && cur_buffer->opt.status_bar) {
 		/* This is the space occupied up to "L:", included. */
 		const int offset = cur_buffer->opt.fast_gui || !standout_ok ? 5: 3;
@@ -652,7 +657,7 @@ void draw_status_bar(void) {
 
 		if (update_linecols) {
 			move_cursor(ne_lines - 1, offset);
-			output_chars(bar_buffer, i, TRUE);
+			output_chars(bar_buffer, NULL, i, TRUE);
 		}
 
 		if (strcmp(flag_string, p = gen_flag_string(cur_buffer))) {
@@ -678,7 +683,7 @@ void draw_status_bar(void) {
 		len = sprintf(bar_buffer, cur_buffer->opt.fast_gui || !standout_ok ? ">> L:%8d C:%8d %3d%% %s " : " L:%8d C:%8d %3d%% %s ", y + 1, x + 1, ((y + 1) * 100) / cur_buffer->num_lines, flag_string);
 
 		move_cursor(ne_lines - 1, 0);
-		output_chars(bar_buffer, len, TRUE);
+		output_chars(bar_buffer, NULL, len, TRUE);
 
 		if (len < ne_columns - 1) {
 			if (cur_buffer->filename) {
@@ -699,7 +704,7 @@ void draw_status_bar(void) {
 		}
 
 		if (!cur_buffer->opt.fast_gui && standout_ok) {
-			output_spaces(ne_columns);
+			output_spaces(ne_columns, NULL);
 			standout_off();
 		}
 		else clear_to_eol();
@@ -721,6 +726,8 @@ void print_message(const char * const message) {
 
 	move_cursor(ne_lines - 1, 0);
 
+	set_attr(0);
+
 	if (cur_buffer->opt.fast_gui || !standout_ok || !cur_buffer->opt.status_bar) {
 		clear_to_eol();
 		output_string(message, TRUE);
@@ -728,7 +735,7 @@ void print_message(const char * const message) {
 	else {
 		standout_on();
 		output_string(message, TRUE);
-		output_spaces(ne_columns - strlen(message));
+		output_spaces(ne_columns - strlen(message), NULL);
 		standout_off();
 	}
 
@@ -749,7 +756,7 @@ int print_error(const int error_num) {
 
 	if (error_num > 0) {
 		print_message(error_msg[error_num]);
-		ring_bell();
+		alert();
 	}
 	return error_num;
 }
@@ -769,6 +776,15 @@ void print_info(const int info_num) {
 
 
 
+/* Rings a bell or flashes the screen, depending on the user preference. */
+
+void alert(void) {
+	if (cur_buffer->opt.visual_bell) do_flash();
+	else ring_bell();
+}
+
+
+
 /* Handles the menu system: it displays the menus, parses the keyboard input,
 	and eventually executes the correct command line. Note that we support ':'
 	for going to the command line, alphabetic search (upper case for menus,
@@ -790,7 +806,7 @@ void handle_menus(void) {
 
 		switch(ic) {
 		case INVALID:
-			ring_bell();
+			alert();
 			break;
 			
 		case ALPHA:
