@@ -33,6 +33,9 @@ const char ABOUT_ABBREV[] = "About";
 const char ADJUSTVIEW_NAME[]   = "AdjustView";
 const char ADJUSTVIEW_ABBREV[] = "AV";
 
+const char ALERT_NAME[]   = "Alert";
+const char ALERT_ABBREV[] = "AL";
+
 const char AUTOINDENT_NAME[]   = "AutoIndent";
 const char AUTOINDENT_ABBREV[] = "AI";
 
@@ -342,6 +345,9 @@ const char STATUSBAR_ABBREV[] = "ST";
 const char SUSPEND_NAME[]   = "Suspend";
 const char SUSPEND_ABBREV[] = "SU";
 
+const char SYNTAX_NAME[]   = "Syntax";
+const char SYNTAX_ABBREV[] = "SY";
+
 const char SYSTEM_NAME[]   = "System";
 const char SYSTEM_ABBREV[] = "SYS";
 
@@ -390,6 +396,9 @@ const char UTF8IO_ABBREV[] = "U8IO";
 const char VERBOSEMACROS_NAME[]   = "VerboseMacros";
 const char VERBOSEMACROS_ABBREV[] = "VM";
 
+const char VISUALBELL_NAME[]   = "VisualBell";
+const char VISUALBELL_ABBREV[] = "VB";
+
 const char WORDWRAP_NAME[]   = "WordWrap";
 const char WORDWRAP_ABBREV[] = "WW";
 
@@ -401,35 +410,36 @@ const char CENTERVIEW_ABBREV[] = "AV C";
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
 const char * const command_names[ACTION_COUNT+1] = {
-  ABOUT_NAME,  ADJUSTVIEW_NAME,  AUTOINDENT_NAME,  AUTOPREFS_NAME,
-  BACKSPACE_NAME,  BEEP_NAME,  BINARY_NAME,  CAPITALIZE_NAME,
-  CASESEARCH_NAME,  CENTER_NAME,  CLEAR_NAME,  CLIPNUMBER_NAME,
-  CLOSEDOC_NAME,  COPY_NAME,  CRLF_NAME,  CUT_NAME,
-  DELETECHAR_NAME,  DELETEEOL_NAME,  DELETELINE_NAME,  DOUNDO_NAME,
-  ERASE_NAME,  ESCAPE_NAME,  ESCAPETIME_NAME,  EXEC_NAME,
-  EXIT_NAME,  FASTGUI_NAME,  FIND_NAME,  FINDREGEXP_NAME,
-  FLAGS_NAME,  FLASH_NAME,  FREEFORM_NAME,  GOTOBOOKMARK_NAME,
-  GOTOCOLUMN_NAME,  GOTOLINE_NAME,  GOTOMARK_NAME,  HELP_NAME,
-  HEXCODE_NAME,  INSERT_NAME,  INSERTCHAR_NAME,  INSERTLINE_NAME,
-  INSERTSTRING_NAME,  LINEDOWN_NAME,  LINEUP_NAME,  LOADAUTOPREFS_NAME,
-  LOADPREFS_NAME,  MACRO_NAME,  MARK_NAME,  MARKVERT_NAME,
-  MATCHBRACKET_NAME,  MODIFIED_NAME,  MOVEBOS_NAME,  MOVEEOF_NAME,
-  MOVEEOL_NAME,  MOVEEOW_NAME,  MOVEINCDOWN_NAME,  MOVEINCUP_NAME,
-  MOVELEFT_NAME,  MOVERIGHT_NAME,  MOVESOF_NAME,  MOVESOL_NAME,
-  MOVETOS_NAME,  NEWDOC_NAME,  NEXTDOC_NAME,  NEXTPAGE_NAME,
-  NEXTWORD_NAME,  NOFILEREQ_NAME,  NOP_NAME,  OPEN_NAME,
-  OPENCLIP_NAME,  OPENMACRO_NAME,  OPENNEW_NAME,  PAGEDOWN_NAME,
-  PAGEUP_NAME,  PARAGRAPH_NAME,  PASTE_NAME,  PASTEVERT_NAME,
-  PLAY_NAME,  POPPREFS_NAME,  PRESERVECR_NAME,  PREVDOC_NAME,
-  PREVPAGE_NAME,  PREVWORD_NAME,  PUSHPREFS_NAME,  QUIT_NAME,
-  READONLY_NAME,  RECORD_NAME,  REDO_NAME,  REFRESH_NAME,
-  REPEATLAST_NAME,  REPLACE_NAME,  REPLACEALL_NAME,  REPLACEONCE_NAME,
-  RIGHTMARGIN_NAME,  SAVE_NAME,  SAVEAS_NAME,  SAVEAUTOPREFS_NAME,
-  SAVECLIP_NAME,  SAVEDEFPREFS_NAME,  SAVEMACRO_NAME,  SAVEPREFS_NAME,
-  SEARCHBACK_NAME,  SELECTDOC_NAME,  SETBOOKMARK_NAME,  STATUSBAR_NAME,
-  SUSPEND_NAME,  SYSTEM_NAME,  TABSIZE_NAME,  THROUGH_NAME,
-  TOGGLESEOF_NAME,  TOGGLESEOL_NAME,  TOLOWER_NAME,  TOUPPER_NAME,
-  TURBO_NAME,  UNDELLINE_NAME,  UNDO_NAME,  UNLOADMACROS_NAME,
-  UNSETBOOKMARK_NAME,  UTF8_NAME,  UTF8AUTO_NAME,  UTF8IO_NAME,
-  VERBOSEMACROS_NAME,  WORDWRAP_NAME,  NULL
+  ABOUT_NAME,  ADJUSTVIEW_NAME,  ALERT_NAME,  AUTOINDENT_NAME,
+  AUTOPREFS_NAME,  BACKSPACE_NAME,  BEEP_NAME,  BINARY_NAME,
+  CAPITALIZE_NAME,  CASESEARCH_NAME,  CENTER_NAME,  CLEAR_NAME,
+  CLIPNUMBER_NAME,  CLOSEDOC_NAME,  COPY_NAME,  CRLF_NAME,
+  CUT_NAME,  DELETECHAR_NAME,  DELETEEOL_NAME,  DELETELINE_NAME,
+  DOUNDO_NAME,  ERASE_NAME,  ESCAPE_NAME,  ESCAPETIME_NAME,
+  EXEC_NAME,  EXIT_NAME,  FASTGUI_NAME,  FIND_NAME,
+  FINDREGEXP_NAME,  FLAGS_NAME,  FLASH_NAME,  FREEFORM_NAME,
+  GOTOBOOKMARK_NAME,  GOTOCOLUMN_NAME,  GOTOLINE_NAME,  GOTOMARK_NAME,
+  HELP_NAME,  HEXCODE_NAME,  INSERT_NAME,  INSERTCHAR_NAME,
+  INSERTLINE_NAME,  INSERTSTRING_NAME,  LINEDOWN_NAME,  LINEUP_NAME,
+  LOADAUTOPREFS_NAME,  LOADPREFS_NAME,  MACRO_NAME,  MARK_NAME,
+  MARKVERT_NAME,  MATCHBRACKET_NAME,  MODIFIED_NAME,  MOVEBOS_NAME,
+  MOVEEOF_NAME,  MOVEEOL_NAME,  MOVEEOW_NAME,  MOVEINCDOWN_NAME,
+  MOVEINCUP_NAME,  MOVELEFT_NAME,  MOVERIGHT_NAME,  MOVESOF_NAME,
+  MOVESOL_NAME,  MOVETOS_NAME,  NEWDOC_NAME,  NEXTDOC_NAME,
+  NEXTPAGE_NAME,  NEXTWORD_NAME,  NOFILEREQ_NAME,  NOP_NAME,
+  OPEN_NAME,  OPENCLIP_NAME,  OPENMACRO_NAME,  OPENNEW_NAME,
+  PAGEDOWN_NAME,  PAGEUP_NAME,  PARAGRAPH_NAME,  PASTE_NAME,
+  PASTEVERT_NAME,  PLAY_NAME,  POPPREFS_NAME,  PRESERVECR_NAME,
+  PREVDOC_NAME,  PREVPAGE_NAME,  PREVWORD_NAME,  PUSHPREFS_NAME,
+  QUIT_NAME,  READONLY_NAME,  RECORD_NAME,  REDO_NAME,
+  REFRESH_NAME,  REPEATLAST_NAME,  REPLACE_NAME,  REPLACEALL_NAME,
+  REPLACEONCE_NAME,  RIGHTMARGIN_NAME,  SAVE_NAME,  SAVEAS_NAME,
+  SAVEAUTOPREFS_NAME,  SAVECLIP_NAME,  SAVEDEFPREFS_NAME,  SAVEMACRO_NAME,
+  SAVEPREFS_NAME,  SEARCHBACK_NAME,  SELECTDOC_NAME,  SETBOOKMARK_NAME,
+  STATUSBAR_NAME,  SUSPEND_NAME,  SYNTAX_NAME,  SYSTEM_NAME,
+  TABSIZE_NAME,  THROUGH_NAME,  TOGGLESEOF_NAME,  TOGGLESEOL_NAME,
+  TOLOWER_NAME,  TOUPPER_NAME,  TURBO_NAME,  UNDELLINE_NAME,
+  UNDO_NAME,  UNLOADMACROS_NAME,  UNSETBOOKMARK_NAME,  UTF8_NAME,
+  UTF8AUTO_NAME,  UTF8IO_NAME,  VERBOSEMACROS_NAME,  VISUALBELL_NAME,
+  WORDWRAP_NAME,  NULL
 };
