@@ -27,6 +27,9 @@ extern const char ABOUT_ABBREV[];
 extern const char ADJUSTVIEW_NAME[];
 extern const char ADJUSTVIEW_ABBREV[];
 
+extern const char ALERT_NAME[];
+extern const char ALERT_ABBREV[];
+
 extern const char AUTOINDENT_NAME[];
 extern const char AUTOINDENT_ABBREV[];
 
@@ -336,6 +339,9 @@ extern const char STATUSBAR_ABBREV[];
 extern const char SUSPEND_NAME[];
 extern const char SUSPEND_ABBREV[];
 
+extern const char SYNTAX_NAME[];
+extern const char SYNTAX_ABBREV[];
+
 extern const char SYSTEM_NAME[];
 extern const char SYSTEM_ABBREV[];
 
@@ -384,6 +390,9 @@ extern const char UTF8IO_ABBREV[];
 extern const char VERBOSEMACROS_NAME[];
 extern const char VERBOSEMACROS_ABBREV[];
 
+extern const char VISUALBELL_NAME[];
+extern const char VISUALBELL_ABBREV[];
+
 extern const char WORDWRAP_NAME[];
 extern const char WORDWRAP_ABBREV[];
 
