@@ -1,7 +1,7 @@
 /* Navigation functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -41,9 +41,10 @@ in an obvious way. (Yeah, right.) */
 You stand absolutely no chance of understanding this code if you aren't intimately
 familiar with this diagram.      
 
-          |<- cur_pos ------->|
+          |<- cur_pos (in bytes)-->|
+          |<- cur_char (in chars)->|
 
-                    |< cur_x >|
+                    |< cur_x >-----|
           |< win_x >|
 
  ---      +----------------------------------------------+ ---      -----
@@ -55,7 +56,7 @@ familiar with this diagram.
     cur_y |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
-     ---  |         |         @ <-Cursor      |          | ---        |      ne_lines   
+     ---  |         |              @ <-Cursor |          | ---        |      ne_lines   
           |         |                         |          |          num_lines |
           |         |                         |          |            |       |
           |         |                         |          |            |       |
@@ -81,28 +82,28 @@ familiar with this diagram.
 	which is of course unacceptable. Note that if x_wanted is TRUE, then the
 	wanted_x position is used rather tham cur_x+win_x. */
 
-static void resync_pos(buffer * const b) {
+void resync_pos(buffer * const b) {
 
 	line_desc *ld = b->cur_line_desc;
-
-	int pos, width, last_char_width, x = b->win_x + b->cur_x;
+	int pos, i, width, last_char_width, x = b->win_x + b->cur_x;
 
 	if (b->x_wanted) x = b->wanted_x;
 
 	assert(b->opt.tab_size < ne_columns / 2);
 
 	if (x == 0) {
-		b->cur_pos = 0;
+		b->cur_pos = b->cur_char = 0;
 		return;
 	}
 
-	for(pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding)) {
+	for(i = pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding), i++) {
 
 		if (ld->line[pos] != '\t') width += (last_char_width = get_char_width(&ld->line[pos], b->encoding));
 		else width += (last_char_width = b->opt.tab_size - width % b->opt.tab_size);
 
 		if (width == x) {
 			b->cur_pos = pos + (b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1);
+			b->cur_char = i + 1;
 			if (b->x_wanted) {
 
 				b->x_wanted = 0;
@@ -121,7 +122,7 @@ static void resync_pos(buffer * const b) {
 		if (width > x) {
 
 			b->cur_pos = pos;
-
+			b->cur_char = i;
 			width -= last_char_width;
 			b->x_wanted = 1;
 			b->wanted_x = x;
@@ -149,6 +150,7 @@ static void resync_pos(buffer * const b) {
 
 	if (b->opt.free_form) {
 		b->cur_pos = ld->line_len + x - width;
+		b->cur_char = i + x - width;
 		b->cur_x = x - b->win_x;
 		b->x_wanted = 0;
 	}
@@ -165,18 +167,22 @@ int line_up(buffer * const b) {
 
 	b->y_wanted = 0;
 	if (b->cur_y > 0) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		b->cur_y--;
 		b->cur_line--;
 		b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
+		b->attr_len = -1;
 		resync_pos(b);
 		return OK;
 	}
 	else {
 		if (b->win_y > 0) {
+			update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 			b->win_y--;
 			b->cur_line--;
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
+			b->attr_len = -1;
 			if (b == cur_buffer) scroll_window(b, 0, 1);
 			resync_pos(b);
 			return OK;
@@ -191,18 +197,22 @@ int line_down(buffer * const b) {
 
 	b->y_wanted = 0;
 	if (b->cur_y < ne_lines - 2 && b->cur_line < b->num_lines - 1) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		b->cur_y++;
 		b->cur_line++;
+		b->attr_len = -1;
 		b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 		resync_pos(b);
 		return OK;
 	}
 	else {
 		if (b->win_y < b->num_lines - ne_lines + 1) {
+			update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 			b->win_y++;
 			b->cur_line++;
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
+			b->attr_len = -1;
 			if (b == cur_buffer) scroll_window(b, 0, -1);
 			resync_pos(b);
 			return OK;
@@ -216,26 +226,23 @@ int line_down(buffer * const b) {
  * were here.
  */
  
-void keep_cursor_on_screen(buffer * const b)
-  {
-    if ( b->cur_y > ne_lines - 2 )
-        {
-          while( b->cur_y > ne_lines - 2 )
-            {
-              b->cur_y--;
-              b->win_y++;
-              b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
-            }  
-          assert(b->win_y = b->cur_line - b->cur_y);
-          b->y_wanted = FALSE;
-          
-        }
-    while ( b->cur_x >= ne_columns  )
-        {
-          b->win_x += b->opt.tab_size; 
-          b->cur_x -= b->opt.tab_size;
-        }
-  }
+void keep_cursor_on_screen(buffer * const b) {
+	if (b->cur_y > ne_lines - 2) {
+		while(b->cur_y > ne_lines - 2) {
+			b->cur_y--;
+			b->win_y++;
+			b->attr_len = -1;
+			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
+		}  
+		assert(b->win_y = b->cur_line - b->cur_y);
+		b->y_wanted = FALSE;
+	}
+
+	while(b->cur_x >= ne_columns) {
+		b->win_x += b->opt.tab_size; 
+		b->cur_x -= b->opt.tab_size;
+	}
+}
 
 
 
@@ -278,6 +285,7 @@ int char_left(buffer * const b) {
 
 		/* If the buffer is UTF-8 encoded, we move back until we find a sequence initiator. */
 		b->cur_pos = b->cur_pos > ld->line_len ? b->cur_pos - 1 : prev_pos(ld->line, b->cur_pos, b->encoding);
+		b->cur_char--;
 		return OK;
 	}
 	else if (b->cur_line > 0) {
@@ -324,6 +332,7 @@ int char_right(buffer * const b) {
 
 	b->cur_x += disp;
 	b->cur_pos = b->cur_pos >= ld->line_len ? b->cur_pos + 1 : next_pos(ld->line, b->cur_pos, b->encoding);
+	b->cur_char++;
 
 	/* If the current x position would be beyond the right screen margin, or if the same happens
 		for the character we are currently over, we shift the screen to the right. */
@@ -341,15 +350,20 @@ int prev_page(buffer * const b) {
 	b->y_wanted = 0;
 
 	if (b->cur_y > 0) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		b->cur_line -= b->cur_y;
 		b->cur_y = 0;
 		b->cur_line_desc = b->top_line_desc;
+		b->attr_len = -1;
 		resync_pos(b);
 		return OK;
 	}
 
 	if (b->win_y == 0) return ERROR;
 
+	update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	b->attr_len = -1;
+
 	if ((b->win_y -= ne_lines - 2)<0) b->win_y = 0;
 
 	ld_top = b->top_line_desc;
@@ -378,6 +392,7 @@ int next_page(buffer * const b) {
 	b->y_wanted = 0;
 
 	if (b->cur_y < ne_lines - 2) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
 			for(i = 0; i < ne_lines - 2 && ld_cur->ld_node.next->next; i++)
@@ -392,6 +407,7 @@ int next_page(buffer * const b) {
 			for(i = 0; i < ne_lines - 2; i++)
 			  ld_cur = (line_desc *)ld_cur->ld_node.next;
 		}
+		b->attr_len = -1;
 		b->cur_line_desc = ld_cur;
 		resync_pos(b);
 		return OK;
@@ -399,6 +415,9 @@ int next_page(buffer * const b) {
 
 	if (b->win_y >= b->num_lines - (ne_lines - 1)) return ERROR;
 
+	update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	b->attr_len = -1;
+
 	disp = ne_lines - 2;
 
 	if (b->win_y + disp > b->num_lines - (ne_lines - 1))
@@ -430,39 +449,38 @@ int page_up(buffer * const b) {
 	disp = ne_lines - 2;
 
 	/* Already on the top line? */
-	if ( b->cur_line == 0 ) return OK;
+	if (b->cur_line == 0) return OK;
 	
-	if (  !b->y_wanted )
-		{
-			b->y_wanted     = TRUE;
-			b->wanted_y     = b->cur_line;
-			b->wanted_cur_y = b->cur_y;
-	   }
+	update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	b->attr_len = -1;
 
-	for (i = 0; i < disp; i++)
-   	{
-			b->wanted_y--;                        /* We want to move up */
+	if (!b->y_wanted) {
+		b->y_wanted = TRUE;
+		b->wanted_y = b->cur_line;
+		b->wanted_cur_y = b->cur_y;
+   }
+
+	for (i = 0; i < disp; i++) {
+		b->wanted_y--; /* We want to move up */
 			
-			/* Can we move up? */
-			if (     b->wanted_y >= 0             /* We aren't yet off the top    */
-			     &&  b->wanted_y < b->num_lines - 1) /* we aren't still past the end */
-				{
-					b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
-					b->cur_line--;
-				}
-		   
-		   /* Should we shift the view up? */
-			if (    b->win_y > 0                               /* We aren't already at the top */
-			     && b->win_y + b->wanted_cur_y > b->wanted_y)  /* Gap between virtual cursor and TOS is to small */
-				{
-					b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
-					b->win_y--;
-			 	}
+		/* Can we move up? */
+		if (b->wanted_y >= 0 /* We aren't yet off the top    */
+				&& b->wanted_y < b->num_lines - 1) { /* we aren't still past the end */
+			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
+			b->cur_line--;
 		}
+		   
+	   /* Should we shift the view up? */
+		if (b->win_y > 0 /* We aren't already at the top */
+				&& b->win_y + b->wanted_cur_y > b->wanted_y) { /* Gap between virtual cursor and TOS is to small */
+			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
+			b->win_y--;
+	 	}
+	}
 
 	b->cur_y = b->cur_line - b->win_y;
 
-	keep_cursor_on_screen( b );
+	keep_cursor_on_screen(b);
 	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
 	return OK;
@@ -475,41 +493,40 @@ int page_down(buffer * const b) {
 	disp = ne_lines - 2;
 
 	/* Already on the bottom line? */
-	if ( b->cur_line == b->num_lines - 1 ) return OK;
+	if (b->cur_line == b->num_lines - 1) return OK;
 	
-	if ( !b->y_wanted )
-		{
-			b->y_wanted     = TRUE;
-			b->wanted_y     = b->cur_line;
-			b->wanted_cur_y = b->cur_y;
-	   }
+	update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	b->attr_len = -1;
 
-   shift_view = ( b->win_y + disp < b->num_lines ); /* can't already see the last line */
+	if (!b->y_wanted) {
+		b->y_wanted = TRUE;
+		b->wanted_y = b->cur_line;
+		b->wanted_cur_y = b->cur_y;
+   }
+
+   shift_view = (b->win_y + disp < b->num_lines); /* can't already see the last line */
    
-	for (i = 0; i < disp; i++)
-   	{
-			b->wanted_y++;                        /* We want to move down */
+	for (i = 0; i < disp; i++) {
+		b->wanted_y++; /* We want to move down */
 			
-			/* Can we move down? */
-			if (     b->wanted_y > 0              /* We aren't still above the top  */
-			     &&  b->wanted_y < b->num_lines )  /* we aren't yet to the end       */
-				{
-					b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
-					b->cur_line++;
-				}
-		   
-		   /* Should we shift the view down? */
-			if (    shift_view                                  /* already decided we should            */
-			     && b->wanted_y - b->wanted_cur_y >  b->win_y ) /* Gap between virtual cursor and TOS is to big */
-				{
-					b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
-					b->win_y++;
-			 	}
+		/* Can we move down? */
+		if (b->wanted_y > 0 /* We aren't still above the top  */
+				&&  b->wanted_y < b->num_lines) { /* we aren't yet to the end */
+			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
+			b->cur_line++;
 		}
+		   
+	   /* Should we shift the view down? */
+		if (shift_view /* already decided we should */
+				&& b->wanted_y - b->wanted_cur_y >  b->win_y) { /* Gap between virtual cursor and TOS is to big */
+			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
+			b->win_y++;
+	 	}
+	}
 
 	b->cur_y = b->cur_line - b->win_y;
 
-	keep_cursor_on_screen( b );
+	keep_cursor_on_screen(b);
 	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
 	return OK;
@@ -520,9 +537,11 @@ int move_tos(buffer * const b) {
 	b->y_wanted = 0;
 
 	if (b->cur_y > 0) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		b->cur_line -= b->cur_y;
 		b->cur_y = 0;
 		b->cur_line_desc = b->top_line_desc;
+		b->attr_len = -1;
 		resync_pos(b);
 	}
 	return OK;
@@ -535,6 +554,8 @@ int move_bos(buffer * const b) {
 	b->y_wanted = 0;
 
 	if (b->cur_y < ne_lines - 2) {
+		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+		b->attr_len = -1;
 		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
 			for(i = 0; i < ne_lines - 2  &&   ld_cur->ld_node.next->next; i++)
@@ -559,109 +580,97 @@ int move_bos(buffer * const b) {
    win_x, cur_x, win_y and cur_y -- the variables which control which part
    of the file is visible in the terminal window. */
 
-int adjust_view(buffer * const b, const unsigned char *p)
-	{
-		int i, disp, rc = OK;
+int adjust_view(buffer * const b, const unsigned char *p) {
+	int i, disp, rc = OK;
 
-		b->y_wanted = 0;
+	b->y_wanted = 0;
 
-		if ( !p ) p = "t";
-		  
-		while(*p)
-			{
-	         disp = 0;
-				switch ( *p )
-					{
-						case 't' :
-						case 'T' :
-							/* Shift the view so that the current line is displayed at the top. */
-							disp = -b->cur_y;
-							break;
+	if (!p) p = "t";
+	  
+	while(*p) {
+		disp = 0;
+		switch (*p) {
+			case 't' :
+			case 'T' :
+				/* Shift the view so that the current line is displayed at the top. */
+				disp = -b->cur_y;
+				break;
 
-						case 'c' :
-						case 'C' :
-							/* Shift the view so that the current line is displayed at the center. */
-							disp = (ne_lines - 2) / 2 - b->cur_y;
-							break;
+			case 'c' :
+			case 'C' :
+				/* Shift the view so that the current line is displayed at the center. */
+				disp = (ne_lines - 2) / 2 - b->cur_y;
+				break;
 
-						case 'b' :
-						case 'B' :
-							/* Shift the view so that the current line is displayed at the bottom. */
-							disp = (ne_lines - 2) - b->cur_y;
-							break;
+			case 'b' :
+			case 'B' :
+				/* Shift the view so that the current line is displayed at the bottom. */
+				disp = (ne_lines - 2) - b->cur_y;
+				break;
 
-						case 'l' :
-						case 'L' :
-							/* Shift the view as far left as possible. */
-							while ( b->cur_x >= b->opt.tab_size )
-								{
-									b->win_x += b->opt.tab_size;
-									b->cur_x -= b->opt.tab_size;
-								}
-							break;
+			case 'l' :
+			case 'L' :
+				/* Shift the view as far left as possible. */
+				while (b->cur_x >= b->opt.tab_size) {
+					b->win_x += b->opt.tab_size;
+					b->cur_x -= b->opt.tab_size;
+				}
+				break;
 
-						case 'm' :
-						case 'M' :
-						   /* Shift the view as far left as possible. This way we don't have to
-						      deal with figuring out which side of Middle the view started on. */
-							while ( b->cur_x >= b->opt.tab_size )
-								{
-									b->win_x += b->opt.tab_size;
-									b->cur_x -= b->opt.tab_size;
-								}
-							/* Since we now know that the cursor is left of center, we can start
-							   to shift the view right until the cursor is centered or until we
-							   run out of text to shift right. */
-							while ( b->cur_x < (ne_columns / 2) - (ne_columns / 2) % b->opt.tab_size &&
-							        b->win_x >= b->opt.tab_size                      )
-								{
-									b->win_x -= b->opt.tab_size;
-									b->cur_x += b->opt.tab_size;
-								}
-							break;
+			case 'm' :
+			case 'M' :
+			   /* Shift the view as far left as possible. This way we don't have to
+			      deal with figuring out which side of Middle the view started on. */
+				while (b->cur_x >= b->opt.tab_size) {
+					b->win_x += b->opt.tab_size;
+					b->cur_x -= b->opt.tab_size;
+				}
+				/* Since we now know that the cursor is left of center, we can start
+				   to shift the view right until the cursor is centered or until we
+				   run out of text to shift right. */
+				while (b->cur_x < (ne_columns / 2) - (ne_columns / 2) % b->opt.tab_size && b->win_x >= b->opt.tab_size) {
+					b->win_x -= b->opt.tab_size;
+					b->cur_x += b->opt.tab_size;
+				}
+				break;
 
-						case 'r' :
-						case 'R' :
-							/* Shift the view as far right as possible. */
-							while ( b->cur_x < ne_columns - ne_columns % b->opt.tab_size &&
-							        b->win_x >= b->opt.tab_size                      )
-								{
-									b->win_x -= b->opt.tab_size;
-									b->cur_x += b->opt.tab_size;
-								}
-							break;
+			case 'r' :
+			case 'R' :
+				/* Shift the view as far right as possible. */
+				while (b->cur_x < ne_columns - ne_columns % b->opt.tab_size && b->win_x >= b->opt.tab_size) {
+					b->win_x -= b->opt.tab_size;
+					b->cur_x += b->opt.tab_size;
+				}
+				break;
 
-						default  :
-							/* When we hit a character we don't recognize, we set the rc, but
-							   we still process other valid view displacements. */
-							rc = ERROR;
-							break;
-					}
-				if ( disp > 0 )
-						{
-							for(i = 0; i < disp && b->top_line_desc->ld_node.prev->prev; i++)
-								{
-									b->win_y--;
-									b->cur_y++;
-									b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
-								}
-						}
-					else if ( disp < 0 )
-						{
-							for(i = 0; i > disp && b->top_line_desc->ld_node.next->next; i--)
-								{
-									b->win_y++;
-									b->cur_y--;
-									b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
-								}
-						}
-				p++;
+			default  :
+				/* When we hit a character we don't recognize, we set the rc, but
+				   we still process other valid view displacements. */
+				rc = ERROR;
+				break;
+		}
+		if (disp > 0) {
+			for(i = 0; i < disp && b->top_line_desc->ld_node.prev->prev; i++) {
+				b->win_y--;
+				b->cur_y++;
+				b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
 			}
-		if (b == cur_buffer) update_window(b);
-		resync_pos(b);
-		return rc;
+		}
+		else if (disp < 0) {
+			for(i = 0; i > disp && b->top_line_desc->ld_node.next->next; i--) {
+				b->win_y++;
+				b->cur_y--;
+				b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
+			}
+		}
+		p++;
 	}
 
+	if (b == cur_buffer) update_window(b);
+	resync_pos(b);
+	return rc;
+}
+
 
 
 void goto_line(buffer * const b, const int n) {
@@ -673,6 +682,8 @@ void goto_line(buffer * const b, const int n) {
 	if (n >= b->num_lines || n == b->cur_line) return;
 
 	if (n >= b->win_y && n < b->win_y + ne_lines - 1) {
+		update_syntax_states(b, -1, b->cur_line_desc, NULL);
+		b->attr_len = -1;
 		b->cur_y = n - b->win_y;
 		b->cur_line = n;
 		ld = b->top_line_desc;
@@ -682,6 +693,9 @@ void goto_line(buffer * const b, const int n) {
 		return;
 	}
 
+	update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	b->attr_len = -1;
+
 	b->win_y = n - (ne_lines - 1) / 2;
 
 	if (b->win_y > b->num_lines - (ne_lines - 1)) b->win_y = b->num_lines - (ne_lines - 1);
@@ -736,7 +750,6 @@ void goto_column(buffer * const b, const int n) {
 a character offset) instead. */
 
 void goto_pos(buffer * const b, const int pos) {
-
 	goto_column(b, calc_width(b->cur_line_desc, pos, b->opt.tab_size, b->encoding));
 }
 
@@ -752,14 +765,15 @@ void move_to_sol(buffer * const b) {
 	t = b->win_x;
 	b->win_x =
 	b->cur_x =
-	b->cur_pos = 0;
+	b->cur_pos = 
+	b->cur_char = 0;
 
 	if (t &&  b == cur_buffer) update_window(b);
 }
 
 
 void move_to_eol(buffer * const b) {
-	int i, len, total_len;
+	int i, pos, width, total_width;
 	line_desc *ld = b->cur_line_desc;
 
 	assert(ld->ld_node.next != NULL);
@@ -773,23 +787,26 @@ void move_to_eol(buffer * const b) {
 		return;
 	}
 
-	total_len = calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding);
+	total_width = calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding);
 
-	if (total_len >= b->win_x && total_len < b->win_x + ne_columns) {
-		b->cur_x = total_len - b->win_x;
+	if (total_width >= b->win_x && total_width < b->win_x + ne_columns) {
+		/* We move to a visible position. */
+		b->cur_x = total_width - b->win_x;
 		b->cur_pos = ld->line_len;
+		b->cur_char = calc_char_len(ld, b->encoding);
 		return;
 	}
 
-	for(len = 0, i = 0; i < ld->line_len; i++)  {
-		if (ld->line[i] != '\t') len++;
-		else len += b->opt.tab_size - len % b->opt.tab_size;
+	for(i = pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding), i++)  {
+		if (ld->line[pos] != '\t') width += get_char_width(&ld->line[pos], b->encoding);
+		else width += b->opt.tab_size - width % b->opt.tab_size;
 
-		if (total_len - len < ne_columns - b->opt.tab_size) {
+		if (total_width - width < ne_columns - b->opt.tab_size) {
 			int t = b->win_x;
-			b->win_x = len - len % b->opt.tab_size;
-			b->cur_x = total_len - b->win_x;
+			b->win_x = width - width % b->opt.tab_size;
+			b->cur_x = total_width - b->win_x;
 			b->cur_pos = ld->line_len;
+			b->cur_char = calc_char_len(ld, b->encoding);
 			if (t != b->win_x) update_window(b);
 			return;
 		}
@@ -811,20 +828,22 @@ void reset_position_to_sof(buffer * const b) {
 	b->cur_x =
 	b->cur_y =
 	b->cur_line =
-	b->cur_pos = 0;
+	b->cur_pos =
+	b->cur_char = 0;
+	b->attr_len = -1;
 	b->cur_line_desc = b->top_line_desc = (line_desc *)b->line_desc_list.head;
 }
 
 
 
 void move_to_sof(buffer * const b) {
+	int moving = b->win_x || b->win_y;
 
-	int old_win_x = b->win_x, old_win_y = b->win_y;
+	if (moving) update_syntax_states(b, -1, b->cur_line_desc, NULL);
+	else update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 
 	reset_position_to_sof(b);
-
-	if (old_win_x != b->win_x || old_win_y != b->win_y)
-		if (b == cur_buffer) update_window(b);
+	if (moving && b == cur_buffer) update_window(b);
 }
 
 
@@ -840,11 +859,20 @@ void move_to_bof(buffer * const b) {
 	for(i = 0; i < ne_lines - 2 && ld->ld_node.prev->prev; i++) ld = (line_desc *)ld->ld_node.prev;
 
 	b->cur_line = b->num_lines - 1;
+	b->win_x = 0;
+	b->win_y = ld->ld_node.prev->prev ? b->num_lines - (ne_lines - 1) : b->num_lines - 1;
+
+	if (old_win_x != b->win_x || old_win_y != b->win_y) {
+		update_syntax_states(b, -1, b->cur_line_desc, NULL);
+		if (b == cur_buffer) reset_window();
+	}
+	else update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+	b->attr_len = -1;
 
 	if (!ld->ld_node.prev->prev) {
-		b->win_x =
 		b->win_y =
 		b->cur_x =
+		b->cur_char =
 		b->cur_pos = 0;
 		b->cur_y = b->num_lines - 1;
 		b->top_line_desc = (line_desc *)b->line_desc_list.head;
@@ -853,15 +881,12 @@ void move_to_bof(buffer * const b) {
 	else {
 		b->win_x =
 		b->cur_x =
+		b->cur_char =
 		b->cur_pos = 0;
-		b->win_y = b->num_lines - (ne_lines - 1);
 		b->cur_y = ne_lines - 2;
 		b->top_line_desc = ld;
 		b->cur_line_desc = (line_desc *)b->line_desc_list.tail_pred;
 	}
-
-	if (old_win_x != b->win_x || old_win_y != b->win_y)
-		if (b == cur_buffer) update_window(b);
 }
 
 
