@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -22,21 +22,22 @@
 
 
 #include "ne.h"
+
 #include "version.h"
 #include "termchar.h"
 #include "regex.h"
+
 #include <signal.h>
 #include <limits.h>
 #include <locale.h>
 
-
 /* This is the array containing the "NO WARRANTY" message, which is displayed
 when ne is called without any specific file name or macro to execute. The
 message disappears as soon as any key is typed. */
 
 char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify",
 										"it under the terms of the GNU General Public License as published by",
@@ -58,13 +59,15 @@ char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 
 char ARG_HELP[] = ABOUT_MSG "\n"
 						"Usage: ne [options] [files]\n"
-						"--help        prints this message.\n"
+						"--help        print this message.\n"
+						"--            next token is a filename.\n"
+						"+[N]          move to the N-th (or the last, if N is missing) line of the first file.\n"
 						"--utf8        use UTF-8 I/O.\n"
 						"--no-utf8     do not use UTF-8 I/O.\n"
 						"--ansi        use built-in ANSI control sequences.\n"
 						"--no-ansi     do not use built-in ANSI control sequences.\n"
-						"--            next token is a filename.\n"
 						"--no-config   do not read configuration files.\n"
+						"--no-syntax   disable syntax-highlighting support.\n"
 						"--keys FILE   use this file for keyboard configuration.\n"
 						"--menus FILE  use this file for menu configuration.\n"
 						"--macro FILE  exec this macro after start.\n";
@@ -83,6 +86,7 @@ list macros = { (node *)&macros.tail, NULL, (node *)&macros.head };
 
 buffer *cur_buffer;
 int turbo;
+int do_syntax = TRUE;
 
 /* These function live here because they access cur_buffer. new_buffer()
 creates a new buffer, adds it to the buffer list, and assign it to
@@ -94,8 +98,13 @@ buffer *new_buffer(void) {
 
 	if (b) {
 		clear_buffer(b);
-		if (cur_buffer) add(&b->b_node, &cur_buffer->b_node);
-		else add_head(&buffers, &b->b_node);
+		if (cur_buffer) 
+			add(&b->b_node, &cur_buffer->b_node);
+		else 
+			add_head(&buffers, &b->b_node);
+		
+		
+		
 		cur_buffer = b;
 	}
 
@@ -129,14 +138,15 @@ some terminal and signal initialization functions, and entering the
 event loop. */
 
 int main(int argc, char **argv) {
-
+	
 	input_class ic;
-	int i, c, no_config = FALSE, displaying_info = FALSE;
+	int i, c, no_config = FALSE, displaying_info = FALSE, first_line = 0;
 	char *macro_name = NULL, *key_bindings_name = NULL, *menu_conf_name = NULL;
+
 	clip_desc *cd;
 	char *skiplist, *locale;
 
-	if ( !(skiplist = calloc( argc, 1 )))  /* We need this many flags. */
+	if (!(skiplist = calloc(argc, 1)))  /* We need this many flags. */
 	  exit(1);                             /* This would be bad.       */
 
 	locale = setlocale(LC_ALL, "");
@@ -161,7 +171,8 @@ int main(int argc, char **argv) {
 
 		/* Special arguments start with two dashes. If we find one, we
 		cancel its entry in argv[], so that it will be skipped when opening
-		the specified files. */
+		the specified files. The only exception is +N for skipping to the
+		N-th line. */
 
 		if (argv[i][0] == '-' && argv[i][1] == '-') {
 			if (!argv[i][2]) skiplist[i++] = 1; /* You can use "--" to force the next token to be a filename */
@@ -177,6 +188,10 @@ int main(int argc, char **argv) {
 				ansi = FALSE;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
+			else if (!strcmp(&argv[i][2], "no-syntax")) {
+				do_syntax = FALSE;
+				skiplist[i] = 1; /* argv[i] = NULL; */
+			}
 			else if (!strcmp(&argv[i][2], "ansi")) {
 				ansi = TRUE;
 				skiplist[i] = 1; /* argv[i] = NULL; */
@@ -208,6 +223,11 @@ int main(int argc, char **argv) {
 				}
 			}
 		}
+		else if (argv[i][0] == '+') {
+			first_line = argv[i][1] ? strtol(argv[i]+1, NULL, 10) : INT_MAX;
+			if (errno || first_line < 0) first_line = 0;
+			skiplist[i] = 1; /* argv[i] = NULL; */
+		}
 	}
 
 	/* Unless --noconfig was specified, we try to configure the
@@ -282,7 +302,7 @@ int main(int argc, char **argv) {
 		stop = FALSE;
 
 		for(i = 1; i < argc && !stop; i++) {
-			if (argv[i] && !skiplist[i] ) {
+			if (argv[i] && !skiplist[i]) {
 				if (!first_file) do_action(cur_buffer, NEWDOC_A, -1, NULL);
 				else first_file = FALSE;
 
@@ -290,12 +310,13 @@ int main(int argc, char **argv) {
 			}
 		}
 
-		free( skiplist );
+		free(skiplist);
 
 		/* This call makes current the first specified file. It is called
 		only if more than one buffer exist. */
 
 		if (get_nth_buffer(1)) do_action(cur_buffer, NEXTDOC_A, -1, NULL);
+		if (first_line) do_action(cur_buffer, GOTOLINE_A, first_line, NULL);
 	}
 
 	/* Note that we do not need to update the display. clear_entire_screen() is
@@ -315,6 +336,12 @@ int main(int argc, char **argv) {
 			move_cursor(i, 0);
 			output_string(NO_WARRANTY_msg[i], FALSE);
 		}
+		move_cursor(++i,0);
+		if (exists_gprefs_dir()) {
+			output_string("Global Directory: ", FALSE);
+			output_string(exists_gprefs_dir(), FALSE);
+		}
+		else output_string("Global directory not found!", FALSE);
 	}
 
 	while(TRUE) {
@@ -331,7 +358,7 @@ int main(int argc, char **argv) {
 		c = get_key_code();
 		ic = CHAR_CLASS(c);
 
-		if ( window_changed_size ) {
+		if (window_changed_size) {
 			print_error(do_action(cur_buffer, REFRESH_A, 0, NULL));
 			window_changed_size = FALSE;
 		}
@@ -339,7 +366,7 @@ int main(int argc, char **argv) {
 		if (displaying_info) {
 			displaying_info = FALSE;
 
-			for(i = 0; NO_WARRANTY_msg[i]; i++) {
+			for(i = 0; i < sizeof NO_WARRANTY_msg / sizeof *NO_WARRANTY_msg + 2; i++) {
 				move_cursor(i, 0);
 				clear_to_eol();
 			}
