@@ -1,7 +1,7 @@
-/* Main typdefs and defines.
+/* Main typedefs and defines.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -20,27 +20,37 @@
 	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	02111-1307, USA.  */
 
-
-#ifndef TERMCAP
-#include <curses.h>
-#include <term.h>
-#else
 #include <stdio.h>
-#include "info2cap.h"
-#endif
-
-#include "termchar.h"
-#include "utf8.h"
-
-#include <stddef.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
+
+#define PARAMS(protos) protos
+#include "syn_types.h"
+#include "syn_hash.h"
+#include "syn_regex.h"
+#include "syn_utf8.h"
+#include "syn_utils.h"
+
 #include <limits.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifndef TERMCAP
+#include <curses.h>
+#include <term.h>
+#else
+#include "info2cap.h"
+#endif
+
+#include <termios.h>
+
+
+#include "termchar.h"
+#include "utf8.h"
+
 #ifdef _AMIGA
 #define CURDIR ""
 #define _S_IFDIR S_IFDIR
@@ -51,7 +61,6 @@
 #else
 #define CURDIR "."
 #include <unistd.h>
-#include <termios.h>
 #endif
 
 #ifndef min
@@ -61,7 +70,6 @@
 #define   max(a,b)    ((a)>(b)?(a):(b))
 #endif
 
-
 /* Include the list of all possible actions that do_action() can execute.
 Note also that menu handling is governed by such a command (ESCAPE). */
 
@@ -75,27 +83,22 @@ typedef enum {
 	 ALPHA, COMMAND, RETURN, TAB, IGNORE, INVALID, INPUT_CLASS_COUNT
 } input_class;
 
-/* A buffer or clip at any given time may be marked as ASCII, UTF-8 or
-8-bit. This means, respectively, that it contains just bytes below 0x80, an
-UTF-8-encoded byte sequence or an arbitrary byte sequence.
-
-The attribution is lazy, but stable. This means that a buffer starts as ASCII,
-and becomes UTF-8 or 8-bit as soon as some insertion makes it necessary. This
-is useful to delay the encoding choice as much as possible. */
-
-typedef enum {
-	ENC_ASCII = 0, ENC_UTF8, ENC_8_BIT
-} encoding_type;
-
-
-/* The number of bookmarks per buffer. It comprises the automatic bookmark (0). */
-
-#define NUM_BOOKMARKS 11
-
 /* The name of the default preferences file name. */
 
 #define DEF_PREFS_NAME			".default"
 
+/* The name of the syntax subdirectory (global and local). */
+
+#define SYNTAX_DIR			"syntax"
+
+/* This is the extension of syntax definition files. */
+
+#define SYNTAX_EXT			".jsf"
+
+/* The name of the file containing the mappings from extensions to syntax names. */
+
+#define EXT_2_SYN				"ext2syn"
+
 /* This is the name taken by unnamed documents. */
 
 #define UNNAMED_NAME				"<unnamed>"
@@ -121,6 +124,21 @@ but possible. */
 
 #define MAX_INT_LEN				32
 
+/* A buffer or clip at any given time may be marked as ASCII, UTF-8 or
+8-bit. This means, respectively, that it contains just bytes below 0x80, an
+UTF-8-encoded byte sequence or an arbitrary byte sequence.
+
+The attribution is lazy, but stable. This means that a buffer starts as ASCII,
+and becomes UTF-8 or 8-bit as soon as some insertion makes it necessary. This
+is useful to delay the encoding choice as much as possible. */
+
+typedef enum {
+	ENC_ASCII = 0, ENC_UTF8, ENC_8_BIT
+} encoding_type;
+
+/* The number of bookmarks per buffer. It comprises the automatic bookmark (0). */
+
+#define NUM_BOOKMARKS 11
 
 /* These are the list and node structures used throughout ne. See the exec.c
 source for some elaborations on the subject. */
@@ -178,8 +196,18 @@ typedef struct {
 	node ld_node;
 	unsigned char *line;
 	int line_len;
+	HIGHLIGHT_STATE highlight_state;	/* Initial highlight state for this line */
 } line_desc;
 
+/* The purpose of this structure is to provide the byte count for allocating
+line descriptors when no syntax highlighting is required.  */
+
+typedef struct {
+	node ld_node;
+	unsigned char *line;
+	int line_len;
+} no_syntax_line_desc;
+
 #define assert_line_desc(ld, encoding) {if ((ld)) {		\
 	assert((ld)->line_len >= 0);\
 	assert(((ld)->line == NULL) == ((ld)->line_len == 0));\
@@ -344,7 +372,8 @@ typedef struct {
 		search_back:1,     /* Last search was backwards */
 		case_search:1,     /* Look at case matching in searches */
 		binary:1,          /* Load and save in binary mode */
-		utf8auto:1;        /* Try to detect automatically UTF-8 */
+		utf8auto:1,        /* Try to detect automatically UTF-8 */
+		visual_bell:1;     /* Prefer visible bell to audible */
   } options_t;
 
 #define assert_options(o) {if ((o)) {\
@@ -380,7 +409,9 @@ typedef struct {
 	int wanted_x;             /* desired x position modulo short lines, tabs, etc. Valid only if x_wanted is TRUE. */
 	int wanted_y;             /* desired y position modulo top or bottom of buffer. Valid if y_wanted is TRUE. */
 	int wanted_cur_y;         /* desired cur_y, valid if y_wanted is TRUE */
-	int cur_line, cur_pos;    /* position of cursor withing the doc buffer */
+	int cur_line;             /* the current line */
+	int cur_pos;              /* position of cursor within the document buffer (counts bytes) */
+	int cur_char;             /* position of cursor within the attribute buffer (counts characters) */
 	int num_lines;
 	int block_start_line, block_start_col;
 	int allocated_chars;
@@ -392,27 +423,32 @@ typedef struct {
 		int line;
 	} bookmark[NUM_BOOKMARKS];
 	int bookmark_mask;        /* bit N is set if bookmark[N] is set */
+	
+	struct high_syntax *syn;				/* Syntax loaded for this buffer. */
+	int *attr_buf;								/* If attr_len >= 0, a pointer to the list of *current* attributes of the *current* line. */ 
+	int attr_size;								/* attr_buf size. */						
+	int attr_len;								/* attr_buf valid number of characters, or -1 to denote that attr_buf is not valid. */						
+	HIGHLIGHT_STATE next_state;			/* If attr_len >= 0, the state after the *current* line. */						
 
 	unsigned int
 
-		link_undos,									/* Link the undo steps. Multilevel. */
+		link_undos,					/* Link the undo steps. Multilevel. */
 		is_modified:1, 					/* Buffer has been modified since last save */
-		recording:1,								/* We are recording a macro */
-		marking:1,									/* We are marking a block */
-		x_wanted:1,									/* We're not where we would like to be */
-		y_wanted:1,									/* We've been paging up/down */
-		exec_only_options:1,						/* Only option changes can be executed */
-		find_string_changed:1,					/* The search string has to be recompiled before use. */
-		last_was_replace:1,						/* The last search operation was a replace */
-		last_was_regexp:1,						/* The last search operation was done with regexps */
-		undoing:1,									/* We are currently undoing an action */
-		redoing:1,									/* We are currently redoing an action */
-		mark_is_vertical:1,						/* The current marking is vertical */
+		recording:1,					/* We are recording a macro */
+		marking:1,					/* We are marking a block */
+		x_wanted:1,					/* We're not where we would like to be */
+		y_wanted:1,					/* We've been paging up/down */
+		exec_only_options:1,				/* Only option changes can be executed */
+		find_string_changed:1,				/* The search string has to be recompiled before use. */
+		last_was_replace:1,				/* The last search operation was a replace */
+		last_was_regexp:1,				/* The last search operation was done with regexps */
+		undoing:1,					/* We are currently undoing an action */
+		redoing:1,					/* We are currently redoing an action */
+		mark_is_vertical:1,				/* The current marking is vertical */
 		executing_internal_macro:1,			/* We are currently executing the internal macro of the current buffer */
-		is_CRLF:1; 			                  /* Buffer should be saved with CR/LF terminators */
+		is_CRLF:1; 			                /* Buffer should be saved with CR/LF terminators */
 
-	options_t opt;                         /* These get pushed/popped on the prefs stack */
-	
+	options_t opt;                         			/* These get pushed/popped on the prefs stack */
 } buffer;
 
 
@@ -432,6 +468,18 @@ typedef struct {
 	assert_undo_buffer(&(b)->undo);\
 }}
 
+#define assert_buffer_content(b) {if ((b)) {\
+	line_desc *ld;\
+	ld = (line_desc *)b->line_desc_list.head;\
+	while(ld->ld_node.next) {\
+		assert_line_desc(ld, (b)->encoding);\
+		if ((b)->syn) assert(ld->highlight_state.state != -1);\
+		ld = (line_desc *)ld->ld_node.next;\
+	}\
+	assert(b->attr_len < 0 || b->attr_len == calc_char_len(b->cur_line_desc, b->encoding));\
+}}
+
+#include "syntax.h"
 
 extern const char *key_binding[];
 
@@ -445,18 +493,25 @@ extern list buffers, clips, macros;
 
 extern int turbo;
 
+/* If true, the current line has changed and care must be taken to update the initial state of the following lines. */
+
+extern int need_attr_update;
+
 /* If true, we want the hardwired ANSI terminal, not a real one. */
 
 extern int ansi;
 
+/* If true, we want syntax highlighting. */
+
+extern int do_syntax;
+
 /* This flag can be set anywhere to FALSE, and will become TRUE if the user
 hits the interrupt key (usually CTRL-'\'). It is handled through SIGQUIT and
 SIGINT. */
 
 extern unsigned int stop;
 
-/* This is set by the signal handler whenever a SIGWINCH happens.
-   Somebody ought to use this. */
+/* This is set by the signal handler whenever a SIGWINCH happens. */
 
 extern unsigned int window_changed_size;
 
@@ -481,7 +536,6 @@ extern const unsigned char ascii_up_case[];
 #include "utf8.h"
 #include "debug.h"
 
-
 #ifdef _AMIGA
 #define set_fatal_code()
 #define block_signals()
