@@ -1,7 +1,7 @@
 /* Preferences functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -39,6 +39,9 @@ be enough strange to avoid clashes with macros. */
 
 #define PREF_FILE_SUFFIX	"#ap"
 
+/* This string is appended to the filename extension. */
+
+#define SYNTAX_FILE_SUFFIX ".jsf"
 
 /* We suppose a configuration file won't be bigger than this. Having it
 bigger just causes a reallocation. */
@@ -56,7 +59,7 @@ const char *extension(const char * const filename) {
 
 	assert(filename != NULL);
 
-	for(i = strlen(filename) - 2; i >= 0; i--)
+	for(i = strlen(filename); i-- != 0;)
 		if (filename[i] == '.') return &filename[i + 1];
 
 	return NULL;
@@ -124,54 +127,46 @@ char *exists_prefs_dir(void) {
    name is cached internally, so it needs not to be free()ed. If the directory
    does not exist, it is not created. NULL is returned on failure. */
 
-char *exists_gprefs_dir(void)
-  {
-	  static char *gprefs_dir = NULL;
-	  struct stat s;
-     const char *global_dir;
+char *exists_gprefs_dir(void) {
+	static char *gprefs_dir = NULL;
+	struct stat s;
+	const char *global_dir;
 
-    /* If we have been already called, we already computed the name. */
-    /* We should free up the name and re-compute it (because the global */
-    /* dir may have changed). */
+	/* If we have been already called, we already computed the name. We
+		should free up the name and re-compute it (because the global dir may
+		have changed). */
     
-     if (gprefs_dir)
-         {
-           free(gprefs_dir);
-           gprefs_dir = NULL;
-         }
+	if (gprefs_dir) {
+		free(gprefs_dir);
+		gprefs_dir = NULL;
+	}
 
-    if ( (global_dir = get_global_dir()                ) &&
-         (gprefs_dir = malloc(strlen(global_dir) + 3 ) )    )
-        {
-          strcpy( gprefs_dir, global_dir );
-          #ifdef _AMIGA
-            if (stat(gprefs_dir, &s))
-                {
-                  free(gprefs_dir);
-                  return gprefs_dir = NULL;
-                }
-            else if (!(s.st_mode & _S_IFDIR))
-                {
-                  free(gprefs_dir);
-                  return gprefs_dir = NULL;
-                }
-            return strcat(gprefs_dir,"/");
-          #else
-            if (stat(gprefs_dir, &s))
-                {
-                  free(gprefs_dir);
-                  return gprefs_dir = NULL;
-                }
-            else if (!S_ISDIR(s.st_mode))
-                {
-                  free(gprefs_dir);
-                  return gprefs_dir = NULL;
-                }
-            return strcat(gprefs_dir, "/");
-          #endif
-        }
-    return NULL;
-  }
+	if ((global_dir = get_global_dir()) && (gprefs_dir = malloc(strlen(global_dir) + 3 ))) {
+		strcpy(gprefs_dir, global_dir);
+#ifdef _AMIGA
+		if (stat(gprefs_dir, &s)) {
+			free(gprefs_dir);
+			return gprefs_dir = NULL;
+		}
+		else if (!(s.st_mode & _S_IFDIR)) {
+			free(gprefs_dir);
+			return gprefs_dir = NULL;
+		}
+		return strcat(gprefs_dir,"/");
+#else
+		if (stat(gprefs_dir, &s)) {
+			free(gprefs_dir);
+			return gprefs_dir = NULL;
+		}
+		else if (!S_ISDIR(s.st_mode)) {
+			free(gprefs_dir);
+			return gprefs_dir = NULL;
+		}
+		return strcat(gprefs_dir, "/");
+#endif
+	}
+	return NULL;
+}
 
 
 /* Saves the preferences of the given buffer onto the given file name. If b or
@@ -189,6 +184,8 @@ int save_prefs(buffer * const b, const char * const name) {
 	if (cs = alloc_char_stream(PREF_FILE_SIZE_GUESS)) {
 		/* We create a macro by recording an action for each kind of flag. */
 
+		if (b->syn) record_action(cs, SYNTAX_A, -1, b->syn->name, b->opt.verbose_macros);
+
 		record_action(cs, FREEFORM_A,      b->opt.free_form,      NULL, b->opt.verbose_macros);
 		record_action(cs, STATUSBAR_A,     b->opt.status_bar,     NULL, b->opt.verbose_macros);
 		record_action(cs, HEXCODE_A,       b->opt.hex_code,       NULL, b->opt.verbose_macros);
@@ -206,6 +203,7 @@ int save_prefs(buffer * const b, const char * const name) {
 		record_action(cs, RIGHTMARGIN_A,   b->opt.right_margin,   NULL, b->opt.verbose_macros);
 		record_action(cs, CLIPNUMBER_A,    b->opt.cur_clip,       NULL, b->opt.verbose_macros);
 		record_action(cs, PRESERVECR_A,    b->opt.preserve_cr,    NULL, b->opt.verbose_macros);
+		record_action(cs, VISUALBELL_A,    b->opt.visual_bell,    NULL, b->opt.verbose_macros);
 
 		error = save_stream(cs, name, b->is_CRLF, FALSE);
 
@@ -244,6 +242,15 @@ int load_prefs(buffer * const b, const char * const name) {
 	return error;
 }
 
+/* Loads the given syntax. */
+
+int load_syntax_by_name(buffer * const b, const char * const name) {
+	assert_buffer(b);
+	assert(name != NULL);
+	b->syn = load_syntax((char *)name);
+	if (!b->syn) b->syn = load_syntax((char *)ext2syntax(name));
+	return b->syn ? OK : NO_SYNTAX_FOR_EXT;
+}
 
 
 /* Performs an automatic preferences operation, which can be loading or saving,
@@ -266,6 +273,7 @@ static int do_auto_prefs(buffer *b, const char * ext, int (prefs_func)(buffer *,
 	/* Try global autoprefs -- We always load these before ~/.ne autoprefs.
 	That way the user can override whatever he wants, but anything he
 	doesn't override still gets passed through. */
+
 	if (*prefs_func == load_prefs && (prefs_dir = exists_gprefs_dir())) {
 		if (auto_name = malloc(strlen(ext) + strlen(prefs_dir) + strlen(PREF_FILE_SUFFIX) + 2)) {
 			strcat(strcat(strcpy(auto_name, prefs_dir), ext), PREF_FILE_SUFFIX);
@@ -282,13 +290,14 @@ static int do_auto_prefs(buffer *b, const char * ext, int (prefs_func)(buffer *,
 			strcat(strcat(strcpy(auto_name, prefs_dir), ext), PREF_FILE_SUFFIX);
 			error = prefs_func(b, auto_name);
 			free(auto_name);
-			return error;
 		}
-		else
-			return OUT_OF_MEMORY;
+		else return OUT_OF_MEMORY;
 	}
+	else error = CANT_FIND_PREFS_DIR;
 
-	else return CANT_FIND_PREFS_DIR;
+	if (do_syntax && !b->syn) load_syntax_by_name(b, ext);
+
+	return error;
 }
 
 
@@ -297,13 +306,11 @@ load_prefs or save_prefs. */
 
 
 int load_auto_prefs(buffer * const b, const char *name) {
-
 	return do_auto_prefs(b, name, load_prefs);
 }
 
 
 int save_auto_prefs(buffer * const b, const char *name) {
-
 	return do_auto_prefs(b, name, save_prefs);
 }
 
@@ -315,45 +322,39 @@ int save_auto_prefs(buffer * const b, const char *name) {
 
 #define MAX_PREF_STACK_SIZE  32
 
-typedef struct
-  {
-    int        pcount;
-    int        psize;
-    options_t  pref[MAX_PREF_STACK_SIZE];
-  } pref_stack_t;
+typedef struct {
+	int pcount;
+	int psize;
+	options_t pref[MAX_PREF_STACK_SIZE];
+} pref_stack_t;
   
 static pref_stack_t pstack = { 0, MAX_PREF_STACK_SIZE };
 
-int push_prefs( buffer * const b )
-  {
-    char msg[120];
-    if ( pstack.pcount >= MAX_PREF_STACK_SIZE )
-        {
-          sprintf(msg,"PushPrefs failed, stack is full. %d prefs now on stack.", pstack.pcount );
-          print_message(msg);
-          return PREFS_STACK_FULL;
-        }
-    
-    pstack.pref[pstack.pcount++] = b->opt;
-    sprintf(msg,"User Prefs Pushed, %d Prefs now on stack.", pstack.pcount );
-    print_message(msg);
-    return OK;
-  }
+int push_prefs(buffer * const b) {
+	char msg[120];
+	if (pstack.pcount >= MAX_PREF_STACK_SIZE) {
+		sprintf(msg,"PushPrefs failed, stack is full. %d prefs now on stack.", pstack.pcount);
+		print_message(msg);
+		return PREFS_STACK_FULL;
+	}
+
+	pstack.pref[pstack.pcount++] = b->opt;
+	sprintf(msg,"User Prefs Pushed, %d Prefs now on stack.", pstack.pcount);
+	print_message(msg);
+	return OK;
+}
   
-int pop_prefs( buffer * const b )
-  {
-    char msg[120];
-    if ( pstack.pcount <= 0 )
-        {
-          sprintf(msg,"PopPrefs failed, stack is empty." );
-          print_message(msg);
-          return PREFS_STACK_EMPTY;
-        }
-      else
-        {
-          b->opt = pstack.pref[--pstack.pcount];
-          sprintf(msg,"User Prefs Popped, %d Prefs remain on stack.", pstack.pcount );
-          print_message(msg);
-          return OK;
-        }
-  }
+int pop_prefs(buffer * const b) {
+	char msg[120];
+	if (pstack.pcount <= 0) {
+		  sprintf(msg,"PopPrefs failed, stack is empty.");
+		  print_message(msg);
+		  return PREFS_STACK_EMPTY;
+	}
+	else {
+		b->opt = pstack.pref[--pstack.pcount];
+		sprintf(msg,"User Prefs Popped, %d Prefs remain on stack.", pstack.pcount);
+		print_message(msg);
+		return OK;
+	}
+}
