@@ -1,7 +1,7 @@
 /* Function prototypes
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -57,16 +57,18 @@ int insert_spaces(buffer *b, line_desc *ld, int line, int pos, int n);
 int delete_stream(buffer *b, line_desc *ld, int line, int pos, int len);
 int delete_one_char(buffer *b, line_desc *ld, int line, int pos);
 void change_filename(buffer *b, char *name);
+void ensure_attr_buf(buffer * const b, const int capacity);
 int load_file_in_buffer(buffer *b, const char *name);
 int load_fh_in_buffer(buffer *b, int fh);
 int save_buffer_to_file(buffer *b, const char *name);
 void auto_save(buffer *b);
+void reset_syntax_states(buffer *b);
 
 /* clips.c */
 clip_desc *alloc_clip_desc(int n, int size);
 clip_desc *realloc_clip_desc(clip_desc *cd, int n, int size);
 void free_clip_desc(clip_desc *cd);
-int is_encoding_neutral( clip_desc *cd);
+int is_encoding_neutral(clip_desc *cd);
 clip_desc *get_nth_clip(int n);
 int copy_to_clip(buffer *b, int n, int cut);
 int erase_block(buffer *b);
@@ -92,21 +94,25 @@ void help(char *p);
 int cmdcmp(const unsigned char *c, const unsigned char *m);
 void unload_macros(void);
 
-void optimize_macro( char_stream *cs, int verbose);
+void optimize_macro(char_stream *cs, int verbose);
 
 /* display.c */
+void update_syntax_states(buffer *b, int row, line_desc *ld, line_desc *end_ld);
+int highlight_cmp(HIGHLIGHT_STATE *x, HIGHLIGHT_STATE *y);
 void delay_update();
-void output_line_desc(int row, int col, const line_desc *ld, int start, int len, int tab_size, int cleared_at_end, int utf8);
-void update_partial_line(const buffer *b, int n, int start_x, int cleared_at_end);
-void update_line(const buffer *b, int n);
-void update_window_lines(const buffer *b, int start_line, int end_line, int doit);
-void update_window(const buffer *b);
-void update_deleted_char(const buffer *b, int c, const line_desc *ld, int pos, int line, int x);
-void update_inserted_char(const buffer *b, int c, const line_desc *ld, int pos, int line, int x);
-void update_overwritten_char(const buffer *b, int old_char, int new_char, const line_desc *ld, int pos, int line, int x);
+void output_line_desc(int row, int col, line_desc *ld, int start, int len, int tab_size, int cleared_at_end, int utf8, const int * const attr, const int * const diff, const int diff_size);
+line_desc *update_partial_line(buffer *b, int n, int start_x, int cleared_at_end, const int differential);
+void update_line(buffer *b, int n, const int differential);
+void update_window_lines(buffer *b, int start_line, int end_line, int doit);
+void update_syntax_and_lines(buffer *b, line_desc *start_ld, line_desc *end_ld);
+void update_window(buffer *b);
+void update_deleted_char(buffer *b, int c, const line_desc *ld, int pos, int attr_pos, int line, int x);
+void update_inserted_char(buffer *b, int c, const line_desc *ld, int pos, int attr_pos, int line, int x);
+void update_overwritten_char(buffer *b, int old_char, int new_char, const line_desc *ld, int pos, int attr_pos, int line, int x);
 void reset_window(void);
-void refresh_window(const buffer *b);
-void scroll_window(const buffer *b, int line, int n);
+void refresh_window(buffer *b);
+void scroll_window(buffer *b, int line, int n);
+HIGHLIGHT_STATE freeze_attributes(buffer *b, line_desc *ld);
 
 /* edit.c */
 int to_upper(buffer *b);
@@ -116,7 +122,7 @@ int match_bracket(buffer *b);
 int word_wrap(buffer *b);
 int paragraph(buffer *b);
 int center(buffer *b);
-int auto_indent_line(buffer *b);
+int auto_indent_line(buffer * const b, const int line, line_desc * const ld);
 
 /* errors.c */
 
@@ -129,13 +135,16 @@ void add(node *n, node *pos);
 void free_list(list *l, void (func)());
 void apply_to_list(list *l, void (func)());
 
+/* ext.c */
+const char *ext2syntax(const char * const ext);
+
 /* help.c */
 
 /* inputclass.c */
 
 /* keys.c */
 void read_key_capabilities(void);
-void set_escape_time( int new_escape_time);
+void set_escape_time(int new_escape_time);
 int get_key_code(void);
 
 /* menu.c */
@@ -147,6 +156,7 @@ char *gen_flag_string(const buffer *b);
 void draw_status_bar(void);
 int print_error(int error_num);
 void print_info(int info_num);
+void alert(void);
 void handle_menus(void);
 void get_menu_configuration(const char *);
 void get_key_bindings(const char *);
@@ -154,7 +164,7 @@ void get_key_bindings(const char *);
 /* names.c */
 
 /* navigation.c */
-int  adjust_view( buffer *b, const unsigned char *p);
+int  adjust_view(buffer *b, const unsigned char *p);
 int  char_left(buffer *b);
 int  char_right(buffer *b);
 int  line_down(buffer *b);
@@ -168,18 +178,19 @@ int  prev_page(buffer *b);
 void goto_column(buffer *b, int n);
 void goto_line(buffer *b, int n);
 void goto_pos(buffer *b, int pos);
-void keep_cursor_on_screen( buffer *b);
+void keep_cursor_on_screen(buffer *b);
 void move_to_bof(buffer *b);
 void move_to_eol(buffer *b);
 void move_to_sof(buffer *b);
 void move_to_sol(buffer *b);
 void reset_position_to_sof(buffer *b);
+void resync_pos(buffer * const b);
 void toggle_sof_eof(buffer *b);
 void toggle_sol_eol(buffer *b);
 int  search_word(buffer *b, int dir);
 void move_to_eow(buffer *b);
-void move_inc_down( buffer *b);
-void move_inc_up( buffer *b);
+void move_inc_down(buffer *b);
+void move_inc_up(buffer *b);
 
 /* ne.c */
 buffer *new_buffer(void);
@@ -189,13 +200,14 @@ int main(int argc, char **argv);
 /* prefs.c */
 const char *extension(const char *filename);
 char *exists_prefs_dir(void);
-char *exists_gprefs_dir( void);
+char *exists_gprefs_dir(void);
 int   save_prefs(buffer *b, const char *name);
 int   load_prefs(buffer *b, const char *name);
+int   load_syntax_by_name(buffer *b, const char *name);
 int   load_auto_prefs(buffer *b, const char *name);
 int   save_auto_prefs(buffer *b, const char *name);
-int   pop_prefs( buffer *b);
-int   push_prefs( buffer *b);
+int   pop_prefs(buffer *b);
+int   push_prefs(buffer *b);
 
 /* input.c */
 void  close_history(void);
@@ -206,7 +218,7 @@ char *request_string(const char *prompt, const char *default_string, int accept_
 char *request(const char *prompt, const char *default_string, int alpha_allowed, int completion_allowed, int prefer_utf8);
 
 /* request.c */
-int   request_strings(const char * const * const entries, int num_entries, int max_name_len);
+int   request_strings(const char * const * const entries, int num_entries, int max_name_len, int mark_dirs);
 char *request_files(const char *filename, int use_prefix);
 char *request_file(const buffer *b, const char *prompt, const char *default_name);
 int   request_document(void);
@@ -218,7 +230,7 @@ int  find_regexp(buffer *b, const unsigned char *regex, int dir, int skip_first)
 int  replace_regexp(buffer *b, const char *string);
 
 /* signals.c */
-void stop_ne( void);
+void stop_ne(void);
 void set_fatal_code(void);
 void block_signals(void);
 void release_signals(void);
@@ -243,7 +255,7 @@ int insert_in_stream(char_stream *cs, const char *s, int p, int len);
 
 /* support.c */
 char *ne_getcwd(const int bufsize);
-const char *get_global_dir( void);
+const char *get_global_dir(void);
 const char *tilde_expand(const char *filename);
 const char *file_part(const char *pathname);
 char *str_dup(const char *s);
@@ -253,23 +265,27 @@ int filenamecmpp(const void *a, const void *b);
 void set_interactive_mode(void);
 void unset_interactive_mode(void);
 int calc_width(const line_desc *ld, int n, int tab_size, encoding_type encoding);
+int calc_char_len(const line_desc *ld, encoding_type encoding);
 int calc_pos(const line_desc *ld, int n, int tab_size, encoding_type encoding);
+int get_string_width(const unsigned char * const s, const int len, const encoding_type encoding);
 int max_prefix(const char *s, const char *t);
 int is_prefix(const char *p, const char *s);
 char *complete(const char *start_prefix);
-int is_migrated( const char *name);
-int is_directory( const char *name);
-int is_ascii( const unsigned char *s, int len);
+int is_migrated(const char *name);
+int is_directory(const char *name);
+int is_ascii(const unsigned char *s, int len);
 int isasciispace(int c);
 int isasciipunct(int c);
 int isasciialpha(int c);
 int asciitoupper(int c);
 int asciitolower(int c);
 encoding_type detect_encoding(const unsigned char *s, int len);
-int next_pos( const unsigned char *s, int pos, encoding_type encoding);
-int prev_pos( const unsigned char *s, int pos, encoding_type encoding);
-int get_char( const unsigned char *s, encoding_type encoding);
+int next_pos(const unsigned char *s, int pos, encoding_type encoding);
+int prev_pos(const unsigned char *s, int pos, encoding_type encoding);
+int get_char(const unsigned char *s, encoding_type encoding);
 int get_char_width(const unsigned char * const s, const encoding_type encoding);
+int ne_ispunct(const int c, const int encoding);
+int ne_isspace(const int c, const int encoding);
 
 /* term.c */
 int output_width(int c);
@@ -287,12 +303,13 @@ void clear_end_of_line(int first_unused_hpos);
 void clear_to_eol(void);
 void clear_to_end(void);
 void clear_entire_screen(void);
-void output_chars(const unsigned char *string, int raw_len, int utf8);
+void set_attr(const unsigned int);
+void output_chars(const unsigned char *string, const unsigned int *attr, int raw_len, int utf8);
 void output_string(const char *s, int utf8);
-void output_spaces(int n);
-void output_char(int c, int utf8);
-void insert_chars(const unsigned char *start, int raw_len, int utf8);
-void insert_char(int c, int utf8);
+void output_spaces(int n, const unsigned int *attr);
+void output_char(int c, unsigned int attr, int utf8);
+void insert_chars(const unsigned char *start, const unsigned int *attr, int raw_len, int utf8);
+void insert_char(int c, const unsigned int attr, int utf8);
 void delete_chars(int n);
 void ins_del_lines(int vpos, int n);
 int ttysize(void);
@@ -306,4 +323,3 @@ int add_to_undo_stream(undo_buffer *ub, const char *p, int len);
 void reset_undo_buffer(undo_buffer *ub);
 int undo(buffer *b);
 int redo(buffer *b);
-
