@@ -1,7 +1,7 @@
 /* Requester handling.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -50,7 +50,7 @@ reasonable number of retries. */
 We rely on a series of auxiliary functions. */
 
 
-static int x, y, page, names_per_line, names_per_page, num_entries, max_name_len;
+static int x, y, page, names_per_line, names_per_page, num_entries, max_name_len, mark_dirs;
 
 static const char * const *entries;
 
@@ -59,9 +59,12 @@ static const char * const *entries;
 strings from the entries array existing in a certain page (a page contains
 (lines-1)*names_per_line items) with max_name_len maximum width. */
 
-static void print_strings(void) {
+static void print_strings() {
 
 	int i,j;
+	const char *p;
+
+	set_attr(0);
 
 	for(i = 0; i < ne_lines - 1; i++) {
 		move_cursor(i, 0);
@@ -70,7 +73,9 @@ static void print_strings(void) {
 		for(j = 0; j < names_per_line; j++) {
 			if ((i + page * (ne_lines - 1)) * names_per_line + j < num_entries) {
 				move_cursor(i, j * max_name_len);
-				output_string(entries[(i + page * (ne_lines - 1)) * names_per_line + j], io_utf8);
+				p = entries[(i + page * (ne_lines - 1)) * names_per_line + j];
+				if (mark_dirs) set_attr(p[strlen(p) - 1] == '/' ? BOLD : 0);
+				output_string(p, io_utf8);
 			}
 		}
 	}
@@ -94,11 +99,11 @@ static void request_move_to_sof(void) {
 	int i = page;
 
 	x = y = page = 0;
-	if (i != page) print_strings();
+	if (i != page) print_strings(FALSE);
 }
 
 
-static void request_move_to_eof(void) {
+static void request_move_to_eof() {
 
 	int i = page;
 
@@ -175,19 +180,20 @@ void request_move_inc_down(void) {
 }
 
 
-
-int request_strings(const char * const * const local_entries, const int local_num_entries, const int local_max_name_len) {
+/* If mark_dirs is TRUE, we bold names ending with a slash. */
+int request_strings(const char * const * const _entries, const int _num_entries, const int _max_name_len, int _mark_dirs) {
 
 	action a;
 	input_class ic;
 	int c, i, n;
 
-	assert(local_num_entries > 0);
+	assert(_num_entries > 0);
 
 	x = y = page = 0;
-	entries = local_entries;
-	num_entries = local_num_entries;
-	max_name_len = local_max_name_len;
+	entries = _entries;
+	num_entries = _num_entries;
+	max_name_len = _max_name_len;
+	mark_dirs = _mark_dirs;
 
 	if (!(names_per_line = ne_columns / (++max_name_len))) names_per_line = 1;
 	names_per_page = names_per_line * (ne_lines - 1);
@@ -462,13 +468,13 @@ char *request_files(const char * const filename, int use_prefix) {
 					if (num_entries) {
 						qsort(entries, num_entries, sizeof(char *), filenamecmpp);
 
-						if ((i = request_strings((const char * const *)entries, num_entries, max_name_len)) != ERROR) {
+						if ((i = request_strings((const char * const *)entries, num_entries, max_name_len, TRUE)) != ERROR) {
 							p = entries[i >= 0 ? i : -i - 2];
 							if (p[strlen(p) - 1] == '/' && i >= 0) {
 #ifndef _AMIGA
 								p[strlen(p) - 1] = 0;
 #endif
-								if (chdir(p)) ring_bell();
+								if (chdir(p)) alert();
 								else use_prefix = FALSE;
 								next_dir = TRUE;
 							}
@@ -486,7 +492,7 @@ char *request_files(const char * const filename, int use_prefix) {
 
 					closedir(d);
 				}
-				else ring_bell();
+				else alert();
 
 			} while(next_dir);
 
@@ -566,7 +572,7 @@ int request_document(void) {
 					b = (buffer *)b->b_node.next;
 				}
 
-				i = request_strings((const char * const *)entries, num_entries, max_name_len);
+				i = request_strings((const char * const *)entries, num_entries, max_name_len, FALSE);
 
 				reset_window();
 
