@@ -1,7 +1,7 @@
 /* Search/replace functions (with and without regular expressions).
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -395,7 +395,7 @@ int find_regexp(buffer * const b, const unsigned char *regex, int dir, int skip_
 				regex, we print it here. Which means that we access term.c's
 				functions. 8^( */
 			print_message(p);
-			ring_bell();
+			alert();
 			return ERROR;
 		}
 
