@@ -1,7 +1,7 @@
 /* Miscellaneous support functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -56,13 +56,13 @@ void __assert(int x, char *expr, char *file, int line) {
 const char *get_global_dir(void) {
 	char *ne_global_dir;
 	ne_global_dir = getenv("NE_GLOBAL_DIR");
-	if ( !ne_global_dir ) ne_global_dir = GLOBALDIR;
+	if (!ne_global_dir) ne_global_dir = GLOBALDIR;
 	return tilde_expand(ne_global_dir);
 }
 
 
 
-/* Some UNIXes allow "getcwd( NULL, size )" and will allocate the buffer for
+/* Some UNIXes allow "getcwd(NULL, size)" and will allocate the buffer for
 	you when your first parm is NULL.  This is not really legal, so we've put a
 	front end onto getcwd() called ne_getcwd() that allocates the buffer for you
 	first.
@@ -87,11 +87,10 @@ char *ne_getcwd(const int bufsize) {
 /* The Convex system uses CDVM, and can call cvxstat to see if a file is
   migrated. */
 #include <sys/dmon.h>
-int is_migrated( const char * const name ) {
+int is_migrated(const char * const name) {
 	struct cvxstat st;
-	if (cvxstat( name, &st, sizeof(struct cvxstat) ) == 0 )
-		if ( st.st_dmonflags & IMIGRATED ) 
-			return 1;
+	if (cvxstat(name, &st, sizeof(struct cvxstat)) == 0)
+		if (st.st_dmonflags & IMIGRATED) return 1;
 
 	return 0;
 }
@@ -100,12 +99,12 @@ int is_migrated( const char * const name ) {
   size but zero blocks used.  (Since the file is on tape rather than disc, it's
   using no disc blocks.)  If this describes the behaviour of your system,
   define ZERO_STAT_MIG_TEST when building ne. */
-int is_migrated( const char * const name ) {
+int is_migrated(const char * const name) {
 	struct stat statbuf;
 	
-	if ( (stat( tilde_expand(name), &statbuf ) == 0) &&
-		  (statbuf.st_size         > 0) &&
-		  (statbuf.st_blocks      == 0)    )
+	if ((stat(tilde_expand(name), &statbuf) == 0) &&
+		  (statbuf.st_size > 0) &&
+		  (statbuf.st_blocks == 0))
 		return 1;
 	else return 0;
   }
@@ -114,19 +113,16 @@ int is_migrated( const char * const name ) {
 /* Most systems have no hierarchical storage facility and need never concern
   themselves with this problem. For these systems, is_migrated() will always be
   false. */
-int is_migrated( const char * const name ) {
+int is_migrated(const char * const name) {
 	 return 0;
 }
 #endif  
 
 
-int is_directory( const char * const name ) {
+int is_directory(const char * const name) {
 	struct stat statbuf;
 	
-	if ( (stat( tilde_expand(name), &statbuf ) == 0) &&
-		  (S_ISDIR(statbuf.st_mode)   )    )
-		return 1;
-	else return 0;
+	return stat(tilde_expand(name), &statbuf) == 0 && S_ISDIR(statbuf.st_mode);
 }
 
 
@@ -274,7 +270,7 @@ int strcmpp(const void *a, const void *b) {
    and then orders lexicographically. */
 
 int filenamecmpp(const void *a, const void *b) {
-	const char *s = *(const char **)a, *t = *(const char **)b; 
+	const char * const s = *(const char **)a, * const t = *(const char **)b; 
 	if (strcmp(s, "../")==0) return strcmp(t, "../") == 0 ? 0 : -1;
 	if (strcmp(t, "../")==0) return 1;
 	if (strcmp(s, "./")==0) return strcmp(t, "./") == 0 ? 0 : -1;
@@ -431,17 +427,25 @@ void unset_interactive_mode(void) {
 
 int calc_width(const line_desc * const ld, const int n, const int tab_size, const encoding_type encoding) {
 
-	int i, len;
+	int pos, len;
 
-	for(i = len = 0; i < n; i = next_pos(ld->line, i, encoding)) {
-		if (i >= ld->line_len) len++;
-		else if (ld->line[i] != '\t') len += get_char_width(&ld->line[i], encoding);
+	for(pos = len = 0; pos < n; pos = next_pos(ld->line, pos, encoding)) {
+		if (pos >= ld->line_len) len++;
+		else if (ld->line[pos] != '\t') len += get_char_width(&ld->line[pos], encoding);
 		else len += tab_size - len % tab_size;
 	}
 
 	return len;
 }
 
+/* Computes character length of a line descriptor. */
+
+int calc_char_len(const line_desc * const ld, const encoding_type encoding) {
+	int pos, len;
+	for(pos = len = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, encoding), len++);
+	return len;
+}
+
 
 /* Given a column, the index of the character "containing" that position is
 	given, that is, calc_width(index) > n, and index is minimum with this
@@ -575,7 +579,7 @@ int get_string_width(const unsigned char * const s, const int len, const encodin
 	return width;
 }
 
-/* Returns whether the given character is a punctutation character. This function is 
+/* Returns whether the given character is a punctuation character. This function is 
 	compiled differently depending on whether wide-character function support is inhibited. */
 
 int ne_ispunct(const int c, const int encoding) {
