@@ -27,11 +27,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#ifndef _AMIGA
-#include <termios.h>
-#include <sys/ioctl.h>
-#endif
-
 #ifndef TERMCAP
 #include <curses.h>
 #include <term.h>
@@ -39,6 +34,11 @@
 #include "info2cap.h"
 #endif
 
+#ifndef _AMIGA
+#include <termios.h>
+#include <sys/ioctl.h>
+#endif
+
 #include "termchar.h"
 #include "cm.h"
 #include "utf8.h"
@@ -61,6 +61,26 @@ int ansi = TRUE;
 int ansi = FALSE;
 #endif
 
+/* Value is non-zero if attribute ATTR may be used with color.  ATTR
+   should be one of the enumerators from enum no_color_bit, or a bit set
+   built from them. */
+
+#define MAY_USE_WITH_COLORS(ATTR) (color_ok && ! (ne_no_color_video & (ATTR)))
+
+/* The mask values for no_color_video. */
+
+enum no_color_bit {
+	NC_STANDOUT    = 1 << 0,
+	NC_UNDERLINE   = 1 << 1,
+	NC_REVERSE     = 1 << 2,
+	NC_BLINK       = 1 << 3,
+	NC_DIM         = 1 << 4,
+	NC_BOLD        = 1 << 5,
+	NC_INVIS       = 1 << 6,
+	NC_PROTECT     = 1 << 7,
+	NC_ALT_CHARSET = 1 << 8
+};
+
 /* We use internal copies of the terminfo capabilities because we want to be
 	able to use a hardwired set. */
 
@@ -68,6 +88,7 @@ int ne_generic_type;
 
 int ne_lines;
 int ne_columns;
+int ne_no_color_video;
 
 char *ne_column_address;
 char *ne_row_address;
@@ -102,6 +123,7 @@ char *ne_enter_insert_mode;
 char *ne_exit_insert_mode;
 
 char *ne_enter_standout_mode;
+char *ne_enter_bold_mode;
 char *ne_exit_standout_mode;
 int ne_magic_cookie_glitch;
 int ne_move_standout_mode;
@@ -151,6 +173,18 @@ char *ne_keypad_xmit;
 char *ne_clr_eol;
 int ne_transparent_underline;
 
+char *ne_set_background;
+char *ne_set_foreground;
+
+char *ne_enter_underline_mode;
+char *ne_exit_underline_mode;
+
+char *ne_enter_bold_mode;
+char *ne_enter_blink_mode;
+char *ne_enter_dim_mode;
+char *ne_enter_reverse_mode;
+char *ne_exit_attribute_mode;
+
 
 
 
@@ -179,6 +213,8 @@ int	char_ins_del_ok;		/* Terminal can insert and delete chars */
 int	scroll_region_ok;		/* Terminal supports setting the scroll window */
 int	standout_ok;			/* Terminal supports standout without magic cookies */
 int	cursor_on_off_ok;		/* Terminal can make the cursor visible or invisible */
+int	ansi_color_ok;			/* Terminal supports ANSI color */
+int	color_ok;				/* Terminal supports color */
 
 
 static int	RPov;		/* Least number of chars to start a TS_repeat.
@@ -186,11 +222,12 @@ static int	RPov;		/* Least number of chars to start a TS_repeat.
 
 static int	delete_in_insert_mode;	/* delete mode == insert mode */
 static int	se_is_so;					/* 1 if same string both enters and leaves standout mode */
+static int	esm_is_eam;					/* 1 if exiting standout mode turns off all attributes */
 
-static int	standout_requested;	/* Nonzero when supposed to write text in standout mode. */
 static int	insert_mode;			/* Nonzero when in insert mode. */
 static int	standout_mode;			/* Nonzero when in standout mode. */
-
+static int	standout_wanted;		/* Nonzero if we should be writing in standout mode. */
+static int	curr_attr;				/* The current video attributes. */
 
 /* Size of window specified by higher levels. This is the number of lines,
 starting from top of screen, to participate in ins/del line operations.
@@ -203,29 +240,9 @@ int	specified_window;
 
 int	io_utf8;
 
-
-/* PORTABILITY PROBLEM: this function is responsible for filtering nonprintable
-	characters. On systems with a wider system character set, it could be
-	redefined, for instance, in order to allow characters between 128 and 160 to
-	be printed. Currently, it returns '?' on all control characters (and
-	non-ISO-8859-1 characters, if io_utf8 is false), space on 160, and the
-	obvious capital letter for control characters below 32. */
-
-int de_control(const int c) {
-	if (c >= 127 && c < 160) return '?';
-	if (c == 160) return ' ';
-	if (c < ' ') return '@'+c;
-	if (c > 0xFF && !io_utf8) return '?';
-
-	/* If io_utf8 is off, we consider all characters in the range of ISO-8859-x
-	encoding schemes as printable. */
-
-	return io_utf8 && wcwidth(c) <= 0 ? '?' : c;
-}
-
 /* Returns the output width of the given character. It is maximised with 1
  w.r.t. wcwidth(), so its result is equivalent to the width of the character
- returned by de_control(). */
+ that will be output by out(). */
 
 int output_width(const int c) {
 	const int width = wcwidth(c);
@@ -267,41 +284,203 @@ static int string_output_width(const unsigned char *s, int *len, int maxWidth, i
 	}
 }
 
-/* Depending on the value of io_utf8, this function will do a simple putchar(),
-	or a series of putchar() that expand the given character in UTF-8 encoding. */
 
-void out(const int c) {
+static int joe2color(const int joe_color) {
+	if (ansi_color_ok) return joe_color & 7;
+
+	switch(joe_color & 7) {
+		case 0: return 0; /* BLACK */
+		case 1: return 4; /* RED */
+		case 2: return 2; /* GREEN	*/
+		case 3: return 6; /* YELLOW */
+		case 4: return 1; /* BLUE */
+		case 5: return 5; /* MAGENTA */
+		case 6: return 3; /* CYAN */
+		case 7: return 7; /* WHITE */
+	}
+
+	return -1;
+}
+
+
+/* Sets up attributes */
+
+
+#ifdef PLAIN_SET_ATTR
+
+void set_attr(const unsigned int attr) {
+	char *buf;
+	OUTPUT1(ne_exit_attribute_mode);
+
+	if (attr & INVERSE) OUTPUT1(ne_enter_reverse_mode); 
+	if (attr & BOLD) OUTPUT1(ne_enter_bold_mode);
+	if (attr & UNDERLINE) OUTPUT1(ne_enter_underline_mode);
+	if (attr & DIM) OUTPUT1(ne_enter_dim_mode);
+	if (attr & BLINK) OUTPUT1(ne_enter_blink_mode);
+
+	if (color_ok) {
+		if (attr & FG_NOT_DEFAULT) {
+			buf = tparm(ne_set_foreground , joe2color(attr >> FG_SHIFT));
+			OUTPUT1(buf);
+		}
+		if (attr & BG_NOT_DEFAULT) {
+			buf = tparm(ne_set_background , joe2color(attr >> BG_SHIFT));
+			OUTPUT1(buf);
+		}
+	}
+} 
+
+#else
+
+
+void set_attr(const unsigned int attr) {
+	int attr_reset = FALSE;
+	char *buf;
+
+	/* If we have to set a different subset of attributes, or if we have to
+		set to the default at least one of the colors
+		(background/foreground) we must necessarily reset all attributes. */
+	if ((curr_attr & AT_MASK) != (attr & AT_MASK) 
+			|| (!(attr & FG_NOT_DEFAULT) && (curr_attr & FG_NOT_DEFAULT)) 
+			|| (!(attr & BG_NOT_DEFAULT) && (curr_attr & BG_NOT_DEFAULT))) {
+
+		OUTPUT1_IF(ne_exit_attribute_mode)
+		attr_reset = TRUE;
+
+		if ((attr & INVERSE) && MAY_USE_WITH_COLORS(NC_REVERSE)) OUTPUT1_IF(ne_enter_reverse_mode)
+		if ((attr & BOLD) && MAY_USE_WITH_COLORS(NC_BOLD)) OUTPUT1_IF(ne_enter_bold_mode)
+		if ((attr & UNDERLINE) && MAY_USE_WITH_COLORS(NC_UNDERLINE)) OUTPUT1_IF(ne_enter_underline_mode)
+		if ((attr & DIM) && MAY_USE_WITH_COLORS(NC_DIM)) OUTPUT1_IF(ne_enter_dim_mode)
+		if ((attr & BLINK) && MAY_USE_WITH_COLORS(NC_BLINK)) OUTPUT1_IF(ne_enter_blink_mode)
+	}
+
+	if (color_ok) {
+		/* Colors must be set if attributes have been reset and the required
+			color is not default, or in any case if the color has changed. */
+		
+		if (attr_reset && (attr & FG_NOT_DEFAULT) || (attr & FG_MASK) != (curr_attr & FG_MASK)) {
+			if (attr & FG_NOT_DEFAULT) {
+				buf = tparm(ne_set_foreground , joe2color(attr >> FG_SHIFT));
+				OUTPUT1(buf);
+			}
+		}
+		if (attr_reset && (attr & BG_NOT_DEFAULT) || (attr & BG_MASK) != (curr_attr & BG_MASK)) {
+			if (attr & BG_NOT_DEFAULT) {
+				buf = tparm(ne_set_background , joe2color(attr >> BG_SHIFT));
+				OUTPUT1(buf);
+			}
+		}
+	}
+
+	curr_attr = attr;
+}
+
+#endif
+
+static void turn_off_standout(void) {
+	OUTPUT1(ne_exit_standout_mode);
+	/* We exiting standout mode deletes all attributes, we update curr_attr. */
+	if (esm_is_eam) curr_attr = 0;
+	standout_mode = FALSE;
+}
+
+static void standout_if_wanted(void) {
+	if (standout_mode != standout_wanted) {
+		if (standout_wanted) {
+			OUTPUT1(ne_enter_standout_mode);
+			standout_mode = TRUE;
+		}
+		else turn_off_standout();
+	}
+}
+
+/* These functions are called on all terminals in order to handle highlighting,
+	but do nothing on terminals with a magic cookie (or without standout).  */
+ 
+void standout_on (void) {
+	if (standout_ok) standout_wanted = TRUE;
+}
+
+void standout_off (void) {
+	standout_wanted = FALSE;
+}
+       
+
+/* Depending on the value of io_utf8, this function will do a simple putchar(),
+	or a series of putchar() that expand the given character in UTF-8 encoding. 
+	If attr is -1, no attribute will be set. */
+
+static void out(int c, const int attr) {
+	int add_attr = 0;
+	
+	/* PORTABILITY PROBLEM: this code is responsible for filtering nonprintable
+		characters. On systems with a wider system character set, it could be
+		redefined, for instance, in order to allow characters between 128 and 160 to
+		be printed. Currently, it returns '?' on all control characters (and
+		non-ISO-8859-1 characters, if io_utf8 is false), space on 160, and the
+		obvious capital letter for control characters below 32. */
+
+	if (c >= 127 && c < 160) {
+		c = '?';
+		add_attr = INVERSE;
+	}
+	
+	if (c == 160) {
+		c = ' ';
+		add_attr = INVERSE;
+	}
+	
+	if (c < ' ') {
+		c += '@';
+		add_attr = INVERSE;
+	}
+	
+	if (c > 0xFF && !io_utf8) {
+		c = '?';
+		add_attr = INVERSE;
+	}
+
+	/* If io_utf8 is off, we consider all characters in the range of ISO-8859-x
+	encoding schemes as printable. */
+
+	if (io_utf8 && wcwidth(c) <= 0) {
+		c = '?';
+		add_attr = INVERSE;
+	}
+
+	if (attr != -1) set_attr(attr | add_attr);
+
 	if (io_utf8) {
-		if (c < 0x80) putchar( c ); /* ASCII */
+		if (c < 0x80) putchar(c); /* ASCII */
 		else if (c < 0x800) {
 			putchar(0xC0 | (c >> 6));
 			putchar(0x80 | (c >> 0) & 0x3F);
 		}
 		else if (c < 0x10000) {
-			putchar( 0xE0 | (c >> 12));
-			putchar( 0x80 | (c >> 6) & 0x3F );
-			putchar( 0x80 | (c >> 0) & 0x3F );
+			putchar(0xE0 | (c >> 12));
+			putchar(0x80 | (c >> 6) & 0x3F);
+			putchar(0x80 | (c >> 0) & 0x3F);
 		}
 		else if (c < 0x200000) {
-			putchar( 0xF0 | (c >> 18));
-			putchar( 0x80 | (c >> 12) & 0x3F );
-			putchar( 0x80 | (c >> 6) & 0x3F );
-			putchar( 0x80 | (c >> 0) & 0x3F );
+			putchar(0xF0 | (c >> 18));
+			putchar(0x80 | (c >> 12) & 0x3F);
+			putchar(0x80 | (c >> 6) & 0x3F);
+			putchar(0x80 | (c >> 0) & 0x3F);
 		}
 		else if (c < 0x4000000) {
-			putchar( 0xF8 | (c >> 24));
-			putchar( 0x80 | (c >> 18) & 0x3F );
-			putchar( 0x80 | (c >> 12) & 0x3F );
-			putchar( 0x80 | (c >> 6) & 0x3F );
-			putchar( 0x80 | (c >> 0) & 0x3F );
+			putchar(0xF8 | (c >> 24));
+			putchar(0x80 | (c >> 18) & 0x3F);
+			putchar(0x80 | (c >> 12) & 0x3F);
+			putchar(0x80 | (c >> 6) & 0x3F);
+			putchar(0x80 | (c >> 0) & 0x3F);
 		}
 		else {
-			putchar( 0xFC | (c >> 30));
-			putchar( 0x80 | (c >> 24) & 0x3F );
-			putchar( 0x80 | (c >> 18) & 0x3F );
-			putchar( 0x80 | (c >> 12) & 0x3F );
-			putchar( 0x80 | (c >> 6) & 0x3F );
-			putchar( 0x80 | (c >> 0) & 0x3F );
+			putchar(0xFC | (c >> 30));
+			putchar(0x80 | (c >> 24) & 0x3F);
+			putchar(0x80 | (c >> 18) & 0x3F);
+			putchar(0x80 | (c >> 12) & 0x3F);
+			putchar(0x80 | (c >> 6) & 0x3F);
+			putchar(0x80 | (c >> 0) & 0x3F);
 		}
 	}
 	else putchar(c);
@@ -357,24 +536,6 @@ static void turn_off_insert (void) {
 }
 
 
-/* These functions are called on all terminals in order to handle highlighting,
-but do nothing on terminals with a magic cookie (or without standout).  */
-
-static void turn_off_highlight (void) {
-	if (standout_ok) {
-		if (standout_mode) OUTPUT1(ne_exit_standout_mode);
-		standout_mode = FALSE;
-	}
-}
-
-
-static void turn_on_highlight (void) {
-	if (standout_ok) {
-		if (!standout_mode) OUTPUT1(ne_enter_standout_mode);
-		standout_mode = TRUE;
-	}
-}
-
 
 /* Prepares the terminal for interactive I/O. It
 initializes the terminal, prepares the cursor address mode, and
@@ -386,10 +547,6 @@ void set_terminal_modes(void) {
 	and the file which should be used, if present, to initialize the
 	terminal. */
 
-	/*OUTPUT1_IF(ne_init_1string);
-	OUTPUT1_IF(ne_init_2string);
-	OUTPUT1_IF(ne_init_3string);*/
-
 	OUTPUT1_IF(ne_exit_attribute_mode);
 	OUTPUT1_IF(ne_exit_alt_charset_mode);
 	OUTPUT1_IF(ne_exit_standout_mode);
@@ -425,21 +582,6 @@ void set_terminal_window(const int size) {
 }
 
 
-
-
-/* These functions are the external interface to standout activation. */
-
-void standout_on (void) {
-	standout_requested = TRUE;
-}
-
-
-void standout_off (void) {
-	standout_requested = FALSE;
-	turn_off_highlight ();
-}
-
-
 /* These functions are the external interface to cursor on/off strings. */
 
 void cursor_on (void) {
@@ -452,19 +594,11 @@ void cursor_off (void) {
 }
 
 
-/* Set standout mode to the mode specified for the text to be output.  */
-
-static void highlight_if_desired (void) {
-	if (standout_requested) turn_on_highlight ();
-	else turn_off_highlight ();
-}
-
-
 /* Move to absolute position, specified origin 0 */
 
 void move_cursor (const int row, const int col) {
 	if (curY == row && curX == col) return;
-	if (!ne_move_standout_mode) turn_off_highlight ();
+	if (!ne_move_standout_mode) turn_off_standout();
 	if (!ne_move_insert_mode) turn_off_insert ();
 	cmgoto (row, col);
 }
@@ -479,6 +613,7 @@ void clear_end_of_line(const int first_unused_hpos) {
 	int i;
 
 	if (curX >= first_unused_hpos) return;
+	standout_if_wanted();
 
 	if (ne_clr_eol) OUTPUT1 (ne_clr_eol);
 	else {
@@ -529,85 +664,59 @@ void clear_entire_screen (void) {
 }
 
 
-/* Outputs len characters pointed at by string. The characters will be
-	truncated to the end of the current line. Passing a NULL for string results
-	in outputting spaces. A len of 0 causes no action. If utf8 is TRUE, the
-	string is UTF-8 encoded. */
+/* Outputs len characters pointed at by string, attributed as indicated by
+   a corresponding vector of attributes, which can be NULL, in which case
+   no attribute will be set. The characters will be truncated to the end
+   of the current line. Passing a NULL for string results in outputting
+   spaces. A len of 0 causes no action. If utf8 is TRUE, the string is
+   UTF-8 encoded. */
+void output_chars(const unsigned char *string, const unsigned int *attr, const int raw_len, const int utf8) {
 
-void output_chars (const unsigned char *string, const int raw_len, const int utf8) {
-
-	const unsigned char *p;
-	unsigned char *buf;
-	int	c, len;
+	int i, c, len;
 	const unsigned char *first_check;
 
 	if (raw_len == 0) return;
 
-	highlight_if_desired ();
-	turn_off_insert ();
+	turn_off_insert();
+	standout_if_wanted();
 
 	/* If the string is UTF-8 encoded, compute its real length. */
 	if (utf8 && string != NULL) len = utf8strlen(string, raw_len);
 	else len = raw_len;
 
 	/* If the width of the string exceeds the remaining columns, we reduce
-		len. Moreovero, we don't dare write in last column of bottom line, if
+		len. Moreover, we don't dare write in last column of bottom line, if
 		AutoWrap, since that would scroll the whole screen on some terminals. */
 
 	cmplus(string_output_width(string, &len, ne_columns - curX - (AutoWrap && curY == ne_lines - 1), utf8));
 
 	if (string == NULL) {
-		if (len > RPov) {
-			buf = tparm (ne_repeat_char, ' ', len);
-			OUTPUT1(buf);
-		}
-		else {
-			while(--len >= 0) putchar(' ');
+		for(i = 0; i < len; i++) {
+			/* When outputting spaces, it's only the first attribute that's used. */
+			if (attr) set_attr(*attr);
+			putchar(' ');
 		}
 		return;
 	}
 
 	first_check = string;
 
-	if (RPov > len && !ne_transparent_underline && !ne_tilde_glitch) {
-		while(--len >= 0) {
+	if (!ne_transparent_underline && !ne_tilde_glitch) {
+		for(i = 0; i < len; i++) {
 			if (utf8) {
 				c = utf8char(string);
 				string += utf8len(*string);
-				out(de_control(c));
+				out(c, attr ? attr[i] : -1);
 			}
 			else {
 				c = *string++;
-				out(de_control(c));
+				out(c, attr ? attr[i] : -1);
 			}
 		}
 	} else
-		while (--len >= 0) {
+		for(i = 0; i < len; i++) {
+			if (attr) set_attr(attr[i]);
 			c = utf8 ? *string : utf8char(string);
-			if (!io_utf8 && RPov + 1 < len && string >= first_check) {
-				int	repeat_count;
-
-				p = string;
-
-				/* Now, len is number of chars left starting at p */
-				while (++p - string <= len && *p == c);
-
-				repeat_count = p - string;
-
-				if (repeat_count > RPov) {
-					buf = tparm (ne_repeat_char, de_control(c), repeat_count);
-					OUTPUT1(buf);
-					string = p;
-					len -= repeat_count - 1;
-					continue;
-				}
-				else {
-					/* If all N identical chars are too few,
-		 			don't even consider the last N-1, the last N-2,...  */
-
-					first_check = p;
-				}
-			}
 
 			if (c == '_' && ne_transparent_underline) {
 				putchar (' ');
@@ -617,24 +726,25 @@ void output_chars (const unsigned char *string, const int raw_len, const int utf
 			if (ne_tilde_glitch && c == '~')
 				c = '`';
 
-			out(de_control(c));
+			out(c, attr ? attr[i] : -1);
 			string += utf8 ? utf8len(*string) : 1;
 		}
 }
 
 
 
-/* Outputs a NULL-terminated string. */
+/* Outputs a NULL-terminated string without setting attributes. */
 
 void output_string(const char * const s, const int utf8) {
 	assert(s != NULL);
-	output_chars(s, strlen(s), utf8);
+	output_chars(s, NULL, strlen(s), utf8);
 }
 
 
-/* Outputs a single ISO 10646 character. */
+/* Outputs a single ISO 10646 character with a given set of attributes. If
+	attr == -1, no attribute is set. */
 
-void output_char(const int c, const int utf8) {
+void output_char(const int c, const unsigned int attr, const int utf8) {
 	static unsigned char t[8];
 
 	if (utf8) {
@@ -648,26 +758,25 @@ void output_char(const int c, const int utf8) {
 
 	assert(c != 0);
 
-	output_string(t, utf8);
+	output_chars(t, attr != -1 ? &attr : NULL, strlen(t), utf8);
 }
 
 
 /* Outputs spaces. */
 
-void output_spaces(const int n) {
-   output_chars(NULL, n, FALSE);
+void output_spaces(const int n, const unsigned int *attr) {
+   output_chars(NULL, attr, n, FALSE);
 }
 
 /* Same as output_chars(), but inserts instead. */
 
-void insert_chars (const unsigned char *start, const int raw_len, const int utf8) {
+void insert_chars(const unsigned char * start, const unsigned int * const attr, const int raw_len, const int utf8) {
 
 	const unsigned char *buf;
-	int c, len, width;
+	int i, c, len;
 
 	if (raw_len == 0) return;
-
-	highlight_if_desired();
+	standout_if_wanted();
 
 	/* If the string is non-NULL and UTF-8 encoded, compute its real length. */
 	if (utf8 && start != NULL) len = utf8strlen(start, raw_len);
@@ -685,7 +794,7 @@ void insert_chars (const unsigned char *start, const int raw_len, const int utf8
 		buf = tparm (ne_parm_ich, width);
 		OUTPUT1 (buf);
 
-		if (start) output_chars (start, raw_len, utf8);
+		if (start) output_chars(start, attr, raw_len, utf8);
 
 		return;
 	}
@@ -693,29 +802,35 @@ void insert_chars (const unsigned char *start, const int raw_len, const int utf8
 	turn_on_insert ();
 
 	/* If the width of the string exceeds the remaining columns, we reduce
-		len. Moreovero, we don't dare write in last column of bottom line, if
-		AutoWrap, since that would scroll the whole screen on some terminals. */
+		len. Moreovero, we don't dare to write in the last column of the
+		bottom line, if AutoWrap, since that would scroll the whole screen
+		on some terminals. */
 
 	cmplus(string_output_width(start, &len, ne_columns - curX - (AutoWrap && curY == ne_lines - 1), utf8));
 
 	if (!ne_transparent_underline && !ne_tilde_glitch && start
 	     && ne_insert_padding == NULL && ne_insert_character == NULL) {
-		while(--len >= 0) {
+		for(i = 0; i < len; i++) {
+			if (attr) set_attr(attr[i]);
 			if (utf8) {
 				c = utf8char(start);
 				start += utf8len(*start);
 			}
 			else c = *start++;
-			out(de_control(c));
+			out(c, attr ? attr[i] : -1);
 		}
 	}
 	else
-		while (--len >= 0) {
+		for(i = 0; i < len; i++) {
 
 			OUTPUT1_IF (ne_insert_character);
 
-			if (!start) c = ' ';
+			if (!start) {
+				/* When outputting spaces, it's only the first attribute that's used. */
+				out(' ', attr ? *attr : -1);
+			}
 			else {
+				if (attr) set_attr(attr[i]);
 				if (utf8) {
 					c = utf8char(start);
 					start += utf8len(*start);
@@ -723,17 +838,18 @@ void insert_chars (const unsigned char *start, const int raw_len, const int utf8
 				else c = *start++;
 				
 				if (ne_tilde_glitch && c == '~') c = '`';
+
+				out(c, attr ? attr[i] : -1);
 			}
 
-			out(de_control(c));
 			OUTPUT1_IF(ne_insert_padding);
 		}
 }
 
 
-/* Inserts a single ISO 10646 character. */
+/* Inserts a single ISO 10646 character. If attr == -1, no attribute is set. */
 
-void insert_char(const int c, const int utf8) {
+void insert_char(const int c, const unsigned int attr, const int utf8) {
 	static unsigned char t[8];
 
 	if (utf8) {
@@ -747,7 +863,7 @@ void insert_char(const int c, const int utf8) {
 
 	assert(c != 0);
 
-	insert_chars(t, strlen(t), utf8);
+	insert_chars(t, attr == -1 ? NULL : &attr, strlen(t), utf8);
 }
 
 /* Deletes n characters at the current cursor position. */
@@ -756,7 +872,7 @@ void delete_chars (int n) {
 
 	char *buf;
 
-	if ( n == 0 ) return;
+	if (n == 0) return;
 
 	if (delete_in_insert_mode) turn_on_insert();
 	else {
@@ -804,6 +920,8 @@ void ins_del_lines (const int vpos, const int n) {
 
 	if (!ne_memory_below && vpos + i >= ne_lines) return;
 
+	standout_if_wanted();
+
 	if (scroll_region_ok) {
 		if (specified_window != ne_lines) set_scroll_region(vpos, specified_window - 1);
 
@@ -884,7 +1002,7 @@ int ttysize(void) {
 	
 	if (ioctl(0, TIOCGWINSZ, &size) >= 0) {
 		D(fprintf(stderr,"ttysize:...size is (%d,%d)\n",size.ws_row,size.ws_col);)
-		if ( (ne_lines != size.ws_row) || ( ne_columns != size.ws_col ) ) {
+		if ((ne_lines != size.ws_row) || (ne_columns != size.ws_col)) {
 			ScreenRows = ne_lines    = size.ws_row;
 			ScreenCols = ne_columns  = size.ws_col;
 			set_terminal_window(ne_lines - 1);
@@ -910,6 +1028,7 @@ void copy_caps(void) {
 
 	ne_lines = lines;
 	ne_columns = columns;
+	ne_no_color_video = no_color_video == -1 ? 0 : no_color_video;
 
 	ne_column_address = column_address;
 	ne_row_address = row_address;
@@ -992,6 +1111,24 @@ void copy_caps(void) {
 
 	ne_clr_eol = clr_eol;
 	ne_transparent_underline = transparent_underline;
+
+	if (ansi_color_ok = (set_a_foreground && set_a_background)) {
+		ne_set_background = set_a_background;
+		ne_set_foreground = set_a_foreground;
+	}
+	else {
+		ne_set_background = set_background;
+		ne_set_foreground = set_foreground;
+	}
+
+	ne_enter_underline_mode = enter_underline_mode;
+	ne_exit_underline_mode = exit_underline_mode;
+
+	ne_enter_bold_mode = enter_bold_mode;
+	ne_enter_blink_mode = enter_blink_mode;
+	ne_enter_dim_mode = enter_dim_mode;
+	ne_enter_reverse_mode = enter_reverse_mode;
+	ne_exit_attribute_mode = exit_attribute_mode;
 }
 
 
@@ -999,46 +1136,6 @@ void copy_caps(void) {
 
 
 
-void setup_ansi_term(void) {
-
-#ifdef TERMCAP
-	ne_cursor_address = "\x1b[%i%d;%dH";
-#else
-	ne_cursor_address = "\x1b[%i%p1%d;%p2%dH";
-#endif
-	ne_lines = 25;
-	ne_columns = 80;
-	ne_carriage_return = "\xd";
-	ne_cursor_home = "\x1b[H";
-	ne_cursor_right = "\x1b[C";
-	ne_cursor_down = "\x1b[B";
-	ne_cursor_left = "\x1b[D";
-	ne_cursor_up = "\x1b[A";
-	ne_auto_right_margin = 1;
-	ne_eat_newline_glitch = 0;
-	ne_clr_eos = "\x1b[J";
-	ne_clear_screen = "\x1b[H\x1b[J";
-	ne_bell = "\x7";
-	ne_scroll_forward = "\xa";
-	ne_enter_standout_mode = "\x1b[7m";
-	ne_exit_standout_mode = "\x1b[m";
-	ne_magic_cookie_glitch = -1;
-	ne_move_standout_mode = 0;
-	ne_insert_line = "\x1b[L";
-	ne_delete_line = "\x1b[M";
-	ne_delete_character = "\x1b[P";
-	ne_move_insert_mode = 1;
-	ne_exit_attribute_mode = "\x1b[0;10m";
-	ne_exit_alt_charset_mode = "\x1b[10m";
-	ne_tilde_glitch = 0;
-	ne_memory_below = 0;
-	ne_has_meta_key = 0;
-	ne_clr_eol = "\x1b[K";
-	ne_transparent_underline = 0;
-}
-
-
-
 /* This is the main terminal initialization function. It sets up Wcm,
 patches here and there the terminfo database, calculates the costs, and
 initializes the terminal characteristics variables. Note that this function
@@ -1094,7 +1191,7 @@ void term_init (void) {
 
 	specified_window = ne_lines;
 
-	if (Wcm_init ())	{
+	if (Wcm_init()) {
 
 		/* We can't do cursor motion */
 
@@ -1126,6 +1223,9 @@ void term_init (void) {
 	se_is_so = ne_enter_standout_mode && ne_exit_standout_mode
 	     && !strcmp (ne_enter_standout_mode, ne_exit_standout_mode);
 
+	esm_is_eam = ne_exit_standout_mode && ne_exit_attribute_mode
+	     && !strcmp (ne_exit_standout_mode, ne_exit_attribute_mode);
+
 	scroll_region_ok = ne_set_window || ne_change_scroll_region;
 
 	line_ins_del_ok = (((ne_insert_line || ne_parm_insert_line)
@@ -1141,4 +1241,6 @@ void term_init (void) {
 	standout_ok = (ne_enter_standout_mode && ne_exit_standout_mode && ne_magic_cookie_glitch < 0);
 
 	cursor_on_off_ok = (ne_cursor_invisible && ne_cursor_normal);
+
+	color_ok = (ne_set_foreground && ne_set_background);
 }
