@@ -1,7 +1,7 @@
 /* Undo/redo system management functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2006 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -31,7 +31,6 @@
 
 #define STD_UNDO_STREAM_SIZE	(16*1024)
 
-
 /* This is the main function for recording an undo step (though it should be
 	called through add_undo_step). It adds to the given undo buffer an undo step
 	with given line, position and length, possibly enlarging the undo step
@@ -179,16 +178,21 @@ int undo(buffer * const b) {
 		b->undo.cur_step--;
 
 		if (b->undo.steps[b->undo.cur_step].len) {
-
+			line_desc *cur_line_desc;
 			goto_line(b, b->undo.steps[b->undo.cur_step].line);
 			goto_pos(b, b->undo.steps[b->undo.cur_step].pos >= 0 ? b->undo.steps[b->undo.cur_step].pos : -(1 + b->undo.steps[b->undo.cur_step].pos));
+			cur_line_desc = b->cur_line_desc;
 
 			if (b->undo.steps[b->undo.cur_step].len < 0) {
 				delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, -b->undo.steps[b->undo.cur_step].len);
+				update_syntax_and_lines(b, b->cur_line_desc, NULL);
 			}
 			else {
+				line_desc *end_ld = (line_desc *)b->cur_line_desc->ld_node.next;
 				insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, b->undo.streams + (b->undo.cur_stream -= b->undo.steps[b->undo.cur_step].len), b->undo.steps[b->undo.cur_step].len);
+				update_syntax_and_lines(b, b->cur_line_desc, end_ld);
 			}
+
 		}
 
 	} while(b->undo.cur_step && b->undo.steps[b->undo.cur_step - 1].pos < 0);
@@ -217,15 +221,20 @@ int redo(buffer * const b) {
 
 	do {
 		if (b->undo.steps[b->undo.cur_step].len) {
-
+			line_desc *cur_line_desc;
 			goto_line(b, b->undo.steps[b->undo.cur_step].line);
 			goto_pos(b, b->undo.steps[b->undo.cur_step].pos >= 0 ? b->undo.steps[b->undo.cur_step].pos : -(1 + b->undo.steps[b->undo.cur_step].pos));
+			cur_line_desc = b->cur_line_desc;
 
-			if (b->undo.steps[b->undo.cur_step].len < 0)
+			if (b->undo.steps[b->undo.cur_step].len < 0) { 
+				line_desc *end_ld = (line_desc *)b->cur_line_desc->ld_node.next;
 				insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, b->undo.redo.stream + (b->undo.redo.len += b->undo.steps[b->undo.cur_step].len), -b->undo.steps[b->undo.cur_step].len);
+				update_syntax_and_lines(b, b->cur_line_desc, end_ld);
+			}	
 			else {
 				delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, b->undo.steps[b->undo.cur_step].len);
 				b->undo.cur_stream += b->undo.steps[b->undo.cur_step].len;
+				update_syntax_and_lines(b, b->cur_line_desc, NULL);
 			}
 		}
 
